@@ -28,6 +28,7 @@ from wsme import types as wtypes
 
 from ironic.api.controllers import base
 from ironic.api.controllers import link
+from ironic.api.controllers.v1 import bios
 from ironic.api.controllers.v1 import collection
 from ironic.api.controllers.v1 import notification_utils as notify
 from ironic.api.controllers.v1 import port
@@ -146,6 +147,9 @@ def hide_fields_in_newer_versions(obj):
     if pecan.request.version.minor < versions.MINOR_20_NETWORK_INTERFACE:
         obj.network_interface = wsme.Unset
 
+    if pecan.request.version.minor < versions.MINOR_42_FAULT:
+        obj.fault = wsme.Unset
+
     if not api_utils.allow_resource_class():
         obj.resource_class = wsme.Unset
 
@@ -162,6 +166,9 @@ def hide_fields_in_newer_versions(obj):
     if not api_utils.allow_rescue_interface():
         obj.rescue_interface = wsme.Unset
 
+    if not api_utils.allow_bios_interface():
+        obj.bios_interface = wsme.Unset
+
 
 def update_state_in_older_versions(obj):
     """Change provision state names for API backwards compatibility.
@@ -170,9 +177,13 @@ def update_state_in_older_versions(obj):
                 to be updated by this method.
     """
     # if requested version is < 1.2, convert AVAILABLE to the old NOSTATE
-    if (pecan.request.version.minor < versions.MINOR_2_AVAILABLE_STATE and
-            obj.provision_state == ir_states.AVAILABLE):
+    if (pecan.request.version.minor < versions.MINOR_2_AVAILABLE_STATE
+            and obj.provision_state == ir_states.AVAILABLE):
         obj.provision_state = ir_states.NOSTATE
+    # if requested version < 1.39, convert INSPECTWAIT to INSPECTING
+    if (not api_utils.allow_inspect_wait_state()
+            and obj.provision_state == ir_states.INSPECTWAIT):
+        obj.provision_state = ir_states.INSPECTING
 
 
 class BootDeviceController(rest.RestController):
@@ -518,8 +529,8 @@ class NodeStatesController(rest.RestController):
         rpc_node = api_utils.get_rpc_node(node_ident)
         topic = pecan.request.rpcapi.get_topic_for(rpc_node)
 
-        if ((target in [ir_states.SOFT_REBOOT, ir_states.SOFT_POWER_OFF] or
-             timeout) and not api_utils.allow_soft_power_off()):
+        if ((target in [ir_states.SOFT_REBOOT, ir_states.SOFT_POWER_OFF]
+             or timeout) and not api_utils.allow_soft_power_off()):
             raise exception.NotAcceptable()
         # FIXME(naohirot): This check is workaround because
         #                  wtypes.IntegerType(minimum=1) is not effective
@@ -547,6 +558,54 @@ class NodeStatesController(rest.RestController):
         url_args = '/'.join([node_ident, 'states'])
         pecan.response.location = link.build_url('nodes', url_args)
 
+    def _do_provision_action(self, rpc_node, target, configdrive=None,
+                             clean_steps=None, rescue_password=None):
+        topic = pecan.request.rpcapi.get_topic_for(rpc_node)
+        # Note that there is a race condition. The node state(s) could change
+        # by the time the RPC call is made and the TaskManager manager gets a
+        # lock.
+        if target in (ir_states.ACTIVE, ir_states.REBUILD):
+            rebuild = (target == ir_states.REBUILD)
+            pecan.request.rpcapi.do_node_deploy(context=pecan.request.context,
+                                                node_id=rpc_node.uuid,
+                                                rebuild=rebuild,
+                                                configdrive=configdrive,
+                                                topic=topic)
+        elif (target == ir_states.VERBS['unrescue']):
+            pecan.request.rpcapi.do_node_unrescue(
+                pecan.request.context, rpc_node.uuid, topic)
+        elif (target == ir_states.VERBS['rescue']):
+            if not (rescue_password and rescue_password.strip()):
+                msg = (_('A non-empty "rescue_password" is required when '
+                         'setting target provision state to %s') %
+                       ir_states.VERBS['rescue'])
+                raise wsme.exc.ClientSideError(
+                    msg, status_code=http_client.BAD_REQUEST)
+            pecan.request.rpcapi.do_node_rescue(
+                pecan.request.context, rpc_node.uuid, rescue_password, topic)
+        elif target == ir_states.DELETED:
+            pecan.request.rpcapi.do_node_tear_down(
+                pecan.request.context, rpc_node.uuid, topic)
+        elif target == ir_states.VERBS['inspect']:
+            pecan.request.rpcapi.inspect_hardware(
+                pecan.request.context, rpc_node.uuid, topic=topic)
+        elif target == ir_states.VERBS['clean']:
+            if not clean_steps:
+                msg = (_('"clean_steps" is required when setting target '
+                         'provision state to %s') % ir_states.VERBS['clean'])
+                raise wsme.exc.ClientSideError(
+                    msg, status_code=http_client.BAD_REQUEST)
+            _check_clean_steps(clean_steps)
+            pecan.request.rpcapi.do_node_clean(
+                pecan.request.context, rpc_node.uuid, clean_steps, topic)
+        elif target in PROVISION_ACTION_STATES:
+            pecan.request.rpcapi.do_provisioning_action(
+                pecan.request.context, rpc_node.uuid, target, topic)
+        else:
+            msg = (_('The requested action "%(action)s" could not be '
+                     'understood.') % {'action': target})
+            raise exception.InvalidStateRequested(message=msg)
+
     @METRICS.timer('NodeStatesController.provision')
     @expose.expose(None, types.uuid_or_name, wtypes.text,
                    wtypes.text, types.jsontype, wtypes.text,
@@ -606,7 +665,6 @@ class NodeStatesController(rest.RestController):
 
         api_utils.check_allow_management_verbs(target)
         rpc_node = api_utils.get_rpc_node(node_ident)
-        topic = pecan.request.rpcapi.get_topic_for(rpc_node)
 
         if (target in (ir_states.ACTIVE, ir_states.REBUILD)
                 and rpc_node.maintenance):
@@ -640,57 +698,20 @@ class NodeStatesController(rest.RestController):
             raise wsme.exc.ClientSideError(
                 msg, status_code=http_client.BAD_REQUEST)
 
-        if (rescue_password is not None and
-            target != ir_states.VERBS['rescue']):
+        if (rescue_password is not None
+            and target != ir_states.VERBS['rescue']):
             msg = (_('"rescue_password" is only valid when setting target '
                      'provision state to %s') % ir_states.VERBS['rescue'])
             raise wsme.exc.ClientSideError(
                 msg, status_code=http_client.BAD_REQUEST)
 
-        # Note that there is a race condition. The node state(s) could change
-        # by the time the RPC call is made and the TaskManager manager gets a
-        # lock.
-        if target in (ir_states.ACTIVE, ir_states.REBUILD):
-            rebuild = (target == ir_states.REBUILD)
-            pecan.request.rpcapi.do_node_deploy(context=pecan.request.context,
-                                                node_id=rpc_node.uuid,
-                                                rebuild=rebuild,
-                                                configdrive=configdrive,
-                                                topic=topic)
-        elif (target == ir_states.VERBS['unrescue']):
-            pecan.request.rpcapi.do_node_unrescue(
-                pecan.request.context, rpc_node.uuid, topic)
-        elif (target == ir_states.VERBS['rescue']):
-            if not (rescue_password and rescue_password.strip()):
-                msg = (_('A non-empty "rescue_password" is required when '
-                         'setting target provision state to %s') %
-                       ir_states.VERBS['rescue'])
-                raise wsme.exc.ClientSideError(
-                    msg, status_code=http_client.BAD_REQUEST)
-            pecan.request.rpcapi.do_node_rescue(
-                pecan.request.context, rpc_node.uuid, rescue_password, topic)
-        elif target == ir_states.DELETED:
-            pecan.request.rpcapi.do_node_tear_down(
-                pecan.request.context, rpc_node.uuid, topic)
-        elif target == ir_states.VERBS['inspect']:
-            pecan.request.rpcapi.inspect_hardware(
-                pecan.request.context, rpc_node.uuid, topic=topic)
-        elif target == ir_states.VERBS['clean']:
-            if not clean_steps:
-                msg = (_('"clean_steps" is required when setting target '
-                         'provision state to %s') % ir_states.VERBS['clean'])
-                raise wsme.exc.ClientSideError(
-                    msg, status_code=http_client.BAD_REQUEST)
-            _check_clean_steps(clean_steps)
-            pecan.request.rpcapi.do_node_clean(
-                pecan.request.context, rpc_node.uuid, clean_steps, topic)
-        elif target in PROVISION_ACTION_STATES:
-            pecan.request.rpcapi.do_provisioning_action(
-                pecan.request.context, rpc_node.uuid, target, topic)
-        else:
-            msg = (_('The requested action "%(action)s" could not be '
-                     'understood.') % {'action': target})
-            raise exception.InvalidStateRequested(message=msg)
+        if (rpc_node.provision_state == ir_states.INSPECTWAIT and
+            target == ir_states.VERBS['abort']):
+            if not api_utils.allow_inspect_abort():
+                raise exception.NotAcceptable()
+
+        self._do_provision_action(rpc_node, target, configdrive, clean_steps,
+                                  rescue_password)
 
         # Set the HTTP Location Header
         url_args = '/'.join([node_ident, 'states'])
@@ -861,8 +882,13 @@ class NodeTraitsController(rest.RestController):
         with notify.handle_error_notification(context, node, 'update',
                                               chassis_uuid=chassis_uuid):
             topic = pecan.request.rpcapi.get_topic_for(node)
-            pecan.request.rpcapi.remove_node_traits(
-                context, node.id, traits, topic=topic)
+            try:
+                pecan.request.rpcapi.remove_node_traits(
+                    context, node.id, traits, topic=topic)
+            except exception.NodeTraitNotFound:
+                # NOTE(hshiina): Internal node ID should not be exposed.
+                raise exception.NodeTraitNotFound(node_id=node.uuid,
+                                                  trait=trait)
         notify.emit_end_notification(context, node, 'update',
                                      chassis_uuid=chassis_uuid)
 
@@ -938,6 +964,9 @@ class Node(base.APIBase):
     maintenance_reason = wsme.wsattr(wtypes.text, readonly=True)
     """Indicates reason for putting a node in maintenance mode."""
 
+    fault = wsme.wsattr(wtypes.text, readonly=True)
+    """Indicates the active fault of a node."""
+
     target_provision_state = wsme.wsattr(wtypes.text, readonly=True)
     """The user modified desired provision state of the node."""
 
@@ -1036,6 +1065,9 @@ class Node(base.APIBase):
     traits = wtypes.ArrayType(str)
     """The traits associated with this node"""
 
+    bios_interface = wsme.wsattr(wtypes.text)
+    """The bios interface to be used for this node"""
+
     # NOTE(deva): "conductor_affinity" shouldn't be presented on the
     #             API because it's an internal value. Don't add it here.
 
@@ -1181,7 +1213,7 @@ class Node(base.APIBase):
                          'memory_mb': '1024', 'local_gb': '10', 'cpus': '1'},
                      updated_at=time, created_at=time,
                      provision_updated_at=time, instance_info={},
-                     maintenance=False, maintenance_reason=None,
+                     maintenance=False, maintenance_reason=None, fault=None,
                      inspection_finished_at=None, inspection_started_at=time,
                      console_enabled=False, clean_step={},
                      raid_config=None, target_raid_config=None,
@@ -1190,7 +1222,8 @@ class Node(base.APIBase):
                      deploy_interface=None, inspect_interface=None,
                      management_interface=None, power_interface=None,
                      raid_interface=None, vendor_interface=None,
-                     storage_interface=None, traits=[], rescue_interface=None)
+                     storage_interface=None, traits=[], rescue_interface=None,
+                     bios_interface=None)
         # NOTE(matty_dubs): The chassis_uuid getter() is based on the
         # _chassis_uuid variable:
         sample._chassis_uuid = 'edcad704-b2da-41d5-96d9-afd580ecfa12'
@@ -1214,7 +1247,8 @@ class NodePatchType(types.JsonPatchType):
                            '/provision_updated_at', '/maintenance_reason',
                            '/driver_internal_info', '/inspection_finished_at',
                            '/inspection_started_at', '/clean_step',
-                           '/raid_config', '/target_raid_config']
+                           '/raid_config', '/target_raid_config',
+                           '/fault']
 
 
 class NodeCollection(collection.Collection):
@@ -1459,6 +1493,7 @@ class NodesController(rest.RestController):
         'vifs': NodeVIFController,
         'volume': volume.VolumeController,
         'traits': NodeTraitsController,
+        'bios': bios.NodeBiosController,
     }
 
     @pecan.expose()
@@ -1469,10 +1504,12 @@ class NodesController(rest.RestController):
             pecan.abort(http_client.BAD_REQUEST, e.args[0])
         if not remainder:
             return
-        if ((remainder[0] == 'portgroups' and
-                not api_utils.allow_portgroups_subcontrollers()) or
-            (remainder[0] == 'vifs' and
-                not api_utils.allow_vifs_subcontroller())):
+        if ((remainder[0] == 'portgroups'
+                and not api_utils.allow_portgroups_subcontrollers())
+            or (remainder[0] == 'vifs'
+                and not api_utils.allow_vifs_subcontroller())
+            or (remainder[0] == 'bios' and
+                not api_utils.allow_bios_interface())):
             pecan.abort(http_client.NOT_FOUND)
         if remainder[0] == 'traits' and not api_utils.allow_traits():
             # NOTE(mgoddard): Returning here will ensure we exhibit the
@@ -1486,8 +1523,8 @@ class NodesController(rest.RestController):
     def _get_nodes_collection(self, chassis_uuid, instance_uuid, associated,
                               maintenance, provision_state, marker, limit,
                               sort_key, sort_dir, driver=None,
-                              resource_class=None,
-                              resource_url=None, fields=None):
+                              resource_class=None, resource_url=None,
+                              fields=None, fault=None, detail=None):
         if self.from_chassis and not chassis_uuid:
             raise exception.MissingParameterValue(
                 _("Chassis id not specified."))
@@ -1533,6 +1570,8 @@ class NodesController(rest.RestController):
                 filters['driver'] = driver
             if resource_class is not None:
                 filters['resource_class'] = resource_class
+            if fault is not None:
+                filters['fault'] = fault
 
             nodes = objects.Node.list(pecan.request.context, limit, marker_obj,
                                       sort_key=sort_key, sort_dir=sort_dir,
@@ -1544,6 +1583,9 @@ class NodesController(rest.RestController):
             if maintenance:
                 parameters['maintenance'] = maintenance
 
+        if detail is not None:
+            parameters['detail'] = detail
+
         return NodeCollection.convert_with_links(nodes, limit,
                                                  url=resource_url,
                                                  fields=fields,
@@ -1636,11 +1678,12 @@ class NodesController(rest.RestController):
     @METRICS.timer('NodesController.get_all')
     @expose.expose(NodeCollection, types.uuid, types.uuid, types.boolean,
                    types.boolean, wtypes.text, types.uuid, int, wtypes.text,
-                   wtypes.text, wtypes.text, types.listtype, wtypes.text)
+                   wtypes.text, wtypes.text, types.listtype, wtypes.text,
+                   wtypes.text, types.boolean)
     def get_all(self, chassis_uuid=None, instance_uuid=None, associated=None,
                 maintenance=None, provision_state=None, marker=None,
                 limit=None, sort_key='id', sort_dir='asc', driver=None,
-                fields=None, resource_class=None):
+                fields=None, resource_class=None, fault=None, detail=None):
         """Retrieve a list of nodes.
 
         :param chassis_uuid: Optional UUID of a chassis, to get only nodes for
@@ -1668,6 +1711,7 @@ class NodesController(rest.RestController):
                                that resource_class.
         :param fields: Optional, a list with a specified set of fields
                        of the resource to be returned.
+        :param fault: Optional string value to get only nodes with that fault.
         """
         cdict = pecan.request.context.to_policy_values()
         policy.authorize('baremetal:node:get', cdict, cdict)
@@ -1678,24 +1722,28 @@ class NodesController(rest.RestController):
         api_utils.check_for_invalid_state_and_allow_filter(provision_state)
         api_utils.check_allow_specify_driver(driver)
         api_utils.check_allow_specify_resource_class(resource_class)
-        if fields is None:
-            fields = _DEFAULT_RETURN_FIELDS
+        api_utils.check_allow_filter_by_fault(fault)
+
+        fields = api_utils.get_request_return_fields(fields, detail,
+                                                     _DEFAULT_RETURN_FIELDS)
+
         return self._get_nodes_collection(chassis_uuid, instance_uuid,
                                           associated, maintenance,
                                           provision_state, marker,
                                           limit, sort_key, sort_dir,
                                           driver=driver,
                                           resource_class=resource_class,
-                                          fields=fields)
+                                          fields=fields, fault=fault,
+                                          detail=detail)
 
     @METRICS.timer('NodesController.detail')
     @expose.expose(NodeCollection, types.uuid, types.uuid, types.boolean,
                    types.boolean, wtypes.text, types.uuid, int, wtypes.text,
-                   wtypes.text, wtypes.text, wtypes.text)
+                   wtypes.text, wtypes.text, wtypes.text, wtypes.text)
     def detail(self, chassis_uuid=None, instance_uuid=None, associated=None,
                maintenance=None, provision_state=None, marker=None,
                limit=None, sort_key='id', sort_dir='asc', driver=None,
-               resource_class=None):
+               resource_class=None, fault=None):
         """Retrieve a list of nodes with detail.
 
         :param chassis_uuid: Optional UUID of a chassis, to get only nodes for
@@ -1721,6 +1769,7 @@ class NodesController(rest.RestController):
                        driver.
         :param resource_class: Optional string value to get only nodes with
                                that resource_class.
+        :param fault: Optional string value to get only nodes with that fault.
         """
         cdict = pecan.request.context.to_policy_values()
         policy.authorize('baremetal:node:get', cdict, cdict)
@@ -1728,6 +1777,7 @@ class NodesController(rest.RestController):
         api_utils.check_for_invalid_state_and_allow_filter(provision_state)
         api_utils.check_allow_specify_driver(driver)
         api_utils.check_allow_specify_resource_class(resource_class)
+        api_utils.check_allow_filter_by_fault(fault)
         api_utils.check_allowed_fields([sort_key])
         # /detail should only work against collections
         parent = pecan.request.path.split('/')[:-1][-1]
@@ -1741,7 +1791,8 @@ class NodesController(rest.RestController):
                                           limit, sort_key, sort_dir,
                                           driver=driver,
                                           resource_class=resource_class,
-                                          resource_url=resource_url)
+                                          resource_url=resource_url,
+                                          fault=fault)
 
     @METRICS.timer('NodesController.validate')
     @expose.expose(wtypes.text, types.uuid_or_name, types.uuid)
@@ -1760,8 +1811,8 @@ class NodesController(rest.RestController):
         if node is not None:
             # We're invoking this interface using positional notation, or
             # explicitly using 'node'.  Try and determine which one.
-            if (not api_utils.allow_node_logical_names() and
-                not uuidutils.is_uuid_like(node)):
+            if (not api_utils.allow_node_logical_names()
+                and not uuidutils.is_uuid_like(node)):
                 raise exception.NotAcceptable()
 
         rpc_node = api_utils.get_rpc_node(node_uuid or node)
@@ -1805,13 +1856,13 @@ class NodesController(rest.RestController):
         if self.from_chassis:
             raise exception.OperationNotPermitted()
 
-        if (not api_utils.allow_resource_class() and
-                node.resource_class is not wtypes.Unset):
+        if (not api_utils.allow_resource_class()
+                and node.resource_class is not wtypes.Unset):
             raise exception.NotAcceptable()
 
         n_interface = node.network_interface
-        if (not api_utils.allow_network_interface() and
-                n_interface is not wtypes.Unset):
+        if (not api_utils.allow_network_interface()
+                and n_interface is not wtypes.Unset):
             raise exception.NotAcceptable()
 
         if not api_utils.allow_dynamic_interfaces():
@@ -1819,8 +1870,12 @@ class NodesController(rest.RestController):
                 if getattr(node, field) is not wsme.Unset:
                     raise exception.NotAcceptable()
 
-        if (not api_utils.allow_storage_interface() and
-                node.storage_interface is not wtypes.Unset):
+        if (not api_utils.allow_storage_interface()
+                and node.storage_interface is not wtypes.Unset):
+            raise exception.NotAcceptable()
+
+        if (not api_utils.allow_bios_interface() and
+                node.bios_interface is not wtypes.Unset):
             raise exception.NotAcceptable()
 
         if node.traits is not wtypes.Unset:
@@ -1828,8 +1883,8 @@ class NodesController(rest.RestController):
                     "be set via the node traits API.")
             raise exception.Invalid(msg)
 
-        if (not api_utils.allow_rescue_interface() and
-                node.rescue_interface is not wtypes.Unset):
+        if (not api_utils.allow_rescue_interface()
+                and node.rescue_interface is not wtypes.Unset):
             raise exception.NotAcceptable()
 
         # NOTE(deva): get_topic_for checks if node.driver is in the hash ring
@@ -1870,19 +1925,8 @@ class NodesController(rest.RestController):
                                      chassis_uuid=api_node.chassis_uuid)
         return api_node
 
-    @METRICS.timer('NodesController.patch')
-    @wsme.validate(types.uuid, [NodePatchType])
-    @expose.expose(Node, types.uuid_or_name, body=[NodePatchType])
-    def patch(self, node_ident, patch):
-        """Update an existing node.
-
-        :param node_ident: UUID or logical name of a node.
-        :param patch: a json PATCH document to apply to this node.
-        """
-        context = pecan.request.context
-        cdict = context.to_policy_values()
-        policy.authorize('baremetal:node:update', cdict, cdict)
-
+    # NOTE (yolanda): isolate validation to avoid patch too complex pep error
+    def _validate_patch(self, patch):
         if self.from_chassis:
             raise exception.OperationNotPermitted()
 
@@ -1913,6 +1957,25 @@ class NodesController(rest.RestController):
         if r_interface and not api_utils.allow_rescue_interface():
             raise exception.NotAcceptable()
 
+        b_interface = api_utils.get_patch_values(patch, '/bios_interface')
+        if b_interface and not api_utils.allow_bios_interface():
+            raise exception.NotAcceptable()
+
+    @METRICS.timer('NodesController.patch')
+    @wsme.validate(types.uuid, [NodePatchType])
+    @expose.expose(Node, types.uuid_or_name, body=[NodePatchType])
+    def patch(self, node_ident, patch):
+        """Update an existing node.
+
+        :param node_ident: UUID or logical name of a node.
+        :param patch: a json PATCH document to apply to this node.
+        """
+        context = pecan.request.context
+        cdict = context.to_policy_values()
+        policy.authorize('baremetal:node:update', cdict, cdict)
+
+        self._validate_patch(patch)
+
         rpc_node = api_utils.get_rpc_node_with_suffix(node_ident)
 
         remove_inst_uuid_patch = [{'op': 'remove', 'path': '/instance_uuid'}]
@@ -1928,6 +1991,13 @@ class NodesController(rest.RestController):
                     "is in progress.")
             raise wsme.exc.ClientSideError(
                 msg % node_ident, status_code=http_client.CONFLICT)
+        elif (rpc_node.provision_state == ir_states.INSPECTING
+              and api_utils.allow_inspect_wait_state()):
+            msg = _('Cannot update node "%(node)s" while it is in state '
+                    '"%(state)s".') % {'node': rpc_node.uuid,
+                                       'state': ir_states.INSPECTING}
+            raise wsme.exc.ClientSideError(msg,
+                                           status_code=http_client.CONFLICT)
 
         names = api_utils.get_patch_values(patch, '/name')
         if len(names):
