@@ -29,6 +29,7 @@ import wsme
 
 from ironic.api.controllers.v1 import versions
 from ironic.common import exception
+from ironic.common import faults
 from ironic.common.i18n import _
 from ironic.common import states
 from ironic.common import utils
@@ -142,8 +143,8 @@ def is_path_removed(patch, path):
     """
     path = path.rstrip('/')
     for p in patch:
-        if ((p['path'] == path or p['path'].startswith(path + '/')) and
-                p['op'] == 'remove'):
+        if ((p['path'] == path or p['path'].startswith(path + '/'))
+                and p['op'] == 'remove'):
             return True
 
 
@@ -349,8 +350,8 @@ def check_allow_specify_fields(fields):
     attributes, this method checks if the required version is being
     requested.
     """
-    if (fields is not None and pecan.request.version.minor <
-            versions.MINOR_8_FETCHING_SUBSET_OF_FIELDS):
+    if (fields is not None and pecan.request.version.minor
+            < versions.MINOR_8_FETCHING_SUBSET_OF_FIELDS):
         raise exception.NotAcceptable()
 
 
@@ -362,6 +363,8 @@ def check_allowed_fields(fields):
     """
     if fields is None:
         return
+    if 'bios_interface' in fields and not allow_bios_interface():
+        raise exception.NotAcceptable()
     if 'network_interface' in fields and not allow_network_interface():
         raise exception.NotAcceptable()
     if 'resource_class' in fields and not allow_resource_class():
@@ -385,8 +388,8 @@ def check_allowed_portgroup_fields(fields):
     """
     if fields is None:
         return
-    if (('mode' in fields or 'properties' in fields) and
-            not allow_portgroup_mode_properties()):
+    if (('mode' in fields or 'properties' in fields)
+            and not allow_portgroup_mode_properties()):
         raise exception.NotAcceptable()
 
 
@@ -402,8 +405,8 @@ def check_for_invalid_state_and_allow_filter(provision_state):
     Version 1.9 of the API allows filter nodes by provision state.
     """
     if provision_state is not None:
-        if (pecan.request.version.minor <
-                versions.MINOR_9_PROVISION_STATE_FILTER):
+        if (pecan.request.version.minor
+                < versions.MINOR_9_PROVISION_STATE_FILTER):
             raise exception.NotAcceptable()
         valid_states = states.machine.states
         if provision_state not in valid_states:
@@ -416,8 +419,8 @@ def check_allow_specify_driver(driver):
 
     Version 1.16 of the API allows filter nodes by driver.
     """
-    if (driver is not None and pecan.request.version.minor <
-            versions.MINOR_16_DRIVER_FILTER):
+    if (driver is not None and pecan.request.version.minor
+            < versions.MINOR_16_DRIVER_FILTER):
         raise exception.NotAcceptable(_(
             "Request not acceptable. The minimal required API version "
             "should be %(base)s.%(opr)s") %
@@ -430,8 +433,8 @@ def check_allow_specify_resource_class(resource_class):
 
     Version 1.21 of the API allows filtering nodes by resource_class.
     """
-    if (resource_class is not None and pecan.request.version.minor <
-            versions.MINOR_21_RESOURCE_CLASS):
+    if (resource_class is not None and pecan.request.version.minor
+            < versions.MINOR_21_RESOURCE_CLASS):
         raise exception.NotAcceptable(_(
             "Request not acceptable. The minimal required API version "
             "should be %(base)s.%(opr)s") %
@@ -477,6 +480,26 @@ def check_allow_configdrive(target):
             msg, status_code=http_client.BAD_REQUEST)
 
 
+def check_allow_filter_by_fault(fault):
+    """Check if filtering nodes by fault is allowed.
+
+    Version 1.42 of the API allows filtering nodes by fault.
+    """
+    if (fault is not None and pecan.request.version.minor
+            < versions.MINOR_42_FAULT):
+        raise exception.NotAcceptable(_(
+            "Request not acceptable. The minimal required API version "
+            "should be %(base)s.%(opr)s") % {'base': versions.BASE_VERSION,
+                                             'opr': versions.MINOR_42_FAULT})
+
+    if fault is not None and fault not in faults.VALID_FAULTS:
+        msg = (_('Unrecognized fault "%(fault)s" is specified, allowed faults '
+                 'are %(valid_faults)s') %
+               {'fault': fault, 'valid_faults': faults.VALID_FAULTS})
+        raise wsme.exc.ClientSideError(
+            msg, status_code=http_client.BAD_REQUEST)
+
+
 def initial_node_provision_state():
     """Return node state to use by default when creating new nodes.
 
@@ -519,8 +542,8 @@ def allow_links_node_states_and_driver_properties():
     Version 1.14 of the API allows the display of links to node states
     and driver properties.
     """
-    return (pecan.request.version.minor >=
-            versions.MINOR_14_LINKS_NODESTATES_DRIVERPROPERTIES)
+    return (pecan.request.version.minor
+            >= versions.MINOR_14_LINKS_NODESTATES_DRIVERPROPERTIES)
 
 
 def allow_port_internal_info():
@@ -528,8 +551,8 @@ def allow_port_internal_info():
 
     Version 1.18 of the API exposes internal_info readonly field for the port.
     """
-    return (pecan.request.version.minor >=
-            versions.MINOR_18_PORT_INTERNAL_INFO)
+    return (pecan.request.version.minor
+            >= versions.MINOR_18_PORT_INTERNAL_INFO)
 
 
 def allow_port_advanced_net_fields():
@@ -537,8 +560,8 @@ def allow_port_advanced_net_fields():
 
     Version 1.19 of the API added support for these new fields in port object.
     """
-    return (pecan.request.version.minor >=
-            versions.MINOR_19_PORT_ADVANCED_NET_FIELDS)
+    return (pecan.request.version.minor
+            >= versions.MINOR_19_PORT_ADVANCED_NET_FIELDS)
 
 
 def allow_network_interface():
@@ -546,8 +569,8 @@ def allow_network_interface():
 
     Version 1.20 of the API added support for network interfaces.
     """
-    return (pecan.request.version.minor >=
-            versions.MINOR_20_NETWORK_INTERFACE)
+    return (pecan.request.version.minor
+            >= versions.MINOR_20_NETWORK_INTERFACE)
 
 
 def allow_resource_class():
@@ -555,8 +578,8 @@ def allow_resource_class():
 
     Version 1.21 of the API added support for resource_class.
     """
-    return (pecan.request.version.minor >=
-            versions.MINOR_21_RESOURCE_CLASS)
+    return (pecan.request.version.minor
+            >= versions.MINOR_21_RESOURCE_CLASS)
 
 
 def allow_ramdisk_endpoints():
@@ -572,8 +595,8 @@ def allow_portgroups():
 
     Version 1.23 of the API added support for PortGroups.
     """
-    return (pecan.request.version.minor >=
-            versions.MINOR_23_PORTGROUPS)
+    return (pecan.request.version.minor
+            >= versions.MINOR_23_PORTGROUPS)
 
 
 def allow_portgroups_subcontrollers():
@@ -582,8 +605,8 @@ def allow_portgroups_subcontrollers():
     Version 1.24 of the API added support for Portgroups as
     subcontrollers
     """
-    return (pecan.request.version.minor >=
-            versions.MINOR_24_PORTGROUPS_SUBCONTROLLERS)
+    return (pecan.request.version.minor
+            >= versions.MINOR_24_PORTGROUPS_SUBCONTROLLERS)
 
 
 def allow_remove_chassis_uuid():
@@ -592,8 +615,8 @@ def allow_remove_chassis_uuid():
     Version 1.25 of the API added support for chassis_uuid
     removal
     """
-    return (pecan.request.version.minor >=
-            versions.MINOR_25_UNSET_CHASSIS_UUID)
+    return (pecan.request.version.minor
+            >= versions.MINOR_25_UNSET_CHASSIS_UUID)
 
 
 def allow_portgroup_mode_properties():
@@ -602,8 +625,8 @@ def allow_portgroup_mode_properties():
     Version 1.26 of the API added mode and properties fields to portgroup
     object.
     """
-    return (pecan.request.version.minor >=
-            versions.MINOR_26_PORTGROUP_MODE_PROPERTIES)
+    return (pecan.request.version.minor
+            >= versions.MINOR_26_PORTGROUP_MODE_PROPERTIES)
 
 
 def allow_vifs_subcontroller():
@@ -612,8 +635,8 @@ def allow_vifs_subcontroller():
     Version 1.28 of the API added support for VIFs to be
     attached to Nodes.
     """
-    return (pecan.request.version.minor >=
-            versions.MINOR_28_VIFS_SUBCONTROLLER)
+    return (pecan.request.version.minor
+            >= versions.MINOR_28_VIFS_SUBCONTROLLER)
 
 
 def allow_dynamic_drivers():
@@ -622,8 +645,8 @@ def allow_dynamic_drivers():
     Version 1.30 of the API added support for all of the driver
     composition related calls in the /v1/drivers API.
     """
-    return (pecan.request.version.minor >=
-            versions.MINOR_30_DYNAMIC_DRIVERS)
+    return (pecan.request.version.minor
+            >= versions.MINOR_30_DYNAMIC_DRIVERS)
 
 
 def allow_dynamic_interfaces():
@@ -632,8 +655,8 @@ def allow_dynamic_interfaces():
     Version 1.31 of the API added support for viewing and setting the fields
     in ``V31_FIELDS`` on the node object.
     """
-    return (pecan.request.version.minor >=
-            versions.MINOR_31_DYNAMIC_INTERFACES)
+    return (pecan.request.version.minor
+            >= versions.MINOR_31_DYNAMIC_INTERFACES)
 
 
 def allow_volume():
@@ -649,8 +672,8 @@ def allow_storage_interface():
 
     Version 1.33 of the API added support for storage interfaces.
     """
-    return (pecan.request.version.minor >=
-            versions.MINOR_33_STORAGE_INTERFACE)
+    return (pecan.request.version.minor
+            >= versions.MINOR_33_STORAGE_INTERFACE)
 
 
 def allow_port_physical_network():
@@ -661,9 +684,9 @@ def allow_port_physical_network():
     supports the physical_network field as this may not be the case during a
     rolling upgrade.
     """
-    return ((pecan.request.version.minor >=
-             versions.MINOR_34_PORT_PHYSICAL_NETWORK) and
-            objects.Port.supports_physical_network())
+    return ((pecan.request.version.minor
+             >= versions.MINOR_34_PORT_PHYSICAL_NETWORK)
+            and objects.Port.supports_physical_network())
 
 
 def allow_node_rebuild_with_configdrive():
@@ -671,8 +694,8 @@ def allow_node_rebuild_with_configdrive():
 
     Version 1.35 of the API added support for node rebuild with configdrive.
     """
-    return (pecan.request.version.minor >=
-            versions.MINOR_35_REBUILD_CONFIG_DRIVE)
+    return (pecan.request.version.minor
+            >= versions.MINOR_35_REBUILD_CONFIG_DRIVE)
 
 
 def allow_agent_version_in_heartbeat():
@@ -681,8 +704,8 @@ def allow_agent_version_in_heartbeat():
     Version 1.36 of the API added the ability for agents to pass their version
     information to Ironic on heartbeat.
     """
-    return (pecan.request.version.minor >=
-            versions.MINOR_36_AGENT_VERSION_HEARTBEAT)
+    return (pecan.request.version.minor
+            >= versions.MINOR_36_AGENT_VERSION_HEARTBEAT)
 
 
 def allow_rescue_interface():
@@ -693,6 +716,14 @@ def allow_rescue_interface():
     return pecan.request.version.minor >= versions.MINOR_38_RESCUE_INTERFACE
 
 
+def allow_bios_interface():
+    """Check if we should support bios interface.
+
+    Version 1.40 of the API added support for bios interface.
+    """
+    return pecan.request.version.minor >= versions.MINOR_40_BIOS_INTERFACE
+
+
 def get_controller_reserved_names(cls):
     """Get reserved names for a given controller.
 
@@ -708,7 +739,7 @@ def get_controller_reserved_names(cls):
         if isinstance(member, rest.RestController)]
 
     if hasattr(cls, '_custom_actions'):
-        reserved_names += cls._custom_actions.keys()
+        reserved_names += list(cls._custom_actions)
 
     return reserved_names
 
@@ -719,3 +750,146 @@ def allow_traits():
     Version 1.37 of the API allows traits for the node.
     """
     return pecan.request.version.minor >= versions.MINOR_37_NODE_TRAITS
+
+
+def allow_inspect_wait_state():
+    """Check if inspect wait is allowed for the node.
+
+    Version 1.39 of the API adds 'inspect wait' state to substitute
+    'inspecting' state during asynchronous hardware inspection.
+    """
+    return pecan.request.version.minor >= versions.MINOR_39_INSPECT_WAIT
+
+
+def allow_inspect_abort():
+    """Check if inspection abort is allowed.
+
+    Version 1.41 of the API added support for inspection abort
+    """
+    return pecan.request.version.minor >= versions.MINOR_41_INSPECTION_ABORT
+
+
+def handle_post_port_like_extra_vif(p_dict):
+    """Handle a Post request that sets .extra['vif_port_id'].
+
+    This handles attach of VIFs via specifying the VIF port ID
+    in a port or port group's extra['vif_port_id'] field.
+
+    :param p_dict: a dictionary with field names/values for the port or
+                   port group
+    :return: VIF or None
+    """
+    vif = p_dict.get('extra', {}).get('vif_port_id')
+    if vif:
+        # TODO(rloo): in Stein cycle: if API version >= 1.28, remove
+        #             warning and support for extra[]; else (< 1.28)
+        #             still support it; continue copying to internal_info
+        #             (see bug 1722850). i.e., change the 7 lines of code
+        #             below to something like:
+        #                 if not api_utils.allow_vifs_subcontroller():
+        #                     internal_info = {'tenant_vif_port_id': vif}
+        #                     pg_dict['internal_info'] = internal_info
+        if allow_vifs_subcontroller():
+            utils.warn_about_deprecated_extra_vif_port_id()
+        # NOTE(rloo): this value should really be in .internal_info[..]
+        #             which is what would happen if they had used the
+        #             POST /v1/nodes/<node>/vifs API.
+        internal_info = {'tenant_vif_port_id': vif}
+        p_dict['internal_info'] = internal_info
+    return vif
+
+
+def handle_patch_port_like_extra_vif(rpc_object, api_object, patch):
+    """Handle a Patch request that modifies .extra['vif_port_id'].
+
+    This handles attach/detach of VIFs via the VIF port ID
+    in a port or port group's extra['vif_port_id'] field.
+
+    :param rpc_object: a Port or Portgroup RPC object
+    :param api_object: the corresponding Port or Portgroup API object
+    :param patch: the JSON patch in the API request
+    """
+    vif_list = get_patch_values(patch, '/extra/vif_port_id')
+    vif = None
+    if vif_list:
+        # if specified more than once, use the last value
+        vif = vif_list[-1]
+
+        # TODO(rloo): in Stein cycle: if API version >= 1.28, remove this
+        # warning and don't copy to internal_info; else (<1.28) still
+        # support it; continue copying to internal_info (see bug 1722850).
+        # i.e., change the 8 lines of code below to something like:
+        #   if not allow_vifs_subcontroller():
+        #       int_info = rpc_object.internal_info.get('tenant_vif_port_id')
+        #       if (not int_info or
+        #           int_info == rpc_object.extra.get('vif_port_id')):
+        #           api_object.internal_info['tenant_vif_port_id'] = vif
+        if allow_vifs_subcontroller():
+            utils.warn_about_deprecated_extra_vif_port_id()
+        # NOTE(rloo): if the user isn't also using the REST API
+        #             'POST nodes/<node>/vifs', we are safe to copy the
+        #             .extra[] value to the .internal_info location
+        int_info = rpc_object.internal_info.get('tenant_vif_port_id')
+        if (not int_info or int_info == rpc_object.extra.get('vif_port_id')):
+            api_object.internal_info['tenant_vif_port_id'] = vif
+
+    elif is_path_removed(patch, '/extra/vif_port_id'):
+        # TODO(rloo): in Stein cycle: if API version >= 1.28, remove this
+        # warning and don't remove from internal_info; else (<1.28) still
+        # support it; remove from internal_info (see bug 1722850).
+        # i.e., change the 8 lines of code below to something like:
+        #   if not allow_vifs_subcontroller():
+        #     int_info = rpc_object.internal_info.get('tenant_vif...')
+        #     if (int_info and int_info==rpc_object.extra.get('vif_port_id')):
+        #         api_object.internal_info['tenant_vif_port_id'] = None
+        if allow_vifs_subcontroller():
+            utils.warn_about_deprecated_extra_vif_port_id()
+        # NOTE(rloo): if the user isn't also using the REST API
+        #             'POST nodes/<node>/vifs', we are safe to remove the
+        #             .extra[] value from the .internal_info location
+        int_info = rpc_object.internal_info.get('tenant_vif_port_id')
+        if (int_info and int_info == rpc_object.extra.get('vif_port_id')):
+            api_object.internal_info.pop('tenant_vif_port_id')
+
+
+def allow_detail_query():
+    """Check if passing a detail=True query string is allowed.
+
+    Version 1.43 allows a user to pass the detail query string to
+    list the resource with all the fields.
+    """
+    return (pecan.request.version.minor >=
+            versions.MINOR_43_ENABLE_DETAIL_QUERY)
+
+
+def get_request_return_fields(fields, detail, default_fields):
+    """Calculate fields to return from an API request
+
+    The fields query and detail=True query can not be passed into a request at
+    the same time. To use the detail query we need to be on a version of the
+    API greater than 1.43. This function raises an InvalidParameterValue
+    exception if either of these conditions are not met.
+
+    If these checks pass then this function will return either the fields
+    passed in or the default fields provided.
+
+    :param fields: The fields query passed into the API request.
+    :param detail: The detail query passed into the API request.
+    :param default_fields: The default fields to return if fields=None and
+        detail=None.
+    :raises: InvalidParameterValue if there is an invalid combination of query
+        strings or API version.
+    :returns: 'fields' passed in value or 'default_fields'
+    """
+
+    if detail is not None and not allow_detail_query():
+        raise exception.InvalidParameterValue(
+            "Invalid query parameter ?detail=%s received." % detail)
+
+    if fields is not None and detail:
+        raise exception.InvalidParameterValue(
+            "Can not specify ?detail=True and fields in the same request.")
+
+    if fields is None and not detail:
+        return default_fields
+    return fields
