@@ -29,6 +29,8 @@ from ironic.common import service
 from ironic.conf import CONF
 from ironic.db import api as db_api
 from ironic.db import migration
+from ironic.objects import port
+from ironic.objects import portgroup
 from ironic import version
 
 
@@ -45,9 +47,6 @@ dbapi = db_api.get_instance()
 # number migrated. If the function determines that no migrations are needed,
 # it returns (0, 0).
 #
-# The last migration step should always remain the last one -- it migrates
-# all objects to their latest known versions.
-#
 # Example of a function docstring:
 #
 #        def sample_data_migration(context, max_count):
@@ -65,13 +64,12 @@ dbapi = db_api.get_instance()
 # object, in case it is lazy loaded. The attribute will be accessed when needed
 # by doing getattr on the object
 ONLINE_MIGRATIONS = (
-    # Added in Pike, modified in Queens
-    # TODO(rloo): remove in Rocky
-    (dbapi, 'backfill_version_column'),
-    # TODO(dtantsur): remove when classic drivers are removed (Rocky?)
-    (dbapi, 'migrate_to_hardware_types'),
-    # NOTE(rloo): Don't remove this; it should always be last
-    (dbapi, 'update_to_latest_versions'),
+    # Added in Rocky
+    # TODO(rloo): remove in Stein
+    (port, 'migrate_vif_port_id'),
+    # Added in Rocky
+    # TODO(rloo): remove in Stein
+    (portgroup, 'migrate_vif_port_id'),
 )
 
 
