@@ -92,7 +92,6 @@ class IronicException(Exception):
     """
     _msg_fmt = _("An unknown exception occurred.")
     code = http_client.INTERNAL_SERVER_ERROR
-    headers = {}
     safe = False
 
     def __init__(self, message=None, **kwargs):
@@ -105,6 +104,8 @@ class IronicException(Exception):
                 self.kwargs['code'] = self.code
             except AttributeError:
                 pass
+        else:
+            self.code = int(kwargs['code'])
 
         if not message:
             try:
@@ -127,12 +128,10 @@ class IronicException(Exception):
 
     def __str__(self):
         """Encode to utf-8 then wsme api can consume it as well."""
-        value = self.__unicode__()
-        if six.PY3:
-            # On Python 3 unicode is the same as str
-            return value
-        else:
-            return value.encode('utf-8')
+        if not six.PY3:
+            return six.text_type(self.args[0]).encode('utf-8')
+
+        return self.args[0]
 
     def __unicode__(self):
         """Return a unicode representation of the exception message."""
@@ -348,15 +347,10 @@ class NoValidDefaultForInterface(InvalidParameterValue):
     # NOTE(rloo): in the line below, there is no blank space after 'For'
     #             because node_info could be an empty string. If node_info
     #             is not empty, it should start with a space.
-    _msg_fmt = _("For%(node_info)s %(driver_type)s '%(driver)s', no default "
+    _msg_fmt = _("For%(node_info)s hardware type '%(driver)s', no default "
                  "value found for %(interface_type)s interface.")
 
 
-class MustBeNone(InvalidParameterValue):
-    _msg_fmt = _("For node %(node)s with driver %(driver)s, these node "
-                 "fields must be set to None: %(node_fields)s.")
-
-
 class ImageNotFound(NotFound):
     _msg_fmt = _("Image %(image_id)s could not be found.")
 
@@ -426,14 +420,9 @@ class VolumeTargetNotFound(NotFound):
     _msg_fmt = _("Volume target %(target)s could not be found.")
 
 
-class DriverNameConflict(IronicException):
-    _msg_fmt = _("Classic and dynamic drivers cannot have the "
-                 "same names '%(names)s'.")
-
-
 class NoDriversLoaded(IronicException):
     _msg_fmt = _("Conductor %(conductor)s cannot be started "
-                 "because no drivers were loaded.")
+                 "because no hardware types were loaded.")
 
 
 class ConductorNotFound(NotFound):
@@ -587,6 +576,10 @@ class DriverLoadError(IronicException):
                  "loaded. Reason: %(reason)s.")
 
 
+class DriverOperationError(IronicException):
+    _msg_fmt = _("Runtime driver %(driver)s failure. Reason: %(reason)s.")
+
+
 class ConsoleError(IronicException):
     pass
 
@@ -603,15 +596,15 @@ class PasswordFileFailedToCreate(IronicException):
     _msg_fmt = _("Failed to create the password file. %(error)s")
 
 
-class IloOperationError(IronicException):
+class IloOperationError(DriverOperationError):
     _msg_fmt = _("%(operation)s failed, error: %(error)s")
 
 
-class IloOperationNotSupported(IronicException):
+class IloOperationNotSupported(DriverOperationError):
     _msg_fmt = _("%(operation)s not supported. error: %(error)s")
 
 
-class DracOperationError(IronicException):
+class DracOperationError(DriverOperationError):
     _msg_fmt = _('DRAC operation failed. Reason: %(error)s')
 
 
@@ -644,7 +637,7 @@ class SwiftObjectNotFoundError(SwiftOperationError):
                  "not found. Operation '%(operation)s' failed.")
 
 
-class SNMPFailure(IronicException):
+class SNMPFailure(DriverOperationError):
     _msg_fmt = _("SNMP operation '%(operation)s' failed: %(error)s")
 
 
@@ -653,11 +646,11 @@ class FileSystemNotSupported(IronicException):
                  "File system %(fs)s is not supported.")
 
 
-class IRMCOperationError(IronicException):
+class IRMCOperationError(DriverOperationError):
     _msg_fmt = _('iRMC %(operation)s failed. Reason: %(error)s')
 
 
-class IRMCSharedFileSystemNotMounted(IronicException):
+class IRMCSharedFileSystemNotMounted(DriverOperationError):
     _msg_fmt = _("iRMC shared file system '%(share)s' is not mounted.")
 
 
@@ -677,7 +670,7 @@ class DirectoryNotWritable(IronicException):
     _msg_fmt = _("Directory %(dir)s is not writable.")
 
 
-class UcsOperationError(IronicException):
+class UcsOperationError(DriverOperationError):
     _msg_fmt = _("Cisco UCS client: operation %(operation)s failed for node"
                  " %(node)s. Reason: %(error)s")
 
@@ -692,11 +685,11 @@ class ImageUploadFailed(IronicException):
                  "%(web_server)s, reason: %(reason)s")
 
 
-class CIMCException(IronicException):
+class CIMCException(DriverOperationError):
     _msg_fmt = _("Cisco IMC exception occurred for node %(node)s: %(error)s")
 
 
-class OneViewError(IronicException):
+class OneViewError(DriverOperationError):
     _msg_fmt = _("OneView exception occurred. Error: %(error)s")
 
 
@@ -738,7 +731,7 @@ class StorageError(IronicException):
     _msg_fmt = _("Storage operation failure.")
 
 
-class RedfishError(IronicException):
+class RedfishError(DriverOperationError):
     _msg_fmt = _("Redfish exception occurred. Error: %(error)s")
 
 
@@ -774,3 +767,19 @@ class InstanceRescueFailure(IronicException):
 class InstanceUnrescueFailure(IronicException):
     _msg_fmt = _('Failed to unrescue instance %(instance)s for node '
                  '%(node)s: %(reason)s')
+
+
+class XClarityError(IronicException):
+    _msg_fmt = _("XClarity exception occurred. Error: %(error)s")
+
+
+class BIOSSettingAlreadyExists(Conflict):
+    _msg_fmt = _('A BIOS setting %(name)s for node %(node)s already exists.')
+
+
+class BIOSSettingNotFound(NotFound):
+    _msg_fmt = _("Node %(node)s doesn't have a BIOS setting '%(name)s'")
+
+
+class BIOSSettingListNotFound(NotFound):
+    _msg_fmt = _("Node %(node)s doesn't have BIOS settings '%(names)s'")
