@@ -12,6 +12,9 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from ironic.common.i18n import _
+
+
 # NOTE(xek): This decides the version cap of RPC messages sent to conductor
 # and objects during rolling upgrades, when [DEFAULT]/pin_release_version
 # configuration is set.
@@ -55,45 +58,6 @@
 # oldest named release.
 
 RELEASE_MAPPING = {
-    '8.0': {
-        'api': '1.31',
-        'rpc': '1.40',
-        'objects': {
-            'Node': ['1.21'],
-            'Conductor': ['1.2'],
-            'Chassis': ['1.3'],
-            'Port': ['1.6'],
-            'Portgroup': ['1.3'],
-            'VolumeConnector': ['1.0'],
-            'VolumeTarget': ['1.0'],
-        }
-    },
-    '9.0': {
-        'api': '1.34',
-        'rpc': '1.41',
-        'objects': {
-            'Node': ['1.21'],
-            'Conductor': ['1.2'],
-            'Chassis': ['1.3'],
-            'Port': ['1.7'],
-            'Portgroup': ['1.3'],
-            'VolumeConnector': ['1.0'],
-            'VolumeTarget': ['1.0'],
-        }
-    },
-    '9.1': {
-        'api': '1.34',
-        'rpc': '1.41',
-        'objects': {
-            'Node': ['1.21'],
-            'Conductor': ['1.2'],
-            'Chassis': ['1.3'],
-            'Port': ['1.7'],
-            'Portgroup': ['1.3'],
-            'VolumeConnector': ['1.0'],
-            'VolumeTarget': ['1.0'],
-        }
-    },
     '9.2': {
         'rpc': '1.41',
         'api': '1.35',
@@ -136,14 +100,14 @@ RELEASE_MAPPING = {
         }
     },
     'master': {
-        'api': '1.38',
+        'api': '1.43',
         'rpc': '1.44',
         'objects': {
-            'Node': ['1.23'],
+            'Node': ['1.24', '1.25'],
             'Conductor': ['1.2'],
             'Chassis': ['1.3'],
-            'Port': ['1.7'],
-            'Portgroup': ['1.3'],
+            'Port': ['1.8'],
+            'Portgroup': ['1.4'],
             'Trait': ['1.0'],
             'TraitList': ['1.0'],
             'VolumeConnector': ['1.0'],
@@ -164,12 +128,14 @@ RELEASE_MAPPING = {
 #            release (that we are no longer supporting for a rolling upgrade).
 #
 #            There should be at most two named mappings here.
-RELEASE_MAPPING['pike'] = RELEASE_MAPPING['9.1']
 RELEASE_MAPPING['queens'] = RELEASE_MAPPING['10.1']
 
 # List of available versions with named versions first; 'master' is excluded.
 RELEASE_VERSIONS = sorted(set(RELEASE_MAPPING) - {'master'}, reverse=True)
 
+# List of available (version, description) tuples.
+RELEASE_VERSIONS_DESCS = [(v, _('"%s" release') % v) for v in RELEASE_VERSIONS]
+
 
 def get_object_versions(releases=None, objects=None):
     """Gets the supported versions for all objects.
