@@ -164,13 +164,20 @@ INSPECTING = 'inspecting'
 """ Node is under inspection.
 
 This is the provision state used when inspection is started. A successfully
-inspected node shall transition to MANAGEABLE state.
+inspected node shall transition to MANAGEABLE state. For asynchronous
+inspection, node shall transition to INSPECTWAIT state.
 """
 
-
 INSPECTFAIL = 'inspect failed'
 """ Node inspection failed. """
 
+INSPECTWAIT = 'inspect wait'
+""" Node is under inspection.
+
+This is the provision state used when an asynchronous inspection is in
+progress. A successfully inspected node shall transition to MANAGEABLE state.
+"""
+
 ADOPTING = 'adopting'
 """ Node is being adopted.
 
@@ -209,8 +216,11 @@ UNRESCUEFAIL = 'unrescue failed'
 UNRESCUING = 'unrescuing'
 """ Node is being restored from rescue mode (to active state). """
 
-UPDATE_ALLOWED_STATES = (DEPLOYFAIL, INSPECTING, INSPECTFAIL, CLEANFAIL, ERROR,
-                         VERIFYING, ADOPTFAIL, RESCUEFAIL, UNRESCUEFAIL)
+# NOTE(kaifeng): INSPECTING is allowed to keep backwards compatibility,
+# starting from API 1.39 node update is disallowed in this state.
+UPDATE_ALLOWED_STATES = (DEPLOYFAIL, INSPECTING, INSPECTFAIL, INSPECTWAIT,
+                         CLEANFAIL, ERROR, VERIFYING, ADOPTFAIL, RESCUEFAIL,
+                         UNRESCUEFAIL)
 """Transitional states in which we allow updating a node."""
 
 DELETE_ALLOWED_STATES = (AVAILABLE, MANAGEABLE, ENROLL, ADOPTFAIL)
@@ -220,8 +230,8 @@ STABLE_STATES = (ENROLL, MANAGEABLE, AVAILABLE, ACTIVE, ERROR, RESCUE)
 """States that will not transition unless receiving a request."""
 
 UNSTABLE_STATES = (DEPLOYING, DEPLOYWAIT, CLEANING, CLEANWAIT, VERIFYING,
-                   DELETING, INSPECTING, ADOPTING, RESCUING, RESCUEWAIT,
-                   UNRESCUING)
+                   DELETING, INSPECTING, INSPECTWAIT, ADOPTING, RESCUING,
+                   RESCUEWAIT, UNRESCUING)
 """States that can be changed without external request."""
 
 ##############
@@ -258,6 +268,7 @@ def on_enter(new_state, event):
     LOG.debug("Entering new state '%s' in response to event '%s'",
               new_state, event)
 
+
 watchers = {}
 watchers['on_exit'] = on_exit
 watchers['on_enter'] = on_enter
@@ -292,6 +303,7 @@ machine.add_transition(AVAILABLE, DEPLOYING, 'deploy')
 # Add inspect* states.
 machine.add_state(INSPECTING, target=MANAGEABLE, **watchers)
 machine.add_state(INSPECTFAIL, target=MANAGEABLE, **watchers)
+machine.add_state(INSPECTWAIT, target=MANAGEABLE, **watchers)
 
 # Add adopt* states
 machine.add_state(ADOPTING, target=ACTIVE, **watchers)
@@ -392,6 +404,18 @@ machine.add_transition(INSPECTING, MANAGEABLE, 'done')
 # Inspection may fail.
 machine.add_transition(INSPECTING, INSPECTFAIL, 'fail')
 
+# Transition for asynchronous inspection
+machine.add_transition(INSPECTING, INSPECTWAIT, 'wait')
+
+# Inspection is done
+machine.add_transition(INSPECTWAIT, MANAGEABLE, 'done')
+
+# Inspection failed.
+machine.add_transition(INSPECTWAIT, INSPECTFAIL, 'fail')
+
+# Inspection is aborted.
+machine.add_transition(INSPECTWAIT, INSPECTFAIL, 'abort')
+
 # Move the node to manageable state for any other
 # action.
 machine.add_transition(INSPECTFAIL, MANAGEABLE, 'manage')
