@@ -30,6 +30,7 @@ import tempfile
 import jinja2
 from oslo_concurrency import processutils
 from oslo_log import log as logging
+from oslo_serialization import jsonutils
 from oslo_utils import netutils
 from oslo_utils import timeutils
 import pytz
@@ -90,8 +91,8 @@ def is_valid_datapath_id(datapath_id):
 
     """
     m = "^[0-9a-f]{16}$"
-    return (isinstance(datapath_id, six.string_types) and
-            re.match(m, datapath_id.lower()))
+    return (isinstance(datapath_id, six.string_types)
+            and re.match(m, datapath_id.lower()))
 
 
 _is_valid_logical_name_re = re.compile(r'^[A-Z0-9-._~]+$', re.I)
@@ -153,7 +154,7 @@ def is_valid_no_proxy(no_proxy):
     """
     if not isinstance(no_proxy, six.string_types):
         return False
-    hostname_re = re.compile('(?!-)[A-Z\d-]{1,63}(?<!-)$', re.IGNORECASE)
+    hostname_re = re.compile('(?!-)[A-Z\\d-]{1,63}(?<!-)$', re.IGNORECASE)
     for hostname in no_proxy.split(','):
         hostname = hostname.strip().split(':')[0]
         if not hostname:
@@ -497,14 +498,52 @@ def render_template(template, params, is_file=True):
         loader = jinja2.DictLoader({tmpl_name: template})
     env = jinja2.Environment(loader=loader)
     tmpl = env.get_template(tmpl_name)
-    return tmpl.render(params)
+    return tmpl.render(params, enumerate=enumerate)
 
 
 def warn_about_deprecated_extra_vif_port_id():
     global warn_deprecated_extra_vif_port_id
     if not warn_deprecated_extra_vif_port_id:
         warn_deprecated_extra_vif_port_id = True
-        LOG.warning("Attaching VIF to a port/portgroup via "
-                    "extra['vif_port_id'] is deprecated and will not "
-                    "be supported in Pike release. API endpoint "
-                    "v1/nodes/<node>/vifs should be used instead.")
+        LOG.warning("Starting with API version 1.28, attaching/detaching VIF "
+                    "to/from a port or port group via extra['vif_port_id'] is "
+                    "deprecated and will not be supported starting in the "
+                    "Stein release. API endpoint v1/nodes/<node>/vifs should "
+                    "be used instead.")
+
+
+def parse_instance_info_capabilities(node):
+    """Parse the instance_info capabilities.
+
+    One way of having these capabilities set is via Nova, where the
+    capabilities are defined in the Flavor extra_spec and passed to
+    Ironic by the Nova Ironic driver.
+
+    NOTE: Although our API fully supports JSON fields, to maintain the
+    backward compatibility with Juno the Nova Ironic driver is sending
+    it as a string.
+
+    :param node: a single Node.
+    :raises: InvalidParameterValue if the capabilities string is not a
+             dictionary or is malformed.
+    :returns: A dictionary with the capabilities if found, otherwise an
+              empty dictionary.
+    """
+
+    def parse_error():
+        error_msg = (_('Error parsing capabilities from Node %s instance_info '
+                       'field. A dictionary or a "jsonified" dictionary is '
+                       'expected.') % node.uuid)
+        raise exception.InvalidParameterValue(error_msg)
+
+    capabilities = node.instance_info.get('capabilities', {})
+    if isinstance(capabilities, six.string_types):
+        try:
+            capabilities = jsonutils.loads(capabilities)
+        except (ValueError, TypeError):
+            parse_error()
+
+    if not isinstance(capabilities, dict):
+        parse_error()
+
+    return capabilities
