@@ -16,8 +16,10 @@ from oslo_config import cfg
 from oslo_log import log
 from oslo_service import loopingcall
 from oslo_utils import excutils
+import six
 
 from ironic.common import exception
+from ironic.common import faults
 from ironic.common.i18n import _
 from ironic.common import network
 from ironic.common import states
@@ -33,9 +35,10 @@ CLEANING_INTERFACE_PRIORITY = {
     # by which interface is implementing the clean step. The clean step of the
     # interface with the highest value here, will be executed first in that
     # case.
-    'power': 4,
-    'management': 3,
-    'deploy': 2,
+    'power': 5,
+    'management': 4,
+    'deploy': 3,
+    'bios': 2,
     'raid': 1,
 }
 
@@ -44,9 +47,6 @@ CLEANING_INTERFACE_PRIORITY = {
 def node_set_boot_device(task, device, persistent=False):
     """Set the boot device for a node.
 
-    Sets the boot device for a node if the node's driver interface
-    contains a 'management' interface.
-
     If the node that the boot device change is being requested for
     is in ADOPTING state, the boot device will not be set as that
     change could potentially result in the future running state of
@@ -60,12 +60,72 @@ def node_set_boot_device(task, device, persistent=False):
         ManagementInterface fails.
 
     """
-    if getattr(task.driver, 'management', None):
-        task.driver.management.validate(task)
-        if task.node.provision_state != states.ADOPTING:
-            task.driver.management.set_boot_device(task,
-                                                   device=device,
-                                                   persistent=persistent)
+    task.driver.management.validate(task)
+    if task.node.provision_state != states.ADOPTING:
+        task.driver.management.set_boot_device(task,
+                                               device=device,
+                                               persistent=persistent)
+
+
+def node_get_boot_mode(task):
+    """Read currently set boot mode from a node.
+
+    Reads the boot mode for a node. If boot mode can't be discovered,
+    `None` is returned.
+
+    :param task: a TaskManager instance.
+    :raises: DriverOperationError or its derivative in case
+             of driver runtime error.
+    :raises: UnsupportedDriverExtension if current driver does not have
+             management interface or `get_boot_mode()` method is
+             not supported.
+    :returns: Boot mode. One of :mod:`ironic.common.boot_mode` or `None`
+        if boot mode can't be discovered
+    """
+    task.driver.management.validate(task)
+    return task.driver.management.get_boot_mode(task)
+
+
+# TODO(ietingof): remove `Sets the boot mode...` from the docstring
+# once classic drivers are gone
+@task_manager.require_exclusive_lock
+def node_set_boot_mode(task, mode):
+    """Set the boot mode for a node.
+
+    Sets the boot mode for a node if the node's driver interface
+    contains a 'management' interface.
+
+    If the node that the boot mode change is being requested for
+    is in ADOPTING state, the boot mode will not be set as that
+    change could potentially result in the future running state of
+    an adopted node being modified erroneously.
+
+    :param task: a TaskManager instance.
+    :param mode: Boot mode. Values are one of
+        :mod:`ironic.common.boot_modes`
+    :raises: InvalidParameterValue if the validation of the
+             ManagementInterface fails.
+    :raises: DriverOperationError or its derivative in case
+             of driver runtime error.
+    :raises: UnsupportedDriverExtension if current driver does not have
+             vendor interface or method is unsupported.
+    """
+    if task.node.provision_state == states.ADOPTING:
+        return
+
+    task.driver.management.validate(task)
+
+    boot_modes = task.driver.management.get_supported_boot_modes(task)
+
+    if mode not in boot_modes:
+        msg = _("Unsupported boot mode %(mode)s specified for "
+                "node %(node_id)s. Supported boot modes are: "
+                "%(modes)s") % {'mode': mode,
+                                'modes': ', '.join(boot_modes),
+                                'node_id': task.node.uuid}
+        raise exception.InvalidParameterValue(msg)
+
+    task.driver.management.set_boot_mode(task, mode=mode)
 
 
 def node_wait_for_power_state(task, new_state, timeout=None):
@@ -220,8 +280,8 @@ def node_power_action(task, new_state, timeout=None):
 
     # take power action
     try:
-        if (target_state == states.POWER_ON and
-                node.provision_state == states.ACTIVE):
+        if (target_state == states.POWER_ON
+                and node.provision_state == states.ACTIVE):
             task.driver.storage.attach_volumes(task)
 
         if new_state != states.REBOOT:
@@ -259,8 +319,8 @@ def node_power_action(task, new_state, timeout=None):
                   'new_state': new_state})
         # NOTE(TheJulia): Similarly to power-on, when we power-off
         # a node, we should detach any volume attachments.
-        if (target_state == states.POWER_OFF and
-                node.provision_state == states.ACTIVE):
+        if (target_state == states.POWER_OFF
+                and node.provision_state == states.ACTIVE):
             try:
                 task.driver.storage.detach_volumes(task)
             except exception.StorageError as e:
@@ -338,10 +398,8 @@ def cleanup_cleanwait_timeout(task):
                     "check if the ramdisk responsible for the cleaning is "
                     "running on the node. Failed on step %(step)s.") %
                   {'step': task.node.clean_step})
-    # NOTE(rloo): this is called from the periodic task for cleanwait timeouts,
-    # via the task manager's process_event(). The node has already been moved
-    # to CLEANFAIL, so the error handler doesn't need to set the fail state.
-    cleaning_error_handler(task, msg=last_error, set_fail_state=False)
+    cleaning_error_handler(task, msg=last_error,
+                           set_fail_state=True)
 
 
 def cleaning_error_handler(task, msg, tear_down_cleaning=True,
@@ -375,9 +433,10 @@ def cleaning_error_handler(task, msg, tear_down_cleaning=True,
     node.last_error = msg
     node.maintenance = True
     node.maintenance_reason = msg
+    node.fault = faults.CLEAN_FAILURE
     node.save()
 
-    if set_fail_state and node.provision_state != states.CLEANFAIL:
+    if set_fail_state:
         target_state = states.MANAGEABLE if manual_clean else None
         task.process_event('fail', target_state=target_state)
 
@@ -749,3 +808,48 @@ def remove_node_rescue_password(node, save=True):
         node.instance_info = instance_info
         if save:
             node.save()
+
+
+def validate_instance_info_traits(node):
+    """Validate traits in instance_info.
+
+    All traits in instance_info must also exist as node traits.
+
+    :param node: an Ironic node object.
+    :raises: InvalidParameterValue if the instance traits are badly formatted,
+        or contain traits that are not set on the node.
+    """
+
+    def invalid():
+        err = (_("Error parsing traits from Node %(node)s instance_info "
+                 "field. A list of strings is expected.")
+               % {"node": node.uuid})
+        raise exception.InvalidParameterValue(err)
+
+    if not node.instance_info.get('traits'):
+        return
+    instance_traits = node.instance_info['traits']
+    if not isinstance(instance_traits, list):
+        invalid()
+    if not all(isinstance(t, six.string_types) for t in instance_traits):
+        invalid()
+
+    node_traits = node.traits.get_trait_names()
+    missing = set(instance_traits) - set(node_traits)
+    if missing:
+        err = (_("Cannot specify instance traits that are not also set on the "
+                 "node. Node %(node)s is missing traits %(traits)s") %
+               {"node": node.uuid, "traits": ", ".join(missing)})
+        raise exception.InvalidParameterValue(err)
+
+
+def notify_conductor_resume_clean(task):
+    LOG.debug('Sending RPC to conductor to resume cleaning for node %s',
+              task.node.uuid)
+    from ironic.conductor import rpcapi
+    uuid = task.node.uuid
+    rpc = rpcapi.ConductorAPI()
+    topic = rpc.get_topic_for(task.node)
+    # Need to release the lock to let the conductor take it
+    task.release_resources()
+    rpc.continue_node_clean(task.context, uuid, topic=topic)
