@@ -52,11 +52,25 @@ _DEFAULT_IFACE_HELP = _('Default {0} interface to be used for nodes that '
                         'be found by enumerating the '
                         '"ironic.hardware.interfaces.{0}" entrypoint.')
 
+# TODO(zshi) Remove this in BIOS API patch.
+_ENABLED_IFACE_HELP_FOR_BIOS = (_ENABLED_IFACE_HELP +
+                                _(' This option is part of BIOS feature '
+                                  'work, which is not currently exposed to '
+                                  'users.'))
+
+# TODO(zshi) Remove this in BIOS API patch.
+_DEFAULT_IFACE_HELP_FOR_BIOS = (_DEFAULT_IFACE_HELP +
+                                _(' This option is part of BIOS feature '
+                                  'work, which is not currently exposed to '
+                                  'users.'))
+
 api_opts = [
     cfg.StrOpt(
         'auth_strategy',
         default='keystone',
-        choices=['noauth', 'keystone'],
+        choices=[('noauth', _('no authentication')),
+                 ('keystone', _('use the Identity service for '
+                                'authentication'))],
         help=_('Authentication strategy used by ironic-api. "noauth" should '
                'not be used in a production environment because all '
                'authentication will be disabled.')),
@@ -76,16 +90,10 @@ api_opts = [
 
 driver_opts = [
     cfg.ListOpt('enabled_drivers',
-                default=['pxe_ipmitool'],
-                help=_('Specify the list of drivers to load during service '
-                       'initialization. Missing drivers, or drivers which '
-                       'fail to initialize, will prevent the conductor '
-                       'service from starting. The option default is a '
-                       'recommended set of production-oriented drivers. A '
-                       'complete list of drivers present on your system may '
-                       'be found by enumerating the "ironic.drivers" '
-                       'entrypoint. An example may be found in the '
-                       'developer documentation online.'),
+                default=[],
+                help=_('This option is left for a start up check only. '
+                       'Any non-empty value will prevent the conductor '
+                       'from starting.'),
                 deprecated_for_removal=True,
                 deprecated_reason=_('Hardware types should be used instead '
                                     'of classic drivers. They are enabled '
@@ -101,6 +109,11 @@ driver_opts = [
                        'A complete list of hardware types present on your '
                        'system may be found by enumerating the '
                        '"ironic.hardware.types" entrypoint.')),
+    cfg.ListOpt('enabled_bios_interfaces',
+                default=['no-bios'],
+                help=_ENABLED_IFACE_HELP_FOR_BIOS.format('bios')),
+    cfg.StrOpt('default_bios_interface',
+               help=_DEFAULT_IFACE_HELP_FOR_BIOS.format('bios')),
     cfg.ListOpt('enabled_boot_interfaces',
                 default=['pxe'],
                 help=_ENABLED_IFACE_HELP.format('boot')),
@@ -191,10 +204,8 @@ hash_opts = [
                       'and potentially allow the Ironic cluster to recover '
                       'more quickly if a conductor instance is terminated.')),
     cfg.IntOpt('hash_ring_reset_interval',
-               default=15,
-               help=_('Time (in seconds) after which the hash ring is '
-                      'considered outdated and is refreshed on the next '
-                      'access.')),
+               default=180,
+               help=_('Interval (in seconds) between hash ring resets.')),
 ]
 
 image_opts = [
@@ -242,7 +253,11 @@ netconf_opts = [
 # default for options without set defaults that aren't required.
 notification_opts = [
     cfg.StrOpt('notification_level',
-               choices=['debug', 'info', 'warning', 'error', 'critical'],
+               choices=[('debug', _('"debug" level')),
+                        ('info', _('"info" level')),
+                        ('warning', _('"warning" level')),
+                        ('error', _('"error" level')),
+                        ('critical', _('"critical" level'))],
                help=_('Specifies the minimum level for which to send '
                       'notifications. If not set, no notifications will '
                       'be sent. The default is for this option to be unset.'))
@@ -283,8 +298,8 @@ service_opts = [
                       'an AMQP key, and if using ZeroMQ, a valid '
                       'hostname, FQDN, or IP address.')),
     cfg.StrOpt('pin_release_version',
-               choices=versions.RELEASE_VERSIONS,
-               # TODO(xek): mutable=True,
+               choices=versions.RELEASE_VERSIONS_DESCS,
+               mutable=True,
                help=_('Used for rolling upgrades. Setting this option '
                       'downgrades (or pins) the Bare Metal API, '
                       'the internal ironic RPC communication, and '
