@@ -16,6 +16,7 @@
 
 from oslo_config import cfg
 
+from ironic.common import boot_modes
 from ironic.common.i18n import _
 
 
@@ -26,6 +27,10 @@ opts = [
     cfg.StrOpt('http_root',
                default='/httpboot',
                help=_("ironic-conductor node's HTTP root path.")),
+    cfg.BoolOpt('enable_ata_secure_erase',
+                default=True,
+                help=_('Whether to support the use of ATA Secure Erase '
+                       'during the cleaning process. Defaults to True.')),
     cfg.IntOpt('erase_devices_priority',
                help=_('Priority to run in-band erase devices via the Ironic '
                       'Python Agent ramdisk. If unset, will use the priority '
@@ -66,12 +71,24 @@ opts = [
                 help=_('Whether to power off a node after deploy failure. '
                        'Defaults to True.')),
     cfg.StrOpt('default_boot_option',
-               choices=['netboot', 'local'],
+               choices=[('netboot', _('boot from a network')),
+                        ('local', _('local boot'))],
                help=_('Default boot option to use when no boot option is '
                       'requested in node\'s driver_info. Currently the '
                       'default is "netboot", but it will be changed to '
                       '"local" in the future. It is recommended to set '
                       'an explicit value for this option.')),
+    cfg.StrOpt('default_boot_mode',
+               choices=[(boot_modes.UEFI, _('UEFI boot mode')),
+                        (boot_modes.LEGACY_BIOS, _('Legacy BIOS boot mode'))],
+               default=boot_modes.LEGACY_BIOS,
+               help=_('Default boot mode to use when no boot mode is '
+                      'requested in node\'s driver_info, capabilities or '
+                      'in the `instance_info` configuration. Currently the '
+                      'default boot mode is "%(bios)s". This option only '
+                      'has effect when management interface supports boot '
+                      'mode management') % {
+                          'bios': boot_modes.LEGACY_BIOS}),
     cfg.BoolOpt('configdrive_use_object_store',
                 default=False,
                 deprecated_group='conductor',
@@ -83,7 +100,8 @@ opts = [
                default='swift',
                deprecated_group='glance',
                deprecated_name='temp_url_endpoint_type',
-               choices=['swift', 'radosgw'],
+               choices=[('swift', _('use Object Storage service')),
+                        ('radosgw', _('use RADOS object store'))],
                help=_('Type of object store endpoint type to be '
                       'used as a backend')),
 ]
