@@ -25,7 +25,9 @@ opts = [
                help=_('IP of remote image server')),
     cfg.StrOpt('remote_image_share_type',
                default='CIFS',
-               choices=['CIFS', 'NFS'],
+               choices=[('CIFS', _('CIFS (Common Internet File System) '
+                                   'protocol')),
+                        ('NFS', _('NFS (Network File System) protocol'))],
                ignore_case=True,
                help=_('Share type of virtual media')),
     cfg.StrOpt('remote_image_share_name',
@@ -40,11 +42,13 @@ opts = [
                help=_('Domain name of remote_image_user_name')),
     cfg.PortOpt('port',
                 default=443,
-                choices=[443, 80],
+                choices=[(443, _('port 443')),
+                         (80, _('port 80'))],
                 help=_('Port to be used for iRMC operations')),
     cfg.StrOpt('auth_method',
                default='basic',
-               choices=['basic', 'digest'],
+               choices=[('basic', _('Basic authentication')),
+                        ('digest', _('Digest authentication'))],
                help=_('Authentication method to be used for iRMC '
                       'operations')),
     cfg.IntOpt('client_timeout',
@@ -52,11 +56,15 @@ opts = [
                help=_('Timeout (in seconds) for iRMC operations')),
     cfg.StrOpt('sensor_method',
                default='ipmitool',
-               choices=['ipmitool', 'scci'],
+               choices=[('ipmitool', _('IPMItool')),
+                        ('scci', _('Fujitsu SCCI (ServerView Common Command '
+                                   'Interface)'))],
                help=_('Sensor data retrieval method.')),
     cfg.StrOpt('snmp_version',
                default='v2c',
-               choices=['v1', 'v2c', 'v3'],
+               choices=[('v1', _('SNMPv1')),
+                        ('v2c', _('SNMPv2c')),
+                        ('v3', _('SNMPv3'))],
                help=_('SNMP protocol version')),
     cfg.PortOpt('snmp_port',
                 default=161,
