@@ -138,7 +138,7 @@ class Connection(object):
                          'instance_uuid': None,
                          'power_state': states.POWER_OFF,
                          'provision_state': states.AVAILABLE,
-                         'driver': 'pxe_ipmitool',
+                         'driver': 'ipmi',
                          'driver_info': { ... },
                          'properties': { ... },
                          'extra': { ... },
@@ -517,20 +517,6 @@ class Connection(object):
         :raises: ConductorNotFound
         """
 
-    @abc.abstractmethod
-    def get_active_driver_dict(self, interval):
-        """Retrieve drivers for the registered and active conductors.
-
-        :param interval: Seconds since last check-in of a conductor.
-        :returns: A dict which maps driver names to the set of hosts
-                  which support them. For example:
-
-                  ::
-
-                    {driverA: set([host1, host2]),
-                     driverB: set([host2, host3])}
-        """
-
     @abc.abstractmethod
     def get_active_hardware_type_dict(self):
         """Retrieve hardware types for the registered and active conductors.
@@ -887,6 +873,16 @@ class Connection(object):
                  ident does not exist.
         """
 
+    @abc.abstractmethod
+    def get_not_versions(self, model_name, versions):
+        """Returns objects with versions that are not the specified versions.
+
+        :param model_name: the name of the model (class) of desired objects
+        :param versions: list of versions of objects not to be returned
+        :returns: list of the DB objects
+        :raises: IronicException if there is no class associated with the name
+        """
+
     @abc.abstractmethod
     def check_versions(self):
         """Checks the whole database for incompatible objects.
@@ -899,65 +895,6 @@ class Connection(object):
                   False otherwise.
         """
 
-    @abc.abstractmethod
-    def backfill_version_column(self, max_count):
-        """Backfill the Conductor version column with Pike version.
-
-        The version column was added to all the resource tables in the Pike
-        release (via 'ironic-dbsync upgrade'). After upgrading (from Ocata to
-        Pike), the 'ironic-dbsync online_data_migrations' command would have
-        populated (backfilled) the version column for all objects.
-
-        Unfortunately, in the Pike release, we forgot to set the value for the
-        conductor's version column. For the Queens release, we are setting
-        the conductor version, however, we still need to backfill in case new
-        conductors were added between the time the operator ran Pike's
-        'ironic-dbsync online_data_migrations' and their upgrade to Queens.
-        The version used will be the conductor object version from the Pike
-        release.
-
-        :param max_count: The maximum number of objects to migrate. Must be
-                          >= 0. If zero, all the objects will be migrated.
-        :returns: A 2-tuple, 1. the total number of objects that need to be
-                  migrated (at the beginning of this call) and 2. the number
-                  of migrated objects.
-        """
-        # TODO(rloo) Delete this in Rocky cycle.
-
-    @abc.abstractmethod
-    def migrate_to_hardware_types(self, context, max_count,
-                                  reset_unsupported_interfaces=False):
-        """Migrate nodes from classic drivers to hardware types.
-
-        Go through all nodes with a classic driver and try to migrate them to a
-        corresponding hardware type and a correct set of hardware interfaces.
-
-        :param context: the admin context
-        :param max_count: The maximum number of objects to migrate. Must be
-                          >= 0. If zero, all the objects will be migrated.
-        :param reset_unsupported_interfaces: whether to reset unsupported
-            optional interfaces to their no-XXX versions.
-        :returns: A 2-tuple, 1. the total number of objects that need to be
-                  migrated (at the beginning of this call) and 2. the number
-                  of migrated objects.
-        """
-        # TODO(dtantsur) Delete this in Rocky cycle.
-
-    @abc.abstractmethod
-    def update_to_latest_versions(self, context, max_count):
-        """Updates objects to their latest known versions.
-
-        This scans all the tables and for objects that are not in their latest
-        version, updates them to that version.
-
-        :param context: the admin context
-        :param max_count: The maximum number of objects to migrate. Must be
-                          >= 0. If zero, all the objects will be migrated.
-        :returns: A 2-tuple, 1. the total number of objects that need to be
-                  migrated (at the beginning of this call) and 2. the number
-                  of migrated objects.
-        """
-
     @abc.abstractmethod
     def set_node_traits(self, node_id, traits, version):
         """Replace all of the node traits with specified list of traits.
@@ -1025,3 +962,86 @@ class Connection(object):
         :returns: True if the trait exists otherwise False.
         :raises: NodeNotFound if the node is not found.
         """
+
+    @abc.abstractmethod
+    def create_bios_setting_list(self, node_id, settings, version):
+        """Create a list of BIOSSetting records for a given node.
+
+        :param node_id: The node id.
+        :param settings: A list of BIOS Settings to be created.
+
+                       ::
+
+                        [
+                          {
+                           'name': String,
+                           'value': String,
+                          },
+                          {
+                           'name': String,
+                           'value': String,
+                          },
+                          ...
+                        ]
+        :param version: the version of the object.BIOSSetting.
+        :returns: A list of BIOSSetting object.
+        :raises: NodeNotFound if the node is not found.
+        :raises: BIOSSettingAlreadyExists if any of the setting records
+            already exists.
+        """
+
+    @abc.abstractmethod
+    def update_bios_setting_list(self, node_id, settings, version):
+        """Update a list of BIOSSetting records.
+
+        :param node_id: The node id.
+        :param settings: A list of BIOS Settings to be updated.
+
+                       ::
+
+                        [
+                          {
+                           'name': String,
+                           'value': String,
+                          },
+                          {
+                           'name': String,
+                           'value': String,
+                          },
+                          ...
+                        ]
+        :param version: the version of the object.BIOSSetting.
+        :returns: A list of BIOSSetting objects.
+        :raises: NodeNotFound if the node is not found.
+        :raises: BIOSSettingNotFound if any of the settings is not found.
+        """
+
+    @abc.abstractmethod
+    def delete_bios_setting_list(self, node_id, names):
+        """Delete a list of BIOS settings.
+
+        :param node_id: The node id.
+        :param names: List of BIOS setting names to be deleted.
+        :raises: NodeNotFound if the node is not found.
+        :raises: BIOSSettingNotFound if any of BIOS setting name is not found.
+        """
+
+    @abc.abstractmethod
+    def get_bios_setting(self, node_id, name):
+        """Retrieve BIOS setting value.
+
+        :param node_id: The node id.
+        :param name: String containing name of BIOS setting to be retrieved.
+        :returns: The BIOSSetting object.
+        :raises: NodeNotFound if the node is not found.
+        :raises: BIOSSettingNotFound if the BIOS setting is not found.
+        """
+
+    @abc.abstractmethod
+    def get_bios_setting_list(self, node_id):
+        """Retrieve BIOS settings of a given node.
+
+        :param node_id: The node id.
+        :returns: A list of BIOSSetting objects.
+        :raises: NodeNotFound if the node is not found.
+        """
