@@ -33,7 +33,6 @@ from sqlalchemy.orm.exc import NoResultFound, MultipleResultsFound
 from sqlalchemy.orm import joinedload
 from sqlalchemy import sql
 
-from ironic.common import driver_factory
 from ironic.common import exception
 from ironic.common.i18n import _
 from ironic.common import profiler
@@ -77,7 +76,7 @@ def _wrap_session(session):
 
 
 def _get_node_query_with_all():
-    """Return a query object for the Node model joined with all relevant fields.
+    """Return a query object for the Node joined with all relevant fields.
 
     :returns: a query object.
     """
@@ -224,14 +223,15 @@ class Connection(api.Connection):
                              'resource_class', 'provision_state', 'uuid', 'id',
                              'chassis_uuid', 'associated', 'reserved',
                              'reserved_by_any_of', 'provisioned_before',
-                             'inspection_started_before'}
+                             'inspection_started_before', 'fault'}
         unsupported_filters = set(filters).difference(supported_filters)
         if unsupported_filters:
             msg = _("SqlAlchemy API does not support "
                     "filtering by %s") % ', '.join(unsupported_filters)
             raise ValueError(msg)
         for field in ['console_enabled', 'maintenance', 'driver',
-                      'resource_class', 'provision_state', 'uuid', 'id']:
+                      'resource_class', 'provision_state', 'uuid', 'id',
+                      'fault']:
             if field in filters:
                 query = query.filter_by(**{field: filters[field]})
         if 'chassis_uuid' in filters:
@@ -253,12 +253,13 @@ class Connection(api.Connection):
             query = query.filter(models.Node.reservation.in_(
                 filters['reserved_by_any_of']))
         if 'provisioned_before' in filters:
-            limit = (timeutils.utcnow() -
-                     datetime.timedelta(seconds=filters['provisioned_before']))
+            limit = (timeutils.utcnow()
+                     - datetime.timedelta(
+                         seconds=filters['provisioned_before']))
             query = query.filter(models.Node.provision_updated_at < limit)
         if 'inspection_started_before' in filters:
-            limit = ((timeutils.utcnow()) -
-                     (datetime.timedelta(
+            limit = ((timeutils.utcnow())
+                     - (datetime.timedelta(
                          seconds=filters['inspection_started_before'])))
             query = query.filter(models.Node.inspection_started_at < limit)
 
@@ -445,6 +446,11 @@ class Connection(api.Connection):
                 models.VolumeTarget).filter_by(node_id=node_id)
             volume_target_query.delete()
 
+            # delete all bios attached to the node
+            bios_settings_query = model_query(
+                models.BIOSSetting).filter_by(node_id=node_id)
+            bios_settings_query.delete()
+
             query.delete()
 
     def update_node(self, node_id, values):
@@ -484,12 +490,12 @@ class Connection(api.Connection):
                 if values['provision_state'] == states.INSPECTING:
                     values['inspection_started_at'] = timeutils.utcnow()
                     values['inspection_finished_at'] = None
-                elif (ref.provision_state == states.INSPECTING and
-                      values['provision_state'] == states.MANAGEABLE):
+                elif (ref.provision_state == states.INSPECTING
+                      and values['provision_state'] == states.MANAGEABLE):
                     values['inspection_finished_at'] = timeutils.utcnow()
                     values['inspection_started_at'] = None
-                elif (ref.provision_state == states.INSPECTING and
-                      values['provision_state'] == states.INSPECTFAIL):
+                elif (ref.provision_state == states.INSPECTING
+                      and values['provision_state'] == states.INSPECTFAIL):
                     values['inspection_started_at'] = None
 
             ref.update(values)
@@ -849,17 +855,6 @@ class Connection(api.Connection):
                 'powering process, their power state can be incorrect: '
                 '%(nodes)s', {'nodes': nodes})
 
-    def get_active_driver_dict(self, interval=None):
-        query = model_query(models.Conductor)
-        result = _filter_active_conductors(query, interval=interval)
-
-        # build mapping of drivers to the set of hosts which support them
-        d2c = collections.defaultdict(set)
-        for row in result:
-            for driver in row['drivers']:
-                d2c[driver].add(row['hostname'])
-        return d2c
-
     def get_active_hardware_type_dict(self):
         query = (model_query(models.ConductorHardwareInterfaces,
                              models.Conductor)
@@ -1168,6 +1163,29 @@ class Connection(api.Connection):
             if count == 0:
                 raise exception.VolumeTargetNotFound(target=ident)
 
+    def get_not_versions(self, model_name, versions):
+        """Returns objects with versions that are not the specified versions.
+
+        This returns objects with versions that are not the specified versions.
+        Objects with null versions (there shouldn't be any) are also returned.
+
+        :param model_name: the name of the model (class) of desired objects
+        :param versions: list of versions of objects not to be returned
+        :returns: list of the DB objects
+        :raises: IronicException if there is no class associated with the name
+        """
+        if not versions:
+            return []
+
+        model = models.get_class(model_name)
+
+        # NOTE(rloo): .notin_ does not handle null:
+        # http://docs.sqlalchemy.org/en/latest/core/sqlelement.html#sqlalchemy.sql.operators.ColumnOperators.notin_
+        query = model_query(model).filter(
+            sql.or_(model.version == sql.null(),
+                    model.version.notin_(versions)))
+        return query.all()
+
     def check_versions(self):
         """Checks the whole database for incompatible objects.
 
@@ -1186,17 +1204,6 @@ class Connection(api.Connection):
                 if not supported_versions:
                     continue
 
-                # TODO(rloo). Because we forgot to set the version of
-                #     conductors in Pike, we allow the Conductor version to
-                #     be null now (Queens). In Rocky, it cannot be null,
-                #     and this 'if...' can be deleted then.
-                if model.__name__ == 'Conductor':
-                    query = model_query(model.version).filter(
-                        model.version.notin_(supported_versions))
-                    if query.count():
-                        return False
-                    continue
-
                 # NOTE(rloo): we use model.version, not model, because we
                 #             know that the object has a 'version' column
                 #             but we don't know whether the entire object is
@@ -1210,254 +1217,6 @@ class Connection(api.Connection):
                     return False
         return True
 
-    @oslo_db_api.retry_on_deadlock
-    def backfill_version_column(self, context, max_count):
-        """Backfill the Conductor version column with Pike version.
-
-        The version column was added to all the resource tables in the Pike
-        release (via 'ironic-dbsync upgrade'). After upgrading (from Ocata to
-        Pike), the 'ironic-dbsync online_data_migrations' command would have
-        populated (backfilled) the version column for all objects.
-
-        Unfortunately, in the Pike release, we forgot to set the value for the
-        conductor's version column. For the Queens release, we are setting
-        the conductor version, however, we still need to backfill in case new
-        conductors were added between the time the operator ran Pike's
-        'ironic-dbsync online_data_migrations' and their upgrade to Queens.
-        The version used will be the conductor object version from the Pike
-        release.
-
-        :param context: the admin context (not used)
-        :param max_count: The maximum number of objects to migrate. Must be
-                          >= 0. If zero, all the objects will be migrated.
-        :returns: A 2-tuple, 1. the total number of objects that need to be
-                  migrated (at the beginning of this call) and 2. the number
-                  of migrated objects.
-        """
-        # TODO(rloo): Delete this in Rocky cycle.
-        prior_release = 'pike'
-        mapping = release_mappings.RELEASE_MAPPING[prior_release]['objects']
-        total_to_migrate = 0
-        total_migrated = 0
-
-        # backfill only the Conductor.
-        # NOTE(rloo) This code was used in Pike to backfill all the objects.
-        # To make it easier to review, etc., we are re-using that code with
-        # minimal code changes to only backfill the 'Conductor' object.
-        sql_models = [models.Conductor]
-        for model in sql_models:
-            query = model_query(model).filter(model.version.is_(None))
-            total_to_migrate += query.count()
-
-        if not total_to_migrate:
-            return total_to_migrate, 0
-
-        # NOTE(xek): Each of these operations happen in different transactions.
-        # This is to ensure a minimal load on the database, but at the same
-        # time it can cause an inconsistency in the amount of total and
-        # migrated objects returned (total could be > migrated). This is
-        # because some objects may have already migrated or been deleted from
-        # the database between the time the total was computed (above) to the
-        # time we do the updating (below).
-        #
-        # By the time this script is run, only the new release version is
-        # running, so the impact of this error will be minimal - e.g. the
-        # operator will run this script more than once to ensure that all
-        # data have been migrated.
-
-        # If max_count is zero, we want to migrate all the objects.
-        max_to_migrate = max_count or total_to_migrate
-
-        for model in sql_models:
-            num_migrated = 0
-            with _session_for_write():
-                query = model_query(model).filter(model.version.is_(None))
-                # NOTE(rloo) Caution here; after doing query.count(), it is
-                #            possible that the value is different in the
-                #            next invocation of the query.
-                if max_to_migrate < query.count():
-                    # Only want to update max_to_migrate objects; cannot use
-                    # sql's limit(), so we generate a new query with
-                    # max_to_migrate objects.
-                    ids = []
-                    for obj in query.slice(0, max_to_migrate):
-                        ids.append(obj['id'])
-                    num_migrated = (
-                        model_query(model).
-                        filter(sql.and_(model.id.in_(ids),
-                                        model.version.is_(None))).
-                        update({model.version: mapping[model.__name__][0]},
-                               synchronize_session=False))
-                else:
-                    num_migrated = (
-                        model_query(model).
-                        filter(model.version.is_(None)).
-                        update({model.version: mapping[model.__name__][0]},
-                               synchronize_session=False))
-
-            total_migrated += num_migrated
-            max_to_migrate -= num_migrated
-            if max_to_migrate <= 0:
-                break
-
-        return total_to_migrate, total_migrated
-
-    @oslo_db_api.retry_on_deadlock
-    def migrate_to_hardware_types(self, context, max_count,
-                                  reset_unsupported_interfaces=False):
-        """Migrate nodes from classic drivers to hardware types.
-
-        Go through all nodes with a classic driver and try to migrate them to
-        a corresponding hardware type and a correct set of hardware interfaces.
-
-        If migration is not possible for any reason (e.g. the target hardware
-        type is not enabled), the nodes are skipped. An operator is expected to
-        correct the configuration and either rerun online_data_migration or
-        migrate the nodes manually.
-
-        :param context: the admin context (not used)
-        :param max_count: The maximum number of objects to migrate. Must be
-                          >= 0. If zero, all the objects will be migrated.
-        :param reset_unsupported_interfaces: whether to reset unsupported
-            optional interfaces to their no-XXX versions.
-        :returns: A 2-tuple, 1. the total number of objects that need to be
-                  migrated (at the beginning of this call) and 2. the number
-                  of migrated objects.
-        """
-        reset_unsupported_interfaces = strutils.bool_from_string(
-            reset_unsupported_interfaces, strict=True)
-
-        drivers = driver_factory.classic_drivers_to_migrate()
-
-        total_to_migrate = (model_query(models.Node)
-                            .filter(models.Node.driver.in_(list(drivers)))
-                            .count())
-
-        total_migrated = 0
-        for driver, driver_cls in drivers.items():
-            if max_count and total_migrated >= max_count:
-                return total_to_migrate, total_migrated
-
-            # UPDATE with LIMIT seems to be a MySQL-only feature, so first
-            # fetch the required number of Node IDs, then update them.
-            query = model_query(models.Node.id).filter_by(driver=driver)
-            if max_count:
-                query = query.limit(max_count - total_migrated)
-            ids = [obj.id for obj in query]
-            if not ids:
-                continue
-
-            delta = driver_factory.calculate_migration_delta(
-                driver, driver_cls, reset_unsupported_interfaces)
-            if delta is None:
-                # NOTE(dtantsur): mark unsupported nodes as migrated. Otherwise
-                # calling online_data_migration without --max-count will result
-                # in a infinite loop.
-                total_migrated += len(ids)
-                continue
-
-            # UPDATE with LIMIT seems to be a MySQL-only feature, so first
-            # fetch the required number of Node IDs, then update them.
-            query = model_query(models.Node.id).filter_by(driver=driver)
-            if max_count:
-                query = query.limit(max_count - total_migrated)
-            ids = [obj.id for obj in query]
-            if not ids:
-                LOG.debug('No nodes with driver %s', driver)
-                continue
-
-            LOG.info('Migrating nodes with driver %(drv)s to %(delta)s',
-                     {'drv': driver, 'delta': delta})
-
-            with _session_for_write():
-                num_migrated = (model_query(models.Node)
-                                .filter_by(driver=driver)
-                                .filter(models.Node.id.in_(ids))
-                                .update(delta, synchronize_session=False))
-                total_migrated += num_migrated
-
-        return total_to_migrate, total_migrated
-
-    @oslo_db_api.retry_on_deadlock
-    def update_to_latest_versions(self, context, max_count):
-        """Updates objects to their latest known versions.
-
-        This scans all the tables and for objects that are not in their latest
-        version, updates them to that version.
-
-        :param context: the admin context
-        :param max_count: The maximum number of objects to migrate. Must be
-                          >= 0. If zero, all the objects will be migrated.
-        :returns: A 2-tuple, 1. the total number of objects that need to be
-                  migrated (at the beginning of this call) and 2. the number
-                  of migrated objects.
-        """
-        # NOTE(rloo): 'master' has the most recent (latest) versions.
-        mapping = release_mappings.RELEASE_MAPPING['master']['objects']
-        total_to_migrate = 0
-        total_migrated = 0
-
-        sql_models = [model for model in models.Base.__subclasses__()
-                      if model.__name__ in mapping]
-        for model in sql_models:
-            version = mapping[model.__name__][0]
-            query = model_query(model).filter(model.version != version)
-            total_to_migrate += query.count()
-
-        if not total_to_migrate:
-            return total_to_migrate, 0
-
-        # NOTE(xek): Each of these operations happen in different transactions.
-        # This is to ensure a minimal load on the database, but at the same
-        # time it can cause an inconsistency in the amount of total and
-        # migrated objects returned (total could be > migrated). This is
-        # because some objects may have already migrated or been deleted from
-        # the database between the time the total was computed (above) to the
-        # time we do the updating (below).
-        #
-        # By the time this script is run, only the new release version is
-        # running, so the impact of this error will be minimal - e.g. the
-        # operator will run this script more than once to ensure that all
-        # data have been migrated.
-
-        # If max_count is zero, we want to migrate all the objects.
-        max_to_migrate = max_count or total_to_migrate
-
-        for model in sql_models:
-            version = mapping[model.__name__][0]
-            num_migrated = 0
-            with _session_for_write():
-                query = model_query(model).filter(model.version != version)
-                # NOTE(rloo) Caution here; after doing query.count(), it is
-                #            possible that the value is different in the
-                #            next invocation of the query.
-                if max_to_migrate < query.count():
-                    # Only want to update max_to_migrate objects; cannot use
-                    # sql's limit(), so we generate a new query with
-                    # max_to_migrate objects.
-                    ids = []
-                    for obj in query.slice(0, max_to_migrate):
-                        ids.append(obj['id'])
-                    num_migrated = (
-                        model_query(model).
-                        filter(sql.and_(model.id.in_(ids),
-                                        model.version != version)).
-                        update({model.version: version},
-                               synchronize_session=False))
-                else:
-                    num_migrated = (
-                        model_query(model).
-                        filter(model.version != version).
-                        update({model.version: version},
-                               synchronize_session=False))
-
-            total_migrated += num_migrated
-            max_to_migrate -= num_migrated
-            if max_to_migrate <= 0:
-                break
-
-        return total_to_migrate, total_migrated
-
     @staticmethod
     def _verify_max_traits_per_node(node_id, num_traits):
         """Verify that an operation would not exceed the per-node trait limit.
@@ -1543,3 +1302,73 @@ class Connection(api.Connection):
         q = model_query(
             models.NodeTrait).filter_by(node_id=node_id, trait=trait)
         return model_query(q.exists()).scalar()
+
+    @oslo_db_api.retry_on_deadlock
+    def create_bios_setting_list(self, node_id, settings, version):
+        self._check_node_exists(node_id)
+        bios_settings = []
+        with _session_for_write() as session:
+            try:
+                for setting in settings:
+                    bios_setting = models.BIOSSetting(
+                        node_id=node_id,
+                        name=setting['name'],
+                        value=setting['value'],
+                        version=version)
+                    bios_settings.append(bios_setting)
+                    session.add(bios_setting)
+                session.flush()
+            except db_exc.DBDuplicateEntry:
+                raise exception.BIOSSettingAlreadyExists(
+                    node=node_id, name=setting['name'])
+        return bios_settings
+
+    @oslo_db_api.retry_on_deadlock
+    def update_bios_setting_list(self, node_id, settings, version):
+        self._check_node_exists(node_id)
+        bios_settings = []
+        with _session_for_write() as session:
+            try:
+                for setting in settings:
+                    query = model_query(models.BIOSSetting).filter_by(
+                        node_id=node_id, name=setting['name'])
+                    ref = query.one()
+                    ref.update({'value': setting['value'],
+                                'version': version})
+                    bios_settings.append(ref)
+                session.flush()
+            except NoResultFound:
+                raise exception.BIOSSettingNotFound(
+                    node=node_id, name=setting['name'])
+        return bios_settings
+
+    @oslo_db_api.retry_on_deadlock
+    def delete_bios_setting_list(self, node_id, names):
+        self._check_node_exists(node_id)
+        missing_bios_settings = []
+        with _session_for_write():
+            for name in names:
+                count = model_query(models.BIOSSetting).filter_by(
+                    node_id=node_id, name=name).delete()
+                if count == 0:
+                    missing_bios_settings.append(name)
+        if len(missing_bios_settings) > 0:
+            raise exception.BIOSSettingListNotFound(
+                node=node_id, names=','.join(missing_bios_settings))
+
+    def get_bios_setting(self, node_id, name):
+        self._check_node_exists(node_id)
+        query = model_query(models.BIOSSetting).filter_by(
+            node_id=node_id, name=name)
+        try:
+            ref = query.one()
+        except NoResultFound:
+            raise exception.BIOSSettingNotFound(node=node_id, name=name)
+        return ref
+
+    def get_bios_setting_list(self, node_id):
+        self._check_node_exists(node_id)
+        result = (model_query(models.BIOSSetting)
+                  .filter_by(node_id=node_id)
+                  .all())
+        return result
