@@ -30,6 +30,8 @@ from sqlalchemy import schema, String, Text
 from sqlalchemy.ext.declarative import declarative_base
 from sqlalchemy import orm
 
+from ironic.common import exception
+from ironic.common.i18n import _
 from ironic.conf import CONF
 
 _DEFAULT_SQL_CONNECTION = 'sqlite:///' + path.join('$state_path',
@@ -163,11 +165,13 @@ class Node(Base):
 
     maintenance = Column(Boolean, default=False)
     maintenance_reason = Column(Text, nullable=True)
+    fault = Column(String(255), nullable=True)
     console_enabled = Column(Boolean, default=False)
     inspection_finished_at = Column(DateTime, nullable=True)
     inspection_started_at = Column(DateTime, nullable=True)
     extra = Column(db_types.JsonEncodedDict)
 
+    bios_interface = Column(String(255), nullable=True)
     boot_interface = Column(String(255), nullable=True)
     console_interface = Column(String(255), nullable=True)
     deploy_interface = Column(String(255), nullable=True)
@@ -296,3 +300,29 @@ class NodeTrait(Base):
         primaryjoin='and_(NodeTrait.node_id == Node.id)',
         foreign_keys=node_id
     )
+
+
+class BIOSSetting(Base):
+    """Represents a bios setting of a bare metal node."""
+
+    __tablename__ = 'bios_settings'
+    __table_args__ = (table_args())
+    node_id = Column(Integer, ForeignKey('nodes.id'),
+                     primary_key=True, nullable=False)
+    name = Column(String(255), primary_key=True, nullable=False)
+    value = Column(Text, nullable=True)
+
+
+def get_class(model_name):
+    """Returns the model class with the specified name.
+
+    :param model_name: the name of the class
+    :returns: the class with the specified name
+    :raises: Exception if there is no class associated with the name
+    """
+    for model in Base.__subclasses__():
+        if model.__name__ == model_name:
+            return model
+
+    raise exception.IronicException(
+        _("Cannot find model with name: %s") % model_name)
