@@ -39,34 +39,29 @@ RAID_CONFIG_SCHEMA = os.path.join(os.path.dirname(__file__),
                                   'raid_config_schema.json')
 
 
-@six.add_metaclass(abc.ABCMeta)
-class BaseDriver(object):
-    """Base class for all drivers.
+class BareDriver(object):
+    """A bare driver object which will have interfaces attached later.
 
-    Defines the `core`, `standardized`, and `vendor-specific` interfaces for
-    drivers. Any loadable driver must implement all `core` interfaces.
-    Actual implementation may instantiate one or more classes, as long as
-    the interfaces are appropriate.
+    Any composable interfaces should be added as class attributes of this
+    class, as well as appended to core_interfaces or standard_interfaces here.
     """
 
-    supported = False
-    """Indicates if a driver is supported.
+    bios = None
+    """`Standard` attribute for BIOS related features.
 
-    This will be set to False for drivers which are untested in first- or
-    third-party CI, or in the process of being deprecated.
-
-    All classic drivers are now deprecated, and thus unsupported.
+    A reference to an instance of :class:BIOSInterface.
     """
 
-    # NOTE(jlvillal): These should be tuples to help prevent child classes from
-    # accidentally modifying the base class values.
-    core_interfaces = ('deploy', 'power')
-    standard_interfaces = ('boot', 'console', 'inspect', 'management', 'raid')
+    boot = None
+    """`Standard` attribute for boot related features.
 
-    power = None
-    """`Core` attribute for managing power state.
+    A reference to an instance of :class:BootInterface.
+    """
 
-    A reference to an instance of :class:PowerInterface.
+    console = None
+    """`Standard` attribute for managing console access.
+
+    A reference to an instance of :class:ConsoleInterface.
     """
 
     deploy = None
@@ -75,66 +70,71 @@ class BaseDriver(object):
     A reference to an instance of :class:DeployInterface.
     """
 
-    console = None
-    """`Standard` attribute for managing console access.
+    inspect = None
+    """`Standard` attribute for inspection related features.
 
-    A reference to an instance of :class:ConsoleInterface.
-    May be None, if unsupported by a driver.
-    """
-
-    rescue = None
-    """`Standard` attribute for accessing rescue features.
-
-    A reference to an instance of :class:RescueInterface.
-    May be None, if unsupported by a driver.
+    A reference to an instance of :class:InspectInterface.
     """
 
     management = None
     """`Standard` attribute for management related features.
 
     A reference to an instance of :class:ManagementInterface.
-    May be None, if unsupported by a driver.
     """
 
-    boot = None
-    """`Standard` attribute for boot related features.
+    network = None
+    """`Core` attribute for network connectivity.
 
-    A reference to an instance of :class:BootInterface.
-    May be None, if unsupported by a driver.
+    A reference to an instance of :class:NetworkInterface.
     """
 
-    vendor = None
-    """Attribute for accessing any vendor-specific extensions.
+    power = None
+    """`Core` attribute for managing power state.
 
-    A reference to an instance of :class:VendorInterface.
-    May be None, if the driver does not implement any vendor extensions.
-    """
-
-    inspect = None
-    """`Standard` attribute for inspection related features.
-
-    A reference to an instance of :class:InspectInterface.
-    May be None, if unsupported by a driver.
+    A reference to an instance of :class:PowerInterface.
     """
 
     raid = None
     """`Standard` attribute for RAID related features.
 
     A reference to an instance of :class:RaidInterface.
-    May be None, if unsupported by a driver.
     """
 
-    def __init__(self):
-        pass
+    rescue = None
+    """`Standard` attribute for accessing rescue features.
+
+    A reference to an instance of :class:RescueInterface.
+    """
+
+    storage = None
+    """`Standard` attribute for (remote) storage interface.
+
+    A reference to an instance of :class:StorageInterface.
+    """
+
+    vendor = None
+    """Attribute for accessing any vendor-specific extensions.
+
+    A reference to an instance of :class:VendorInterface.
+    """
+
+    @property
+    def core_interfaces(self):
+        """Interfaces that are required to be implemented."""
+        return ['boot', 'deploy', 'management', 'network', 'power']
+
+    @property
+    def optional_interfaces(self):
+        """Interfaces that can be no-op."""
+        return ['bios', 'console', 'inspect', 'raid', 'rescue', 'storage']
 
     @property
     def all_interfaces(self):
-        return (list(self.core_interfaces + self.standard_interfaces) +
-                ['vendor'])
+        return self.non_vendor_interfaces + ['vendor']
 
     @property
     def non_vendor_interfaces(self):
-        return list(self.core_interfaces + self.standard_interfaces)
+        return list(self.core_interfaces + self.optional_interfaces)
 
     def get_properties(self):
         """Get the properties of the driver.
@@ -149,48 +149,9 @@ class BaseDriver(object):
                 properties.update(iface.get_properties())
         return properties
 
-    @classmethod
-    def to_hardware_type(cls):
-        """Return corresponding hardware type and hardware interfaces.
-
-        :returns: a tuple with two items:
-
-            * new driver field - the target hardware type
-            * dictionary containing interfaces to update, e.g.
-              {'deploy': 'iscsi', 'power': 'ipmitool'}
-        """
-        raise NotImplementedError()
-
-
-class BareDriver(BaseDriver):
-    """A bare driver object which will have interfaces attached later.
-
-    Any composable interfaces should be added as class attributes of this
-    class, as well as appended to core_interfaces or standard_interfaces here.
-    """
-
-    network = None
-    """`Core` attribute for network connectivity.
-
-    A reference to an instance of :class:NetworkInterface.
-    """
-    core_interfaces = BaseDriver.core_interfaces + ('network',)
-
-    storage = None
-    """`Standard` attribute for (remote) storage interface.
-
-    A reference to an instance of :class:StorageInterface.
-    """
-
-    standard_interfaces = (BaseDriver.standard_interfaces + ('rescue',
-                           'storage',))
-
 
 ALL_INTERFACES = set(BareDriver().all_interfaces)
-"""Constant holding all known interfaces.
-
-Includes interfaces not exposed via BaseDriver.all_interfaces.
-"""
+"""Constant holding all known interfaces."""
 
 
 @six.add_metaclass(abc.ABCMeta)
@@ -225,7 +186,7 @@ class BaseInterface(object):
         This method is often executed synchronously in API requests, so it
         should not conduct long-running checks.
 
-        :param task: a TaskManager instance containing the node to act on.
+        :param task: A TaskManager instance containing the node to act on.
         :raises: InvalidParameterValue on malformed parameter(s)
         :raises: MissingParameterValue on missing parameter(s)
         """
@@ -312,7 +273,7 @@ class DeployInterface(BaseInterface):
         performed any preparatory steps, such as pre-caching some data for the
         node.
 
-        :param task: a TaskManager instance containing the node to act on.
+        :param task: A TaskManager instance containing the node to act on.
         :returns: status of the deploy. One of ironic.common.states.
         """
 
@@ -323,7 +284,7 @@ class DeployInterface(BaseInterface):
         Given a node that has been previously deployed to,
         do all cleanup and tear down necessary to "un-deploy" that node.
 
-        :param task: a TaskManager instance containing the node to act on.
+        :param task: A TaskManager instance containing the node to act on.
         :returns: status of the deploy. One of ironic.common.states.
         """
 
@@ -339,7 +300,7 @@ class DeployInterface(BaseInterface):
 
         This method is called before `deploy`.
 
-        :param task: a TaskManager instance containing the node to act on.
+        :param task: A TaskManager instance containing the node to act on.
         """
 
     @abc.abstractmethod
@@ -357,7 +318,7 @@ class DeployInterface(BaseInterface):
 
         This method is called before `tear_down`.
 
-        :param task: a TaskManager instance containing the node to act on.
+        :param task: A TaskManager instance containing the node to act on.
         """
 
     @abc.abstractmethod
@@ -376,7 +337,7 @@ class DeployInterface(BaseInterface):
             in Neutron as part of remapping that node's control to itself.
             This is performed within the `takeover` method.
 
-        :param task: a TaskManager instance containing the node to act on.
+        :param task: A TaskManager instance containing the node to act on.
         """
 
     def prepare_cleaning(self, task):
@@ -394,7 +355,7 @@ class DeployInterface(BaseInterface):
         NOTE(JoshNang) this should be moved to BootInterface when it gets
         implemented.
 
-        :param task: a TaskManager instance containing the node to act on.
+        :param task: A TaskManager instance containing the node to act on.
         :returns: If this function is going to be asynchronous, should return
             `states.CLEANWAIT`. Otherwise, should return `None`. The interface
             will need to call _get_cleaning_steps and then RPC to
@@ -411,14 +372,14 @@ class DeployInterface(BaseInterface):
         NOTE(JoshNang) this should be moved to BootInterface when it gets
         implemented.
 
-        :param task: a TaskManager instance containing the node to act on.
+        :param task: A TaskManager instance containing the node to act on.
         """
         pass
 
     def heartbeat(self, task, callback_url, agent_version):
         """Record a heartbeat for the node.
 
-        :param task: a TaskManager instance containing the node to act on.
+        :param task: A TaskManager instance containing the node to act on.
         :param callback_url: a URL to use to call to the ramdisk.
         :param agent_version: The version of the agent that is heartbeating
         :return: None
@@ -439,8 +400,8 @@ class BootInterface(BaseInterface):
         This method prepares the boot of the deploy or rescue ramdisk after
         reading relevant information from the node's database.
 
-        :param task: a task from TaskManager.
-        :param ramdisk_params: the options to be passed to the ironic ramdisk.
+        :param task: A task from TaskManager.
+        :param ramdisk_params: The options to be passed to the ironic ramdisk.
             Different implementations might want to boot the ramdisk in
             different ways by passing parameters to them.  For example,
 
@@ -460,7 +421,7 @@ class BootInterface(BaseInterface):
         This method cleans up the environment that was setup for booting the
         deploy or rescue ramdisk.
 
-        :param task: a task from TaskManager.
+        :param task: A task from TaskManager.
         :returns: None
         """
 
@@ -471,7 +432,7 @@ class BootInterface(BaseInterface):
         This method prepares the boot of the instance after reading
         relevant information from the node's database.
 
-        :param task: a task from TaskManager.
+        :param task: A task from TaskManager.
         :returns: None
         """
 
@@ -482,10 +443,21 @@ class BootInterface(BaseInterface):
         This method cleans up the environment that was setup for booting
         the instance.
 
-        :param task: a task from TaskManager.
+        :param task: A task from TaskManager.
         :returns: None
         """
 
+    def validate_rescue(self, task):
+        """Validate that the node has required properties for rescue.
+
+        :param task: A TaskManager instance with the node being checked
+        :raises: MissingParameterValue if node is missing one or more required
+            parameters
+        :raises: UnsupportedDriverExtension
+        """
+        raise exception.UnsupportedDriverExtension(
+            driver=task.node.driver, extension='validate_rescue')
+
 
 class PowerInterface(BaseInterface):
     """Interface for power-related actions."""
@@ -495,16 +467,16 @@ class PowerInterface(BaseInterface):
     def get_power_state(self, task):
         """Return the power state of the task's node.
 
-        :param task: a TaskManager instance containing the node to act on.
+        :param task: A TaskManager instance containing the node to act on.
         :raises: MissingParameterValue if a required parameter is missing.
-        :returns: a power state. One of :mod:`ironic.common.states`.
+        :returns: A power state. One of :mod:`ironic.common.states`.
         """
 
     @abc.abstractmethod
     def set_power_state(self, task, power_state, timeout=None):
         """Set the power state of the task's node.
 
-        :param task: a TaskManager instance containing the node to act on.
+        :param task: A TaskManager instance containing the node to act on.
         :param power_state: Any power state from :mod:`ironic.common.states`.
         :param timeout: timeout (in seconds) positive integer (> 0) for any
           power state. ``None`` indicates to use default timeout.
@@ -518,7 +490,7 @@ class PowerInterface(BaseInterface):
         Drivers are expected to properly handle case when node is powered off
         by powering it on.
 
-        :param task: a TaskManager instance containing the node to act on.
+        :param task: A TaskManager instance containing the node to act on.
         :param timeout: timeout (in seconds) positive integer (> 0) for any
           power state. ``None`` indicates to use default timeout.
         :raises: MissingParameterValue if a required parameter is missing.
@@ -544,14 +516,14 @@ class ConsoleInterface(BaseInterface):
 
         This method should not raise an exception if console already started.
 
-        :param task: a TaskManager instance containing the node to act on.
+        :param task: A TaskManager instance containing the node to act on.
         """
 
     @abc.abstractmethod
     def stop_console(self, task):
         """Stop the remote console session for the task's node.
 
-        :param task: a TaskManager instance containing the node to act on.
+        :param task: A TaskManager instance containing the node to act on.
         """
 
     @abc.abstractmethod
@@ -561,7 +533,7 @@ class ConsoleInterface(BaseInterface):
         This method should return the necessary information for the
         client to access the console.
 
-        :param task: a TaskManager instance containing the node to act on.
+        :param task: A TaskManager instance containing the node to act on.
         :returns: the console connection information.
         """
 
@@ -574,7 +546,7 @@ class RescueInterface(BaseInterface):
     def rescue(self, task):
         """Boot the task's node into a rescue environment.
 
-        :param task: a TaskManager instance containing the node to act on.
+        :param task: A TaskManager instance containing the node to act on.
         :raises: InstanceRescueFailure if node validation or rescue operation
                  fails.
         :returns: states.RESCUEWAIT if rescue is in progress asynchronously
@@ -585,7 +557,7 @@ class RescueInterface(BaseInterface):
     def unrescue(self, task):
         """Tear down the rescue environment, and return to normal.
 
-        :param task: a TaskManager instance containing the node to act on.
+        :param task: A TaskManager instance containing the node to act on.
         :raises: InstanceUnrescueFailure if node validation or unrescue
                  operation fails.
         :returns: states.ACTIVE if it is successful.
@@ -597,7 +569,7 @@ class RescueInterface(BaseInterface):
         This is particularly useful for nodes where rescuing is asynchronous
         and a timeout occurs.
 
-        :param task: a TaskManager instance containing the node to act on.
+        :param task: A TaskManager instance containing the node to act on.
         :returns: None
         """
         pass
@@ -608,8 +580,9 @@ VendorMetadata = collections.namedtuple('VendorMetadata', ['method',
                                                            'metadata'])
 
 
-def _passthru(http_methods, method=None, async=True, driver_passthru=False,
-              description=None, attach=False, require_exclusive_lock=True):
+def _passthru(http_methods, method=None, async=None, async_call=None,
+              driver_passthru=False, description=None,
+              attach=False, require_exclusive_lock=True):
     """A decorator for registering a function as a passthru function.
 
     Decorator ensures function is ready to catch any ironic exceptions
@@ -623,7 +596,8 @@ def _passthru(http_methods, method=None, async=True, driver_passthru=False,
     :param http_methods: A list of supported HTTP methods by the vendor
                          function.
     :param method: an arbitrary string describing the action to be taken.
-    :param async: Boolean value. If True invoke the passthru function
+    :param async: Deprecated, please use async_call instead.
+    :param async_call: Boolean value. If True invoke the passthru function
                   asynchronously; if False, synchronously. If a passthru
                   function touches the BMC we strongly recommend it to
                   run asynchronously. Defaults to True.
@@ -642,6 +616,27 @@ def _passthru(http_methods, method=None, async=True, driver_passthru=False,
                                    for a synchronous passthru method. If False,
                                    don't lock the node. Defaults to True.
     """
+    # TODO(rloo): In Stein cycle, remove support for 'async' parameter.
+    #             The default value for 'async_call' should then be changed
+    #             to True.
+    if async_call is None:
+        if async is not None:
+            LOG.warning(
+                'The "async" parameter is deprecated, please use "async_call" '
+                'instead. The "async" parameter will be removed in the Stein '
+                'cycle.'
+            )
+            async_call = async
+        else:
+            async_call = True
+    else:
+        if async is not None:
+            raise TypeError(
+                "'async_call' and 'async' parameters cannot be used together. "
+                "Use 'async_call' instead of 'async' since 'async' is "
+                "deprecated and will be removed in the Stein cycle."
+            )
+
     def handle_passthru(func):
         api_method = method
         if api_method is None:
@@ -650,7 +645,7 @@ def _passthru(http_methods, method=None, async=True, driver_passthru=False,
         supported_ = [i.upper() for i in http_methods]
         description_ = description or ''
         metadata = VendorMetadata(api_method, {'http_methods': supported_,
-                                               'async': async,
+                                               'async': async_call,
                                                'description': description_,
                                                'attach': attach})
         if driver_passthru:
@@ -676,17 +671,19 @@ def _passthru(http_methods, method=None, async=True, driver_passthru=False,
     return handle_passthru
 
 
-def passthru(http_methods, method=None, async=True, description=None,
-             attach=False, require_exclusive_lock=True):
-    return _passthru(http_methods, method, async, driver_passthru=False,
+def passthru(http_methods, method=None, async=None, description=None,
+             attach=False, require_exclusive_lock=True, async_call=None):
+    return _passthru(http_methods, method, async, async_call,
+                     driver_passthru=False,
                      description=description, attach=attach,
                      require_exclusive_lock=require_exclusive_lock)
 
 
-def driver_passthru(http_methods, method=None, async=True, description=None,
-                    attach=False):
-    return _passthru(http_methods, method, async, driver_passthru=True,
-                     description=description, attach=attach)
+def driver_passthru(http_methods, method=None, async=None, description=None,
+                    attach=False, async_call=None):
+    return _passthru(http_methods, method, async, async_call,
+                     driver_passthru=True, description=description,
+                     attach=attach)
 
 
 class VendorInterface(BaseInterface):
@@ -733,9 +730,9 @@ class VendorInterface(BaseInterface):
 
         If invalid, raises an exception; otherwise returns None.
 
-        :param task: a task from TaskManager.
-        :param method: method to be validated
-        :param kwargs: info for action.
+        :param task: A task from TaskManager.
+        :param method: Method to be validated
+        :param kwargs: Info for action.
         :raises: UnsupportedDriverExtension if 'method' can not be mapped to
                  the supported interfaces.
         :raises: InvalidParameterValue if kwargs does not contain 'method'.
@@ -764,7 +761,7 @@ class ManagementInterface(BaseInterface):
     def get_supported_boot_devices(self, task):
         """Get a list of the supported boot devices.
 
-        :param task: a task from TaskManager.
+        :param task: A task from TaskManager.
         :returns: A list with the supported boot devices defined
                   in :mod:`ironic.common.boot_devices`.
         """
@@ -775,8 +772,8 @@ class ManagementInterface(BaseInterface):
 
         Set the boot device to use on next reboot of the node.
 
-        :param task: a task from TaskManager.
-        :param device: the boot device, one of
+        :param task: A task from TaskManager.
+        :param device: The boot device, one of
                        :mod:`ironic.common.boot_devices`.
         :param persistent: Boolean value. True if the boot device will
                            persist to all future boots, False if not.
@@ -793,12 +790,12 @@ class ManagementInterface(BaseInterface):
         Provides the current boot device of the node. Be aware that not
         all drivers support this.
 
-        :param task: a task from TaskManager.
+        :param task: A task from TaskManager.
         :raises: MissingParameterValue if a required parameter is missing
-        :returns: a dictionary containing:
+        :returns: A dictionary containing:
 
             :boot_device:
-                the boot device, one of :mod:`ironic.common.boot_devices` or
+                Ahe boot device, one of :mod:`ironic.common.boot_devices` or
                 None if it is unknown.
             :persistent:
                 Whether the boot device will persist to all future boots or
@@ -806,14 +803,79 @@ class ManagementInterface(BaseInterface):
 
         """
 
+    def get_supported_boot_modes(self, task):
+        """Get a list of the supported boot modes.
+
+        NOTE: Not all drivers support this method. Older hardware
+              may not implement that.
+
+        :param task: A task from TaskManager.
+        :raises: UnsupportedDriverExtension if requested operation is
+                 not supported by the driver
+        :raises: DriverOperationError or its derivative in case
+                 of driver runtime error.
+        :raises: MissingParameterValue if a required parameter is missing
+        :returns: A list with the supported boot modes defined
+                  in :mod:`ironic.common.boot_modes`. If boot
+                  mode support can't be determined, empty list
+                  is returned.
+        """
+        raise exception.UnsupportedDriverExtension(
+            driver=task.node.driver, extension='get_supported_boot_modes')
+
+    def set_boot_mode(self, task, mode):
+        """Set the boot mode for a node.
+
+        Set the boot mode to use on next reboot of the node.
+
+        Drivers implementing this method are required to implement
+        the `get_supported_boot_modes` method as well.
+
+        NOTE: Not all drivers support this method. Hardware supporting only
+            one boot mode may not implement that.
+
+        :param task: A task from TaskManager.
+        :param mode: The boot mode, one of
+                     :mod:`ironic.common.boot_modes`.
+        :raises: InvalidParameterValue if an invalid boot mode is
+                 specified.
+        :raises: MissingParameterValue if a required parameter is missing
+        :raises: UnsupportedDriverExtension if requested operation is
+                 not supported by the driver
+        :raises: DriverOperationError or its derivative in case
+                 of driver runtime error.
+        """
+        raise exception.UnsupportedDriverExtension(
+            driver=task.node.driver, extension='set_boot_mode')
+
+    def get_boot_mode(self, task):
+        """Get the current boot mode for a node.
+
+        Provides the current boot mode of the node.
+
+        NOTE: Not all drivers support this method. Older hardware
+              may not implement that.
+
+        :param task: A task from TaskManager.
+        :raises: MissingParameterValue if a required parameter is missing
+        :raises: DriverOperationError or its  derivative in case
+                 of driver runtime error.
+        :raises: UnsupportedDriverExtension if requested operation is
+                 not supported by the driver
+        :returns: The boot mode, one of :mod:`ironic.common.boot_mode` or
+                  None if it is unknown.
+        """
+        raise exception.UnsupportedDriverExtension(
+            driver=task.node.driver, extension='get_boot_mode')
+
     @abc.abstractmethod
     def get_sensors_data(self, task):
         """Get sensors data method.
 
-        :param task: a TaskManager instance.
+        :param task: A TaskManager instance.
         :raises: FailedToGetSensorData when getting the sensor data fails.
         :raises: FailedToParseSensorData when parsing sensor data fails.
-        :returns: returns a consistent format dict of sensor data grouped by
+        :returns: Returns a consistent format dict of sensor data grouped by
                   sensor type, which can be processed by Ceilometer.
                   eg,
 
@@ -873,13 +935,110 @@ class InspectInterface(BaseInterface):
         Inspect hardware to obtain the essential & additional hardware
         properties.
 
-        :param task: a task from TaskManager.
+        :param task: A task from TaskManager.
         :raises: HardwareInspectionFailure, if unable to get essential
                  hardware properties.
-        :returns: resulting state of the inspection i.e. states.MANAGEABLE
+        :returns: Resulting state of the inspection i.e. states.MANAGEABLE
                   or None.
         """
 
+    def abort(self, task):
+        """Abort asynchronized hardware inspection.
+
+        Abort an ongoing hardware introspection, this is only used for
+        asynchronize based inspect interface.
+
+        NOTE: This interface is called with node exclusive lock held, the
+        interface implementation is expected to be a quick processing.
+
+        :param task: a task from TaskManager.
+        :raises: UnsupportedDriverExtension, if the method is not implemented
+                 by specific inspect interface.
+        """
+        raise exception.UnsupportedDriverExtension(
+            driver=task.node.driver, extension='abort')
+
+
+class BIOSInterface(BaseInterface):
+    interface_type = 'bios'
+
+    def __new__(cls, *args, **kwargs):
+        # Wrap the apply_configuration and factory_reset into a decorator
+        # which call cache_bios_settings() to update the node's BIOS setting
+        # table after apply_configuration and factory_reset have finished.
+
+        super_new = super(BIOSInterface, cls).__new__
+        instance = super_new(cls, *args, **kwargs)
+
+        def wrapper(func):
+            @six.wraps(func)
+            def wrapped(task, *args, **kwargs):
+                func(task, *args, **kwargs)
+                instance.cache_bios_settings(task)
+            return wrapped
+
+        for n, method in inspect.getmembers(instance, inspect.ismethod):
+            if n == "apply_configuration":
+                instance.apply_configuration = wrapper(method)
+            elif n == "factory_reset":
+                instance.factory_reset = wrapper(method)
+        return instance
+
+    @abc.abstractmethod
+    def apply_configuration(self, task, settings):
+        """Validate & apply BIOS settings on the given node.
+
+        This method takes the BIOS settings from the settings param and
+        applies BIOS settings on the given node. It may also validate the
+        given bios settings before applying any settings and manage
+        failures when setting an invalid BIOS config. In the case of
+        needing password to update the BIOS config, it will be taken from
+        the driver_info properties. After the BIOS configuration is done,
+        cache_bios_settings will be called to update the node's BIOS setting
+        table with the BIOS configuration applied on the node.
+
+        :param task: a TaskManager instance.
+        :param settings: Dictonary containing the BIOS configuration.
+        :raises: UnsupportedDriverExtension, if the node's driver doesn't
+            support BIOS configuration.
+        :raises: InvalidParameterValue, if validation of settings fails.
+        :raises: MissingParameterValue, if some required parameters are
+            missing.
+        :returns: states.CLEANWAIT if BIOS configuration is in progress
+            asynchronously or None if it is complete.
+        """
+
+    @abc.abstractmethod
+    def factory_reset(self, task):
+        """Reset BIOS configuration to factory default on the given node.
+
+        This method resets BIOS configuration to factory default on the
+        given node. After the BIOS reset action is done, cache_bios_settings
+        will be called to update the node's BIOS settings table with default
+        bios settings.
+
+        :param task: a TaskManager instance.
+        :raises: UnsupportedDriverExtension, if the node's driver doesn't
+            support BIOS reset.
+        :returns: states.CLEANWAIT if BIOS configuration is in progress
+            asynchronously or None if it is complete.
+        """
+
+    @abc.abstractmethod
+    def cache_bios_settings(self, task):
+        """Store or update BIOS properties on the given node.
+
+        This method stores BIOS properties to the bios_settings table during
+        'cleaning' operation and updates bios_settings table when
+        apply_configuration() and factory_reset() are called to set new BIOS
+        configurations. It will also update the timestamp of each bios setting.
+
+        :param task: a TaskManager instance.
+        :raises: UnsupportedDriverExtension, if the node's driver doesn't
+            support getting BIOS properties from bare metal.
+        :returns: None.
+        """
+
 
 class RAIDInterface(BaseInterface):
     interface_type = 'raid'
@@ -903,7 +1062,7 @@ class RAIDInterface(BaseInterface):
         configuration. Driver implementations of this interface can override
         this method for doing more validations (such as BMC's credentials).
 
-        :param task: a TaskManager instance.
+        :param task: A TaskManager instance.
         :raises: InvalidParameterValue, if the RAID configuration is invalid.
         :raises: MissingParameterValue, if some parameters are missing.
         """
@@ -919,7 +1078,7 @@ class RAIDInterface(BaseInterface):
         implementations of this interface can override this method to support
         custom parameters for RAID configuration.
 
-        :param task: a TaskManager instance.
+        :param task: A TaskManager instance.
         :param raid_config: The RAID configuration to validate.
         :raises: InvalidParameterValue, if the RAID configuration is invalid.
         """
@@ -941,7 +1100,7 @@ class RAIDInterface(BaseInterface):
         may be called to sync the node's RAID-related information with the
         RAID configuration applied on the node.
 
-        :param task: a TaskManager instance.
+        :param task: A TaskManager instance.
         :param create_root_volume: Setting this to False indicates
             not to create root volume that is specified in the node's
             target_raid_config. Default value is True.
@@ -960,7 +1119,7 @@ class RAIDInterface(BaseInterface):
         After RAID configuration is deleted, node.raid_config should be
         cleared by the implementation.
 
-        :param task: a TaskManager instance.
+        :param task: A TaskManager instance.
         :returns: states.CLEANWAIT if deletion is in progress
             asynchronously or None if it is complete.
         """
@@ -992,7 +1151,7 @@ class NetworkInterface(BaseInterface):
     def validate(self, task):
         """Validates the network interface.
 
-        :param task: a TaskManager instance.
+        :param task: A TaskManager instance.
         :raises: InvalidParameterValue, if the network interface configuration
             is invalid.
         :raises: MissingParameterValue, if some parameters are missing.
@@ -1002,7 +1161,7 @@ class NetworkInterface(BaseInterface):
     def port_changed(self, task, port_obj):
         """Handle any actions required when a port changes
 
-        :param task: a TaskManager instance.
+        :param task: A TaskManager instance.
         :param port_obj: a changed Port object.
         :raises: Conflict, FailedToUpdateDHCPOptOnPort
         """
@@ -1011,7 +1170,7 @@ class NetworkInterface(BaseInterface):
     def portgroup_changed(self, task, portgroup_obj):
         """Handle any actions required when a port changes
 
-        :param task: a TaskManager instance.
+        :param task: A TaskManager instance.
         :param portgroup_obj: a changed Port object.
         :raises: Conflict, FailedToUpdateDHCPOptOnPort
         """
@@ -1110,7 +1269,7 @@ class NetworkInterface(BaseInterface):
     def validate_rescue(self, task):
         """Validates the network interface for rescue operation.
 
-        :param task: a TaskManager instance.
+        :param task: A TaskManager instance.
         :raises: InvalidParameterValue, if the network interface configuration
             is invalid.
         :raises: MissingParameterValue, if some parameters are missing.
@@ -1150,7 +1309,7 @@ class StorageInterface(BaseInterface):
     def attach_volumes(self, task):
         """Informs the storage subsystem to attach all volumes for the node.
 
-        :param task: a TaskManager instance.
+        :param task: A TaskManager instance.
         :raises: UnsupportedDriverExtension
         """
 
@@ -1158,7 +1317,7 @@ class StorageInterface(BaseInterface):
     def detach_volumes(self, task):
         """Informs the storage subsystem to detach all volumes for the node.
 
-        :param task: a TaskManager instance.
+        :param task: A TaskManager instance.
         :raises: UnsupportedDriverExtension
         """
 
@@ -1166,7 +1325,7 @@ class StorageInterface(BaseInterface):
     def should_write_image(self, task):
         """Determines if deploy should perform the image write-out.
 
-        :param task: a TaskManager instance.
+        :param task: A TaskManager instance.
         :returns: Boolean value to indicate if the interface expects
                   the image to be written by Ironic.
         :raises: UnsupportedDriverExtension
