@@ -28,7 +28,6 @@ from ironic.common import boot_devices
 from ironic.common import exception
 from ironic.common.i18n import _
 from ironic.common import states
-from ironic.conductor import rpcapi
 from ironic.conductor import utils as manager_utils
 from ironic.conf import CONF
 from ironic.drivers.modules import agent_client
@@ -152,17 +151,6 @@ def _cleaning_reboot(task):
     task.node.save()
 
 
-def _notify_conductor_resume_clean(task):
-    LOG.debug('Sending RPC to conductor to resume cleaning for node %s',
-              task.node.uuid)
-    uuid = task.node.uuid
-    rpc = rpcapi.ConductorAPI()
-    topic = rpc.get_topic_for(task.node)
-    # Need to release the lock to let the conductor take it
-    task.release_resources()
-    rpc.continue_node_clean(task.context, uuid, topic=topic)
-
-
 def _get_completed_cleaning_command(task, commands):
     """Returns None or a completed cleaning command from the agent.
 
@@ -191,8 +179,8 @@ def _get_completed_cleaning_command(task, commands):
         LOG.debug('Clean step still running for node %(node)s: %(step)s',
                   {'step': last_step, 'node': task.node.uuid})
         return
-    elif (last_command['command_status'] == 'SUCCEEDED' and
-          last_step != task.node.clean_step):
+    elif (last_command['command_status'] == 'SUCCEEDED'
+          and last_step != task.node.clean_step):
         # A previous clean_step was running, the new command has not yet
         # started.
         LOG.debug('Clean step not yet started for node %(node)s: %(step)s',
@@ -290,7 +278,13 @@ class HeartbeatMixin(object):
                        'state': task.node.provision_state})
             return
 
-        task.upgrade_lock()
+        try:
+            task.upgrade_lock()
+        except exception.NodeLocked:
+            LOG.warning('Node %s is currently locked, skipping heartbeat '
+                        'processing (will retry on the next heartbeat)',
+                        task.node.uuid)
+            return
 
         node = task.node
         LOG.debug('Heartbeat from node %s', node.uuid)
@@ -310,16 +304,16 @@ class HeartbeatMixin(object):
                 LOG.debug('Heartbeat from node %(node)s in maintenance mode; '
                           'not taking any action.', {'node': node.uuid})
                 return
-            elif (node.provision_state == states.DEPLOYWAIT and
-                  not self.deploy_has_started(task)):
+            elif (node.provision_state == states.DEPLOYWAIT
+                  and not self.deploy_has_started(task)):
                 msg = _('Node failed to deploy.')
                 self.continue_deploy(task)
-            elif (node.provision_state == states.DEPLOYWAIT and
-                  self.deploy_is_done(task)):
+            elif (node.provision_state == states.DEPLOYWAIT
+                  and self.deploy_is_done(task)):
                 msg = _('Node failed to move to active state.')
                 self.reboot_to_instance(task)
-            elif (node.provision_state == states.DEPLOYWAIT and
-                  self.deploy_has_started(task)):
+            elif (node.provision_state == states.DEPLOYWAIT
+                  and self.deploy_has_started(task)):
                 node.touch_provisioning()
             elif node.provision_state == states.CLEANWAIT:
                 node.touch_provisioning()
@@ -333,7 +327,7 @@ class HeartbeatMixin(object):
                     manager_utils.set_node_cleaning_steps(task)
                     # The exceptions from RPC are not possible as we using cast
                     # here
-                    _notify_conductor_resume_clean(task)
+                    manager_utils.notify_conductor_resume_clean(task)
                 else:
                     msg = _('Node failed to check cleaning progress.')
                     self.continue_cleaning(task)
@@ -367,8 +361,8 @@ class HeartbeatMixin(object):
             raise exception.InstanceRescueFailure(node=node.uuid,
                                                   instance=node.instance_uuid,
                                                   reason=e)
-        if ((not result.get('command_status')) or
-                result.get('command_status') != 'SUCCEEDED'):
+        if ((not result.get('command_status'))
+                or result.get('command_status') != 'SUCCEEDED'):
             # NOTE(mariojv) Caller will clean up failed rescue in exception
             # handler.
             fail_reason = (_('Agent returned bad result for command '
@@ -472,7 +466,7 @@ class AgentDeployMixin(HeartbeatMixin):
                 info = task.node.driver_internal_info
                 info.pop('cleaning_reboot', None)
                 task.node.driver_internal_info = info
-                _notify_conductor_resume_clean(task)
+                manager_utils.notify_conductor_resume_clean(task)
                 return
             else:
                 # Agent has no commands whatsoever
@@ -537,7 +531,7 @@ class AgentDeployMixin(HeartbeatMixin):
                     LOG.exception(msg)
                     return manager_utils.cleaning_error_handler(task, msg)
 
-            _notify_conductor_resume_clean(task)
+            manager_utils.notify_conductor_resume_clean(task)
 
         elif command.get('command_status') == 'SUCCEEDED':
             clean_step_hook = _get_post_clean_step_hook(node)
@@ -567,7 +561,7 @@ class AgentDeployMixin(HeartbeatMixin):
 
             LOG.info('Agent on node %s returned cleaning command success, '
                      'moving to next clean step', node.uuid)
-            _notify_conductor_resume_clean(task)
+            manager_utils.notify_conductor_resume_clean(task)
         else:
             msg = (_('Agent returned unknown status for clean step %(step)s '
                      'on node %(node)s : %(err)s.') %
