@@ -76,14 +76,6 @@ OPTIONAL_PROPERTIES = {
 }
 
 COMMON_PROPERTIES = OPTIONAL_PROPERTIES
-# TODO(pas-ha) remove in Rocky
-DEPRECATED_PROPERTIES = {
-    'ansible_deploy_username': {
-        'name': 'ansible_username',
-        'warned': False},
-    'ansible_deploy_key_file': {
-        'name': 'ansible_key_file',
-        'warned': False}}
 
 
 class PlaybookNotFound(exception.IronicException):
@@ -96,23 +88,10 @@ def _get_playbooks_path(node):
 
 
 def _parse_ansible_driver_info(node, action='deploy'):
-    # TODO(pas-ha) remove in Rocky
-    for old, new in DEPRECATED_PROPERTIES.items():
-        if old in node.driver_info:
-            if not new['warned']:
-                LOG.warning("Driver property '%(old)s' is deprecated, "
-                            "and will be ignored in Rocky release. "
-                            "Use '%(new)s' instead.", old=old, new=new['name'])
-                new['warned'] = True
-    # TODO(pas-ha) simplify in Rocky
-    user = node.driver_info.get(
-        'ansible_username',
-        node.driver_info.get('ansible_deploy_username',
-                             CONF.ansible.default_username))
-    key = node.driver_info.get(
-        'ansible_key_file',
-        node.driver_info.get('ansible_deploy_key_file',
-                             CONF.ansible.default_key_file))
+    user = node.driver_info.get('ansible_username',
+                                CONF.ansible.default_username)
+    key = node.driver_info.get('ansible_key_file',
+                               CONF.ansible.default_key_file)
     playbook = node.driver_info.get('ansible_%s_playbook' % action,
                                     getattr(CONF.ansible,
                                             'default_%s_playbook' % action,
@@ -417,9 +396,9 @@ class AnsibleDeploy(agent_base.HeartbeatMixin, base.DeployInterface):
         iwdi = node.driver_internal_info.get('is_whole_disk_image')
         if not iwdi and deploy_utils.get_boot_option(node) == "netboot":
             raise exception.InvalidParameterValue(_(
-                "Node %(node)s is configured to use the %(driver)s driver "
-                "which does not support netboot.") % {'node': node.uuid,
-                                                      'driver': node.driver})
+                "Node %(node)s is configured to use the ansible deploy "
+                "interface, which does not support netboot.") %
+                {'node': node.uuid})
 
         params = {}
         image_source = node.instance_info.get('image_source')
@@ -528,10 +507,17 @@ class AnsibleDeploy(agent_base.HeartbeatMixin, base.DeployInterface):
         LOG.debug('Starting cleaning step %(step)s on node %(node)s',
                   {'node': node.uuid, 'step': stepname})
         step_tags = step['args'].get('tags', [])
-        _run_playbook(node, playbook, extra_vars, key, tags=step_tags)
-        LOG.info('Ansible completed cleaning step %(step)s '
-                 'on node %(node)s.',
-                 {'node': node.uuid, 'step': stepname})
+        try:
+            _run_playbook(node, playbook, extra_vars, key, tags=step_tags)
+        except exception.InstanceDeployFailure as e:
+            LOG.error("Ansible failed cleaning step %(step)s "
+                      "on node %(node)s.",
+                      {'node': node.uuid, 'step': stepname})
+            manager_utils.cleaning_error_handler(task, six.text_type(e))
+        else:
+            LOG.info('Ansible completed cleaning step %(step)s '
+                     'on node %(node)s.',
+                     {'node': node.uuid, 'step': stepname})
 
     @METRICS.timer('AnsibleDeploy.prepare_cleaning')
     def prepare_cleaning(self, task):
