@@ -29,6 +29,7 @@ from ironic.common import exception
 from ironic.common.i18n import _
 from ironic.common import states
 from ironic.drivers import base
+from ironic import objects
 
 
 class FakePower(base.PowerInterface):
@@ -59,10 +60,6 @@ class FakePower(base.PowerInterface):
                 states.SOFT_REBOOT, states.SOFT_POWER_OFF]
 
 
-# NOTE(dtantsur): for backward compatibility
-FakeSoftPower = FakePower
-
-
 class FakeBoot(base.BootInterface):
     """Example implementation of a simple boot interface."""
 
@@ -154,7 +151,7 @@ class FakeVendorB(base.VendorInterface):
     def second_method(self, task, http_method, bar):
         return True if bar == 'kazoo' else False
 
-    @base.passthru(['POST'], async=False,
+    @base.passthru(['POST'], async_call=False,
                    description=_("Test if the value of bar is meow"))
     def third_method_sync(self, task, http_method, bar):
         return True if bar == 'meow' else False
@@ -236,6 +233,65 @@ class FakeRAID(base.RAIDInterface):
         pass
 
 
+class FakeBIOS(base.BIOSInterface):
+    """Fake implementation of simple BIOSInterface."""
+
+    def get_properties(self):
+        return {}
+
+    def validate(self, task):
+        pass
+
+    @base.clean_step(priority=0, argsinfo={
+        'settings': {'description': ('List of BIOS settings, each item needs '
+                     'to contain a dictionary with name/value pairs'),
+                     'required': True}})
+    def apply_configuration(self, task, settings):
+        # Note: the implementation of apply_configuration in fake interface
+        # is just for testing purpose, for real driver implementation, please
+        # refer to develop doc at https://docs.openstack.org/ironic/latest/
+        # contributor/bios_develop.html.
+        node_id = task.node.id
+        create_list, update_list, delete_list, nochange_list = (
+            objects.BIOSSettingList.sync_node_setting(task.context, node_id,
+                                                      settings))
+
+        if len(create_list) > 0:
+            objects.BIOSSettingList.create(task.context, node_id, create_list)
+        if len(update_list) > 0:
+            objects.BIOSSettingList.save(task.context, node_id, update_list)
+        if len(delete_list) > 0:
+            delete_names = [setting['name'] for setting in delete_list]
+            objects.BIOSSettingList.delete(task.context, node_id,
+                                           delete_names)
+
+        # nochange_list is part of return of sync_node_setting and it might be
+        # useful to the drivers to give a message if no change is required
+        # during application of settings.
+        if len(nochange_list) > 0:
+            pass
+
+    @base.clean_step(priority=0)
+    def factory_reset(self, task):
+        # Note: the implementation of factory_reset in fake interface is
+        # just for testing purpose, for real driver implementation, please
+        # refer to develop doc at https://docs.openstack.org/ironic/latest/
+        # contributor/bios_develop.html.
+        node_id = task.node.id
+        setting_objs = objects.BIOSSettingList.get_by_node_id(
+            task.context, node_id)
+        for setting in setting_objs:
+            objects.BIOSSetting.delete(task.context, node_id, setting.name)
+
+    @base.clean_step(priority=0)
+    def cache_bios_settings(self, task):
+        # Note: the implementation of cache_bios_settings in fake interface
+        # is just for testing purpose, for real driver implementation, please
+        # refer to develop doc at https://docs.openstack.org/ironic/latest/
+        # contributor/bios_develop.html.
+        pass
+
+
 class FakeStorage(base.StorageInterface):
     """Example implementation of simple storage Interface."""
 
