@@ -12,7 +12,7 @@
 
 """
 Modules required to work with ironic_inspector:
-    https://pypi.python.org/pypi/ironic-inspector
+    https://pypi.org/project/ironic-inspector
 """
 
 import eventlet
@@ -34,7 +34,7 @@ LOG = logging.getLogger(__name__)
 client = importutils.try_import('ironic_inspector_client')
 
 
-INSPECTOR_API_VERSION = (1, 0)
+INSPECTOR_API_VERSION = (1, 3)
 
 _INSPECTOR_SESSION = None
 
@@ -74,21 +74,6 @@ def _get_client(context):
 class Inspector(base.InspectInterface):
     """In-band inspection via ironic-inspector project."""
 
-    @classmethod
-    def create_if_enabled(cls, driver_name):
-        """Create instance of Inspector if it's enabled.
-
-        Reports log warning with given driver_name if it's not.
-
-        :return: Inspector instance or None
-        """
-        if CONF.inspector.enabled:
-            return cls()
-        else:
-            LOG.info("Inspection via ironic-inspector is disabled in "
-                     "configuration for driver %s. To enable, change "
-                     "[inspector] enabled = True.", driver_name)
-
     def __init__(self):
         if not client:
             raise exception.DriverLoadError(
@@ -119,7 +104,7 @@ class Inspector(base.InspectInterface):
         ironic-inspector. Results will be checked in a periodic task.
 
         :param task: a task from TaskManager.
-        :returns: states.INSPECTING
+        :returns: states.INSPECTWAIT
         """
         LOG.debug('Starting inspection for node %(uuid)s using '
                   'ironic-inspector', {'uuid': task.node.uuid})
@@ -128,13 +113,22 @@ class Inspector(base.InspectInterface):
         # we can release a lock as soon as possible and allow ironic-inspector
         # to operate on a node.
         eventlet.spawn_n(_start_inspection, task.node.uuid, task.context)
-        return states.INSPECTING
+        return states.INSPECTWAIT
 
-    @periodics.periodic(spacing=CONF.inspector.status_check_period,
-                        enabled=CONF.inspector.enabled)
+    def abort(self, task):
+        """Abort hardware inspection.
+
+        :param task: a task from TaskManager.
+        """
+        node_uuid = task.node.uuid
+        LOG.debug('Aborting inspection for node %(uuid)s using '
+                  'ironic-inspector', {'uuid': node_uuid})
+        _get_client(task.context).abort(node_uuid)
+
+    @periodics.periodic(spacing=CONF.inspector.status_check_period)
     def _periodic_check_result(self, manager, context):
         """Periodic task checking results of inspection."""
-        filters = {'provision_state': states.INSPECTING}
+        filters = {'provision_state': states.INSPECTWAIT}
         node_iter = manager.iter_nodes(filters=filters)
 
         for node_uuid, driver in node_iter:
@@ -171,7 +165,7 @@ def _start_inspection(node_uuid, context):
 def _check_status(task):
     """Check inspection status for node given by a task."""
     node = task.node
-    if node.provision_state != states.INSPECTING:
+    if node.provision_state != states.INSPECTWAIT:
         return
     if not isinstance(task.driver.inspect, Inspector):
         return
