@@ -53,8 +53,8 @@ from ironic.conductor import task_manager
 from ironic.conductor import utils as cond_utils
 from ironic.conf import CONF
 from ironic.drivers import base
+from ironic.drivers.modules import boot_mode_utils
 from ironic.drivers.modules import console_utils
-from ironic.drivers.modules import deploy_utils
 from ironic.drivers import utils as driver_utils
 
 
@@ -449,9 +449,9 @@ def _exec_ipmitool(driver_info, command, check_exit_code=None):
                 with excutils.save_and_reraise_exception() as ctxt:
                     err_list = [x for x in IPMITOOL_RETRYABLE_FAILURES
                                 if x in six.text_type(e)]
-                    if ((time.time() > end_time) or
-                        (num_tries == 0) or
-                        not err_list):
+                    if ((time.time() > end_time)
+                        or (num_tries == 0)
+                        or not err_list):
                         LOG.error('IPMI Error while attempting "%(cmd)s" '
                                   'for node %(node)s. Error: %(error)s',
                                   {'node': driver_info['uuid'],
@@ -587,9 +587,6 @@ def _process_sensor(sensor_data):
     for field in sensor_data_fields:
         if not field:
             continue
-        if field.startswith('<<'):
-            # This is debug data, and can be safely ignored for this.
-            continue
         kv_value = field.split(':')
         if len(kv_value) != 2:
             continue
@@ -947,7 +944,7 @@ class IPMIManagement(base.ManagementInterface):
         # uefi mode, this will work with newer and older versions of the
         # ipmitool utility. Also see:
         # https://bugs.launchpad.net/ironic/+bug/1611306
-        boot_mode = deploy_utils.get_boot_mode_for_deploy(task.node)
+        boot_mode = boot_mode_utils.get_boot_mode_for_deploy(task.node)
         if persistent and boot_mode == 'uefi':
             raw_cmd = ('0x00 0x08 0x05 0xe0 %s 0x00 0x00 0x00' %
                        BOOT_DEVICE_HEXA_MAP[device])
@@ -996,9 +993,9 @@ class IPMIManagement(base.ManagementInterface):
         driver_info = task.node.driver_info
         driver_internal_info = task.node.driver_internal_info
 
-        if (driver_info.get('ipmi_force_boot_device', False) and
-                driver_internal_info.get('persistent_boot_device') and
-                driver_internal_info.get('is_next_boot_persistent', True)):
+        if (driver_info.get('ipmi_force_boot_device', False)
+                and driver_internal_info.get('persistent_boot_device')
+                and driver_internal_info.get('is_next_boot_persistent', True)):
             return {
                 'boot_device': driver_internal_info['persistent_boot_device'],
                 'persistent': True
