@@ -33,6 +33,7 @@ from ironic.conductor import utils as manager_utils
 from ironic.conf import CONF
 from ironic.drivers import base
 from ironic.drivers.modules import agent_base_vendor
+from ironic.drivers.modules import boot_mode_utils
 from ironic.drivers.modules import deploy_utils
 from ironic.drivers.modules import image_cache
 
@@ -47,9 +48,8 @@ DISK_LAYOUT_PARAMS = ('root_gb', 'swap_mb', 'ephemeral_gb')
 class InstanceImageCache(image_cache.ImageCache):
 
     def __init__(self):
-        master_path = CONF.pxe.instance_master_path or None
         super(self.__class__, self).__init__(
-            master_path,
+            CONF.pxe.instance_master_path,
             # MiB -> B
             cache_size=CONF.pxe.image_cache_size * 1024 * 1024,
             # min -> sec
@@ -350,7 +350,7 @@ def _get_boot_mode(node):
     :param node: A single Node.
     :returns: A string representing the boot mode type. Defaults to 'bios'.
     """
-    boot_mode = deploy_utils.get_boot_mode_for_deploy(node)
+    boot_mode = boot_mode_utils.get_boot_mode_for_deploy(node)
     if boot_mode:
         return boot_mode
     return "bios"
@@ -440,9 +440,8 @@ class ISCSIDeploy(AgentDeployMixin, base.DeployInterface):
                       'volume.', node.uuid)
             return
 
-        # TODO(rameshg87): iscsi_ilo driver uses this method. Remove
-        # and copy-paste it's contents here once iscsi_ilo deploy driver
-        # broken down into separate boot and deploy implementations.
+        # TODO(rameshg87): iscsi_ilo driver used to call this function. Remove
+        # and copy-paste it's contents here.
         validate(task)
 
     @METRICS.timer('ISCSIDeploy.deploy')
