@@ -30,12 +30,19 @@ METRICS = metrics_utils.get_metrics_logger(__name__)
 class OneViewInspect(inspector.Inspector):
     """Interface for in band inspection."""
 
+    # NOTE(TheJulia): Marking as unsupported as 3rd party CI was taken down
+    # shortly before the beginning of the Rocky cycle, and no replies have
+    # indicated that 3rd party CI will be re-established nor visible
+    # actions observed regarding re-establishing 3rd party CI.
+    # TODO(TheJulia): This should be expected to be removed in Stein.
+    supported = False
+
     def get_properties(self):
         return deploy_utils.get_properties()
 
     @METRICS.timer('OneViewInspect.validate')
     def validate(self, task):
-        """Checks required info on 'driver_info' and validates node with OneView
+        """Checks required info on 'driver_info' and validates node for OneView
 
         Validates whether the 'driver_info' property of the supplied
         task's node contains the required info such as server_hardware_uri,
@@ -65,20 +72,19 @@ class OneViewInspect(inspector.Inspector):
 
     @periodics.periodic(spacing=CONF.inspector.status_check_period)
     def _periodic_check_result(self, manager, context):
-        filters = {'provision_state': states.INSPECTING}
+        filters = {'provision_state': states.INSPECTWAIT,
+                   'driver': 'oneview'}
         node_iter = manager.iter_nodes(filters=filters)
 
         for node_uuid, driver in node_iter:
-            if driver in [common.AGENT_PXE_ONEVIEW,
-                          common.ISCSI_PXE_ONEVIEW]:
-                try:
-                    lock_purpose = 'checking hardware inspection status'
-                    with task_manager.acquire(context, node_uuid,
-                                              shared=True,
-                                              purpose=lock_purpose) as task:
-                        self._check_status(task)
-                except (exception.NodeLocked, exception.NodeNotFound):
-                    continue
+            try:
+                lock_purpose = 'checking hardware inspection status'
+                with task_manager.acquire(context, node_uuid,
+                                          shared=True,
+                                          purpose=lock_purpose) as task:
+                    self._check_status(task)
+            except (exception.NodeLocked, exception.NodeNotFound):
+                continue
 
     def _check_status(self, task):
         state_before = task.node.provision_state
@@ -86,7 +92,7 @@ class OneViewInspect(inspector.Inspector):
         state_after = task.node.provision_state
 
         # inspection finished
-        if state_before == states.INSPECTING and state_after in [
+        if state_before == states.INSPECTWAIT and state_after in [
                 states.MANAGEABLE, states.INSPECTFAIL
         ]:
             deploy_utils.deallocate_server_hardware_from_ironic(task.node)
