@@ -35,6 +35,7 @@ from ironic.common import states
 from ironic.conductor import utils as manager_utils
 from ironic.conf import CONF
 from ironic.drivers import base
+from ironic.drivers.modules import boot_mode_utils
 from ironic.drivers.modules import deploy_utils
 from ironic.drivers.modules import image_cache
 from ironic.drivers import utils as driver_utils
@@ -55,6 +56,13 @@ OPTIONAL_PROPERTIES = {
                                       "deploy and cleaning operations. "
                                       "Defaults to False. Optional."),
 }
+RESCUE_PROPERTIES = {
+    'rescue_kernel': _('UUID (from Glance) of the rescue kernel. This value '
+                       'is required for rescue mode.'),
+    'rescue_ramdisk': _('UUID (from Glance) of the rescue ramdisk with agent '
+                        'that is used at node rescue time. This value is '
+                        'required for rescue mode.'),
+}
 COMMON_PROPERTIES = REQUIRED_PROPERTIES.copy()
 COMMON_PROPERTIES.update(OPTIONAL_PROPERTIES)
 
@@ -102,8 +110,8 @@ def _get_instance_image_info(node, ctx):
     image_info = {}
     # NOTE(pas-ha) do not report image kernel and ramdisk for
     # local boot or whole disk images so that they are not cached
-    if (node.driver_internal_info.get('is_whole_disk_image') or
-        deploy_utils.get_boot_option(node) == 'local'):
+    if (node.driver_internal_info.get('is_whole_disk_image')
+        or deploy_utils.get_boot_option(node) == 'local'):
             return image_info
 
     root_dir = pxe_utils.get_root_dir()
@@ -161,8 +169,8 @@ def _build_deploy_pxe_options(task, pxe_info, mode='deploy'):
                           (ramdisk_label, 'deployment_ari_path')):
         if CONF.pxe.ipxe_enabled:
             image_href = pxe_info[label][0]
-            if (CONF.pxe.ipxe_use_swift and
-                service_utils.is_glance_image(image_href)):
+            if (CONF.pxe.ipxe_use_swift
+                and service_utils.is_glance_image(image_href)):
                     pxe_opts[option] = images.get_temp_url_for_glance_image(
                         task.context, image_href)
             else:
@@ -239,8 +247,8 @@ def _build_pxe_config_options(task, pxe_info, service=False):
     mode = deploy_utils.rescue_or_deploy_mode(node)
     if service:
         pxe_options = {}
-    elif (node.driver_internal_info.get('boot_from_volume') and
-            CONF.pxe.ipxe_enabled):
+    elif (node.driver_internal_info.get('boot_from_volume')
+            and CONF.pxe.ipxe_enabled):
         pxe_options = _get_volume_pxe_options(task)
     else:
         pxe_options = _build_deploy_pxe_options(task, pxe_info, mode=mode)
@@ -264,8 +272,8 @@ def _build_service_pxe_config(task, instance_image_info,
     # NOTE(pas-ha) if it is takeover of ACTIVE node or node performing
     # unrescue operation, first ensure that basic PXE configs and links
     # are in place before switching pxe config
-    if (node.provision_state in [states.ACTIVE, states.UNRESCUING] and
-            not os.path.isfile(pxe_config_path)):
+    if (node.provision_state in [states.ACTIVE, states.UNRESCUING]
+            and not os.path.isfile(pxe_config_path)):
         pxe_options = _build_pxe_config_options(task, instance_image_info,
                                                 service=True)
         pxe_config_template = deploy_utils.get_pxe_config_template(node)
@@ -273,7 +281,7 @@ def _build_service_pxe_config(task, instance_image_info,
     iwdi = node.driver_internal_info.get('is_whole_disk_image')
     deploy_utils.switch_pxe_config(
         pxe_config_path, root_uuid_or_disk_id,
-        deploy_utils.get_boot_mode_for_deploy(node),
+        boot_mode_utils.get_boot_mode_for_deploy(node),
         iwdi, deploy_utils.is_trusted_boot_requested(node),
         deploy_utils.is_iscsi_boot(task))
 
@@ -327,29 +335,33 @@ def _get_volume_pxe_options(task):
              'iscsi_initiator_iqn': iscsi_initiator_iqn})
         # NOTE(TheJulia): This may be the route to multi-path, define
         # volumes via sanhook in the ipxe template and let the OS sort it out.
-        additional_targets = []
+        extra_targets = []
+
         for target in task.volume_targets:
             if target.boot_index != 0 and 'iscsi' in target.volume_type:
-                additional_targets.append(
-                    __generate_iscsi_url(target.properties))
-        pxe_options.update({'iscsi_volumes': additional_targets,
+                iscsi_url = __generate_iscsi_url(target.properties)
+                username = target.properties['auth_username']
+                password = target.properties['auth_password']
+                extra_targets.append({'url': iscsi_url,
+                                      'username': username,
+                                      'password': password})
+        pxe_options.update({'iscsi_volumes': extra_targets,
                             'boot_from_volume': True})
     # TODO(TheJulia): FibreChannel boot, i.e. wwpn in volume_type
     # for FCoE, should go here.
     return pxe_options
 
 
-@METRICS.timer('validate_boot_option_for_trusted_boot')
 def validate_boot_parameters_for_trusted_boot(node):
     """Check if boot parameters are valid for trusted boot."""
-    boot_mode = deploy_utils.get_boot_mode_for_deploy(node)
+    boot_mode = boot_mode_utils.get_boot_mode_for_deploy(node)
     boot_option = deploy_utils.get_boot_option(node)
     is_whole_disk_image = node.driver_internal_info.get('is_whole_disk_image')
     # 'is_whole_disk_image' is not supported by trusted boot, because there is
     # no Kernel/Ramdisk to measure at all.
-    if (boot_mode != 'bios' or
-        is_whole_disk_image or
-        boot_option != 'netboot'):
+    if (boot_mode != 'bios'
+        or is_whole_disk_image
+        or boot_option != 'netboot'):
         msg = (_("Trusted boot is only supported in BIOS boot mode with "
                  "netboot and without whole_disk_image, but Node "
                  "%(node_uuid)s was configured with boot_mode: %(boot_mode)s, "
@@ -405,8 +417,9 @@ def _clean_up_pxe_env(task, images_info):
 
 class PXEBoot(base.BootInterface):
 
+    capabilities = ['iscsi_volume_boot']
+
     def __init__(self):
-        self.capabilities = ['iscsi_volume_boot']
         if CONF.pxe.ipxe_enabled:
             pxe_utils.create_ipxe_boot_script()
 
@@ -415,6 +428,9 @@ class PXEBoot(base.BootInterface):
 
         :returns: dictionary of <property name>:<property description> entries.
         """
+        # TODO(stendulker): COMMON_PROPERTIES should also include rescue
+        # related properties (RESCUE_PROPERTIES). We can add them in Rocky,
+        # when classic drivers get removed.
         return COMMON_PROPERTIES
 
     @METRICS.timer('PXEBoot.validate')
@@ -439,8 +455,8 @@ class PXEBoot(base.BootInterface):
                 % node.uuid)
 
         if CONF.pxe.ipxe_enabled:
-            if (not CONF.deploy.http_url or
-                not CONF.deploy.http_root):
+            if (not CONF.deploy.http_url
+                or not CONF.deploy.http_root):
                 raise exception.MissingParameterValue(_(
                     "iPXE boot is enabled but no HTTP URL or HTTP "
                     "root was specified."))
@@ -459,8 +475,8 @@ class PXEBoot(base.BootInterface):
             return
 
         d_info = deploy_utils.get_image_instance_info(node)
-        if (node.driver_internal_info.get('is_whole_disk_image') or
-                deploy_utils.get_boot_option(node) == 'local'):
+        if (node.driver_internal_info.get('is_whole_disk_image')
+                or deploy_utils.get_boot_option(node) == 'local'):
             props = []
         elif service_utils.is_glance_image(d_info['image_source']):
             props = ['kernel_id', 'ramdisk_id']
@@ -513,6 +529,7 @@ class PXEBoot(base.BootInterface):
         # if we are in DEPLOYING state.
         if node.provision_state == states.DEPLOYING:
             pxe_info.update(_get_instance_image_info(node, task.context))
+            boot_mode_utils.sync_boot_mode(task)
 
         pxe_options = _build_pxe_config_options(task, pxe_info)
         pxe_options.update(ramdisk_params)
@@ -575,6 +592,8 @@ class PXEBoot(base.BootInterface):
         :param task: a task from TaskManager.
         :returns: None
         """
+        boot_mode_utils.sync_boot_mode(task)
+
         node = task.node
         boot_option = deploy_utils.get_boot_option(node)
         boot_device = None
@@ -595,7 +614,7 @@ class PXEBoot(base.BootInterface):
                     task, pxe_options, pxe_config_template)
             deploy_utils.switch_pxe_config(
                 pxe_config_path, None,
-                deploy_utils.get_boot_mode_for_deploy(node), False,
+                boot_mode_utils.get_boot_mode_for_deploy(node), False,
                 iscsi_boot=True)
             boot_device = boot_devices.PXE
 
@@ -630,7 +649,10 @@ class PXEBoot(base.BootInterface):
                     LOG.warning("The disk id for the whole disk image can't "
                                 "be found, unable to switch the pxe config "
                                 "from deployment mode to service (boot) mode "
-                                "for node %(node)s", {"node": task.node.uuid})
+                                "for node %(node)s. Booting the instance "
+                                "from disk.", {"node": task.node.uuid})
+                    pxe_utils.clean_up_pxe_config(task)
+                    boot_device = boot_devices.DISK
             else:
                 _build_service_pxe_config(task, instance_image_info,
                                           root_uuid_or_disk_id)
@@ -661,6 +683,7 @@ class PXEBoot(base.BootInterface):
         :returns: None
         """
         node = task.node
+
         try:
             images_info = _get_instance_image_info(node, task.context)
         except exception.MissingParameterValue as e:
@@ -669,3 +692,13 @@ class PXEBoot(base.BootInterface):
                         {'node': node.uuid, 'err': e})
         else:
             _clean_up_pxe_env(task, images_info)
+
+    @METRICS.timer('PXEBoot.validate_rescue')
+    def validate_rescue(self, task):
+        """Validate that the node has required properties for rescue.
+
+        :param task: a TaskManager instance with the node being checked
+        :raises: MissingParameterValue if node is missing one or more required
+            parameters
+        """
+        _parse_driver_info(task.node, mode='rescue')
