@@ -88,8 +88,8 @@ class IronicObject(object_base.VersionedObject):
         object.
         """
         for field in self.fields:
-            if (self.obj_attr_is_set(field) and
-                    self[field] != loaded_object[field]):
+            if (self.obj_attr_is_set(field)
+                    and self[field] != loaded_object[field]):
                 self[field] = loaded_object[field]
 
     def _convert_to_version(self, target_version,
@@ -167,8 +167,8 @@ class IronicObject(object_base.VersionedObject):
         # is supported by this service. self.VERSION is the version of
         # this object instance -- it may get set via e.g. the
         # serialization or deserialization process, or here.
-        if (self.__class__.VERSION != target_version or
-            self.VERSION != self.__class__.VERSION):
+        if (self.__class__.VERSION != target_version
+            or self.VERSION != self.__class__.VERSION):
             self.VERSION = target_version
 
     @classmethod
@@ -254,17 +254,6 @@ class IronicObject(object_base.VersionedObject):
         objname = obj.obj_name()
         db_version = db_object['version']
 
-        if db_version is None:
-            # NOTE(rloo): This can only happen if the DB is corrupt or this
-            # is the conductor object. (Because the rest of the objects will
-            # all have their DB version set properly.)
-            # TODO(rloo): This entire if clause can be deleted in Rocky
-            # since the dbsync online migration populates all the conductor
-            # versions and it must be run to completion before upgrading to
-            # Rocky.
-            db_version = versions.RELEASE_MAPPING['pike']['objects'].get(
-                objname, ['1.0'])[0]
-
         if not versionutils.is_compatible(db_version, obj.__class__.VERSION):
             raise ovo_exception.IncompatibleObjectVersion(
                 objname=objname, objver=db_version,
