@@ -59,7 +59,9 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
     # Version 1.21: Add storage_interface field
     # Version 1.22: Add rescue_interface field
     # Version 1.23: Add traits field
-    VERSION = '1.23'
+    # Version 1.24: Add bios_interface field
+    # Version 1.25: Add fault field
+    VERSION = '1.25'
 
     dbapi = db_api.get_instance()
 
@@ -105,6 +107,7 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
 
         'maintenance': object_fields.BooleanField(),
         'maintenance_reason': object_fields.StringField(nullable=True),
+        'fault': object_fields.StringField(nullable=True),
         'console_enabled': object_fields.BooleanField(),
 
         # Any error from the most recent (last) asynchronous transaction
@@ -119,6 +122,7 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
 
         'extra': object_fields.FlexibleDictField(nullable=True),
 
+        'bios_interface': object_fields.StringField(nullable=True),
         'boot_interface': object_fields.StringField(nullable=True),
         'console_interface': object_fields.StringField(nullable=True),
         'deploy_interface': object_fields.StringField(nullable=True),
@@ -130,7 +134,6 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
         'rescue_interface': object_fields.StringField(nullable=True),
         'storage_interface': object_fields.StringField(nullable=True),
         'vendor_interface': object_fields.StringField(nullable=True),
-
         'traits': object_fields.ObjectField('TraitList', nullable=True),
     }
 
@@ -410,7 +413,7 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
         :raises: BadRequest if fields contains a traits that are not None.
         """
         if 'traits' in fields:
-            # NOTE(mgoddard): Traits should be updated via the the node
+            # NOTE(mgoddard): Traits should be updated via the node
             # object's traits field, which is itself an object. We shouldn't
             # get here with changes to traits, as this should be handled by the
             # API. When services are pinned to Pike, we can get here with
@@ -462,6 +465,18 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
         node = cls._from_db_object(context, cls(), db_node)
         return node
 
+    def _convert_fault_field(self, target_version,
+                             remove_unavailable_fields=True):
+        fault_is_set = self.obj_attr_is_set('fault')
+        if target_version >= (1, 25):
+            if not fault_is_set:
+                self.fault = None
+        elif fault_is_set:
+            if remove_unavailable_fields:
+                delattr(self, 'fault')
+            elif self.fault is not None:
+                self.fault = None
+
     def _convert_to_version(self, target_version,
                             remove_unavailable_fields=True):
         """Convert to the target version.
@@ -476,6 +491,11 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
         Version 1.23: traits field was added. Its default value is
             None. For versions prior to this, it should be set to None (or
             removed).
+        Version 1.24: bios_interface field was added. Its default value is
+            None. For versions prior to this, it should be set to None (or
+            removed).
+        Version 1.25: fault field was added. For versions prior to
+            this, it should be removed.
 
         :param target_version: the desired version of the object
         :param remove_unavailable_fields: True to remove fields that are
@@ -511,6 +531,23 @@ class Node(base.IronicObject, object_base.VersionedObjectDictCompat):
             elif self.traits is not None:
                 self.traits = None
 
+        bios_iface_is_set = self.obj_attr_is_set('bios_interface')
+        if target_version >= (1, 24):
+            # Target version supports bios_interface.
+            if not bios_iface_is_set:
+                # Set it to its default value if it is not set.
+                self.bios_interface = None
+        elif bios_iface_is_set:
+            # Target version does not support bios_interface, and it is set.
+            if remove_unavailable_fields:
+                # (De)serialising: remove unavailable fields.
+                delattr(self, 'bios_interface')
+            elif self.bios_interface is not None:
+                # DB: set unavailable field to the default of None.
+                self.bios_interface = None
+
+        self._convert_fault_field(target_version, remove_unavailable_fields)
+
 
 @base.IronicObjectRegistry.register
 class NodePayload(notification.NotificationPayloadBase):
@@ -535,7 +572,9 @@ class NodePayload(notification.NotificationPayloadBase):
         'last_error': ('node', 'last_error'),
         'maintenance': ('node', 'maintenance'),
         'maintenance_reason': ('node', 'maintenance_reason'),
+        'fault': ('node', 'fault'),
         'name': ('node', 'name'),
+        'bios_interface': ('node', 'bios_interface'),
         'boot_interface': ('node', 'boot_interface'),
         'console_interface': ('node', 'console_interface'),
         'deploy_interface': ('node', 'deploy_interface'),
@@ -566,7 +605,9 @@ class NodePayload(notification.NotificationPayloadBase):
     # Version 1.4: Add storage interface field exposed via API.
     # Version 1.5: Add rescue interface field exposed via API.
     # Version 1.6: Add traits field exposed via API.
-    VERSION = '1.6'
+    # Version 1.7: Add fault field exposed via API.
+    # Version 1.8: Add bios interface field exposed via API.
+    VERSION = '1.8'
     fields = {
         'clean_step': object_fields.FlexibleDictField(nullable=True),
         'console_enabled': object_fields.BooleanField(nullable=True),
@@ -579,6 +620,8 @@ class NodePayload(notification.NotificationPayloadBase):
         'last_error': object_fields.StringField(nullable=True),
         'maintenance': object_fields.BooleanField(nullable=True),
         'maintenance_reason': object_fields.StringField(nullable=True),
+        'fault': object_fields.StringField(nullable=True),
+        'bios_interface': object_fields.StringField(nullable=True),
         'boot_interface': object_fields.StringField(nullable=True),
         'console_interface': object_fields.StringField(nullable=True),
         'deploy_interface': object_fields.StringField(nullable=True),
@@ -635,7 +678,9 @@ class NodeSetPowerStatePayload(NodePayload):
     # Version 1.4: Parent NodePayload version 1.4
     # Version 1.5: Parent NodePayload version 1.5
     # Version 1.6: Parent NodePayload version 1.6
-    VERSION = '1.6'
+    # Version 1.7: Parent NodePayload version 1.7
+    # Version 1.8: Parent NodePayload version 1.8
+    VERSION = '1.8'
 
     fields = {
         # "to_power" indicates the future target_power_state of the node. A
@@ -682,7 +727,9 @@ class NodeCorrectedPowerStatePayload(NodePayload):
     # Version 1.4: Parent NodePayload version 1.4
     # Version 1.5: Parent NodePayload version 1.5
     # Version 1.6: Parent NodePayload version 1.6
-    VERSION = '1.6'
+    # Version 1.7: Parent NodePayload version 1.7
+    # Version 1.8: Parent NodePayload version 1.8
+    VERSION = '1.8'
 
     fields = {
         'from_power': object_fields.StringField(nullable=True)
@@ -713,7 +760,9 @@ class NodeSetProvisionStatePayload(NodePayload):
     # Version 1.3: Parent NodePayload version 1.3
     # Version 1.4: Parent NodePayload version 1.4
     # Version 1.6: Parent NodePayload version 1.6
-    VERSION = '1.6'
+    # Version 1.7: Parent NodePayload version 1.7
+    # Version 1.8: Parent NodePayload version 1.8
+    VERSION = '1.8'
 
     SCHEMA = dict(NodePayload.SCHEMA,
                   **{'instance_info': ('node', 'instance_info')})
@@ -751,7 +800,9 @@ class NodeCRUDPayload(NodePayload):
     # Version 1.2: Parent NodePayload version 1.4
     # Version 1.3: Parent NodePayload version 1.5
     # Version 1.4: Parent NodePayload version 1.6
-    VERSION = '1.4'
+    # Version 1.5: Parent NodePayload version 1.7
+    # Version 1.6: Parent NodePayload version 1.8
+    VERSION = '1.6'
 
     SCHEMA = dict(NodePayload.SCHEMA,
                   **{'instance_info': ('node', 'instance_info'),
