@@ -26,6 +26,36 @@ from ironic.objects import fields as object_fields
 from ironic.objects import notification
 
 
+def migrate_vif_port_id(context, max_count):
+    """Copy port's VIF info from extra to internal_info.
+
+    :param context: The context.
+    :param max_count: The maximum number of objects to migrate. Must be
+                      >= 0. If zero, all the objects will be migrated.
+    :returns: A 2-tuple -- the total number of objects that need to be
+              migrated (at the beginning of this call) and the number
+              of migrated objects.
+    """
+    # TODO(rloo): remove this method in Stein, when we remove from dbsync.py
+
+    # NOTE(rloo): if we introduce newer port versions in the same cycle,
+    # we could add those versions along with 1.8. This is only so we don't
+    # duplicate work; it isn't necessary.
+    db_ports = Port.dbapi.get_not_versions('Port', ['1.8'])
+    total = len(db_ports)
+    max_count = max_count or total
+    done = 0
+    for db_port in db_ports:
+        # NOTE(rloo): this will indirectly invoke Port._convert_to_version()
+        #             which does all the real work
+        port = Port._from_db_object(context, Port(), db_port)
+        port.save()
+        done += 1
+        if done == max_count:
+            break
+    return total, done
+
+
 @base.IronicObjectRegistry.register
 class Port(base.IronicObject, object_base.VersionedObjectDictCompat):
     # Version 1.0: Initial version
@@ -38,7 +68,10 @@ class Port(base.IronicObject, object_base.VersionedObjectDictCompat):
     #              local_link_connection, portgroup_id and pxe_enabled
     # Version 1.6: Add internal_info field
     # Version 1.7: Add physical_network field
-    VERSION = '1.7'
+    # Version 1.8: Migrate/copy extra['vif_port_id'] to
+    #              internal_info['tenant_vif_port_id'] (not an explicit db
+    #              change)
+    VERSION = '1.8'
 
     dbapi = dbapi.get_instance()
 
@@ -68,6 +101,11 @@ class Port(base.IronicObject, object_base.VersionedObjectDictCompat):
             None. For versions prior to this, it should be set to None (or
             removed).
 
+        Version 1.8: if extra['vif_port_id'] is specified (non-null) and
+            internal_info['tenant_vif_port_id'] is not specified, copy the
+            .extra value to internal_info. There is nothing to do here when
+            downgrading to an older version.
+
         :param target_version: the desired version of the object
         :param remove_unavailable_fields: True to remove fields that are
             unavailable in the target version; set this to True when
@@ -75,6 +113,17 @@ class Port(base.IronicObject, object_base.VersionedObjectDictCompat):
             values; set this to False for DB interactions.
         """
         target_version = versionutils.convert_version_to_tuple(target_version)
+        if target_version >= (1, 8):
+            if self.obj_attr_is_set('extra'):
+                vif = self.extra.get('vif_port_id')
+                if vif:
+                    internal_info = (self.internal_info
+                                     if self.obj_attr_is_set('internal_info')
+                                     else {})
+                    if 'tenant_vif_port_id' not in internal_info:
+                        internal_info['tenant_vif_port_id'] = vif
+                        self.internal_info = internal_info
+
         # Convert the physical_network field.
         physnet_is_set = self.obj_attr_is_set('physical_network')
         if target_version >= (1, 7):
