@@ -171,6 +171,15 @@ class TestGlanceImageService(base.TestCase):
                           self.service.show,
                           uuidutils.generate_uuid())
 
+    def test_show_raises_when_image_not_active(self):
+        image_id = uuidutils.generate_uuid()
+        image = self._make_fixture(name='image1', is_public=True,
+                                   id=image_id, status="queued")
+        with mock.patch.object(self.service, 'call', return_value=image,
+                               autospec=True):
+            self.assertRaises(exception.ImageUnacceptable,
+                              self.service.show, image_id)
+
     @mock.patch.object(time, 'sleep', autospec=True)
     def test_download_with_retries(self, mock_sleep):
         tries = [0]
@@ -440,7 +449,6 @@ class CheckImageServiceTestCase(base.TestCase):
                     glance_cafile='cafile',
                     region_name=None,
                     group='glance')
-        self.config(region_name='OtherRegion', group='keystone')
 
         wrapped_func = base_image_service.check_image_service(func)
         self.assertEqual(((), params), wrapped_func(self.service, **params))
@@ -451,7 +459,7 @@ class CheckImageServiceTestCase(base.TestCase):
                                           cacert='cafile')
         mock_adapter.assert_called_once_with(
             'glance', session=mock.sentinel.session,
-            auth=mock.sentinel.auth, region_name='OtherRegion')
+            auth=mock.sentinel.auth)
         self.assertEqual(0, mock_sauth.call_count)
         mock_auth.assert_called_once_with('glance')
 
@@ -691,8 +699,8 @@ class TestGlanceSwiftTempURL(base.TestCase):
         temp_url = self.service.swift_temp_url(image_info=self.fake_image)
 
         self.assertEqual(
-            (urlparse.urljoin(CONF.glance.swift_endpoint_url, 'swift') +
-             tempurl_mock.return_value),
+            (urlparse.urljoin(CONF.glance.swift_endpoint_url, 'swift')
+             + tempurl_mock.return_value),
             temp_url)
         tempurl_mock.assert_called_with(
             path=path,
@@ -856,8 +864,8 @@ class TestSwiftTempUrlCache(base.TestCase):
         temp_url = self.glance_service.swift_temp_url(
             image_info=fake_image)
 
-        self.assertEqual(CONF.glance.swift_endpoint_url +
-                         tempurl_mock.return_value,
+        self.assertEqual(CONF.glance.swift_endpoint_url
+                         + tempurl_mock.return_value,
                          temp_url)
         cleanup_mock.assert_called_once_with()
         tempurl_mock.assert_called_with(
@@ -912,8 +920,8 @@ class TestSwiftTempUrlCache(base.TestCase):
         query = '?temp_url_sig=hmacsig&temp_url_expires=%s'
         self.glance_service._cache[fake_image['id']] = (
             glance_v2.TempUrlCacheElement(
-                url=(CONF.glance.swift_endpoint_url + path +
-                     query % old_exp_time),
+                url=(CONF.glance.swift_endpoint_url + path
+                     + query % old_exp_time),
                 url_expires_at=old_exp_time)
         )
 
@@ -926,8 +934,8 @@ class TestSwiftTempUrlCache(base.TestCase):
         fresh_temp_url = self.glance_service.swift_temp_url(
             image_info=fake_image)
 
-        self.assertEqual(CONF.glance.swift_endpoint_url +
-                         tempurl_mock.return_value,
+        self.assertEqual(CONF.glance.swift_endpoint_url
+                         + tempurl_mock.return_value,
                          fresh_temp_url)
         tempurl_mock.assert_called_with(
             path=path,
@@ -985,8 +993,8 @@ class TestSwiftTempUrlCache(base.TestCase):
             image_id=fake_image['id']
         )
 
-        self.assertEqual(CONF.glance.swift_endpoint_url +
-                         tempurl_mock.return_value,
+        self.assertEqual(CONF.glance.swift_endpoint_url
+                         + tempurl_mock.return_value,
                          temp_url)
         tempurl_mock.assert_called_with(
             path=path,
