@@ -43,11 +43,6 @@ class HashRingManagerTestCase(db_base.DbTestCase):
             self.dbapi.register_conductor_hardware_interfaces(
                 c.id, 'hardware-type', 'deploy', ['iscsi', 'direct'], 'iscsi')
 
-    def test_hash_ring_manager_get_ring_success(self):
-        self.register_conductors()
-        ring = self.ring_manager['driver1']
-        self.assertEqual(sorted(['host1', 'host2']), sorted(ring.nodes))
-
     def test_hash_ring_manager_hardware_type_success(self):
         self.register_conductors()
         ring = self.ring_manager['hardware-type']
@@ -59,34 +54,25 @@ class HashRingManagerTestCase(db_base.DbTestCase):
                           self.ring_manager.__getitem__,
                           'driver3')
 
-    def test_hash_ring_manager_automatic_retry(self):
-        self.assertRaises(exception.TemporaryFailure,
+    def test_hash_ring_manager_no_refresh(self):
+        # If a new conductor is registered after the ring manager is
+        # initialized, it won't be seen. Long term this is probably
+        # undesirable, but today is the intended behavior.
+        self.assertRaises(exception.DriverNotFound,
                           self.ring_manager.__getitem__,
-                          'driver1')
+                          'hardware-type')
         self.register_conductors()
-        self.assertIsNotNone(self.ring_manager['driver1'])
+        self.assertRaises(exception.DriverNotFound,
+                          self.ring_manager.__getitem__,
+                          'hardware-type')
 
-    def test_hash_ring_manager_reset_interval(self):
+    def test_hash_ring_manager_refresh(self):
         CONF.set_override('hash_ring_reset_interval', 30)
-        # Just to simplify calculations
-        CONF.set_override('hash_partition_exponent', 0)
-        self.dbapi.register_conductor({
-            'hostname': 'host1',
-            'drivers': ['driver1', 'driver2'],
-        })
-
-        ring = self.ring_manager['driver1']
-        self.assertEqual(1, len(ring))
-
-        self.dbapi.register_conductor({
-            'hostname': 'host2',
-            'drivers': ['driver1'],
-        })
-        ring = self.ring_manager['driver1']
-        # The new conductor is not known yet. Automatic retry does not kick in,
-        # since there is an active conductor for the requested driver.
-        self.assertEqual(1, len(ring))
-
+        # Initialize the ring manager to make _hash_rings not None, then
+        # hash ring will refresh only when time interval exceeded.
+        self.assertRaises(exception.DriverNotFound,
+                          self.ring_manager.__getitem__,
+                          'hardware-type')
+        self.register_conductors()
         self.ring_manager.updated_at = time.time() - 31
-        ring = self.ring_manager['driver1']
-        self.assertEqual(2, len(ring))
+        self.ring_manager.__getitem__('hardware-type')
