@@ -24,18 +24,18 @@ import six
 from ironic.common import pxe_utils
 from ironic.common import utils
 from ironic.conductor import task_manager
-from ironic.tests.unit.conductor import mgr_utils
 from ironic.tests.unit.db import base as db_base
 from ironic.tests.unit.objects import utils as object_utils
 
 CONF = cfg.CONF
 
 
+# Prevent /httpboot validation on creating the node
+@mock.patch('ironic.drivers.modules.pxe.PXEBoot.__init__', lambda self: None)
 class TestPXEUtils(db_base.DbTestCase):
 
     def setUp(self):
         super(TestPXEUtils, self).setUp()
-        mgr_utils.mock_the_extension_manager(driver="fake")
 
         self.pxe_options = {
             'deployment_aki_path': u'/tftpboot/1be26c0b-03f2-4d2e-ae87-'
@@ -65,16 +65,35 @@ class TestPXEUtils(db_base.DbTestCase):
             'ipxe_timeout': 120
         })
 
-        self.ipxe_options_boot_from_volume = self.ipxe_options.copy()
-        self.ipxe_options_boot_from_volume.update({
+        self.ipxe_options_boot_from_volume_no_extra_volume = \
+            self.ipxe_options.copy()
+        self.ipxe_options_boot_from_volume_no_extra_volume.update({
             'boot_from_volume': True,
             'iscsi_boot_url': 'iscsi:fake_host::3260:0:fake_iqn',
             'iscsi_initiator_iqn': 'fake_iqn',
-            'iscsi_volumes': ['iscsi:fake_host::3260:1:fake_iqn'],
+            'iscsi_volumes': [],
             'username': 'fake_username',
-            'password': 'fake_password'
+            'password': 'fake_password',
         })
-        self.ipxe_options_boot_from_volume.pop('initrd_filename', None)
+
+        self.ipxe_options_boot_from_volume_extra_volume = \
+            self.ipxe_options.copy()
+        self.ipxe_options_boot_from_volume_extra_volume.update({
+            'boot_from_volume': True,
+            'iscsi_boot_url': 'iscsi:fake_host::3260:0:fake_iqn',
+            'iscsi_initiator_iqn': 'fake_iqn',
+            'iscsi_volumes': [{'url': 'iscsi:fake_host::3260:1:fake_iqn',
+                               'username': 'fake_username_1',
+                               'password': 'fake_password_1',
+                               }],
+            'username': 'fake_username',
+            'password': 'fake_password',
+        })
+
+        self.ipxe_options_boot_from_volume_no_extra_volume.pop(
+            'initrd_filename', None)
+        self.ipxe_options_boot_from_volume_extra_volume.pop(
+            'initrd_filename', None)
 
         self.node = object_utils.create_test_node(self.context)
 
@@ -153,25 +172,25 @@ class TestPXEUtils(db_base.DbTestCase):
         self.config(http_url='http://1.2.3.4:1234', group='deploy')
         rendered_template = utils.render_template(
             CONF.pxe.pxe_config_template,
-            {'pxe_options': self.ipxe_options_boot_from_volume,
+            {'pxe_options': self.ipxe_options_boot_from_volume_extra_volume,
              'ROOT': '{{ ROOT }}',
              'DISK_IDENTIFIER': '{{ DISK_IDENTIFIER }}'})
 
         templ_file = 'ironic/tests/unit/drivers/' \
-                     'ipxe_config_boot_from_volume.template'
+                     'ipxe_config_boot_from_volume_extra_volume.template'
         with open(templ_file) as f:
             expected_template = f.read().rstrip()
 
         self.assertEqual(six.text_type(expected_template), rendered_template)
 
-    def test_default_ipxe_boot_from_volume_config_no_volumes(self):
+    def test_default_ipxe_boot_from_volume_config_no_extra_volumes(self):
         self.config(
             pxe_config_template='ironic/drivers/modules/ipxe_config.template',
             group='pxe'
         )
         self.config(http_url='http://1.2.3.4:1234', group='deploy')
 
-        pxe_options = self.ipxe_options_boot_from_volume
+        pxe_options = self.ipxe_options_boot_from_volume_no_extra_volume
         pxe_options['iscsi_volumes'] = []
 
         rendered_template = utils.render_template(
@@ -181,7 +200,7 @@ class TestPXEUtils(db_base.DbTestCase):
              'DISK_IDENTIFIER': '{{ DISK_IDENTIFIER }}'})
 
         templ_file = 'ironic/tests/unit/drivers/' \
-                     'ipxe_config_boot_from_volume_no_volumes.template'
+                     'ipxe_config_boot_from_volume_no_extra_volumes.template'
         with open(templ_file) as f:
             expected_template = f.read().rstrip()
         self.assertEqual(six.text_type(expected_template), rendered_template)
