@@ -30,6 +30,7 @@ from ironic.conductor import notification_utils
 from ironic.conductor import task_manager
 from ironic.drivers import fake_hardware
 from ironic.drivers import generic
+from ironic.drivers.modules import fake
 from ironic import objects
 from ironic.objects import fields
 from ironic.tests import base as tests_base
@@ -81,19 +82,19 @@ class StartStopTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
         self._start_service()
         self.service.del_host()
 
-    @mock.patch.object(driver_factory.DriverFactory, '__getitem__',
+    @mock.patch.object(driver_factory.HardwareTypesFactory, '__getitem__',
                        lambda *args: mock.MagicMock())
-    @mock.patch.object(driver_factory, 'StorageInterfaceFactory')
-    @mock.patch.object(driver_factory, 'NetworkInterfaceFactory')
-    def test_start_registers_driver_names(self, net_factory,
-                                          storage_factory):
+    @mock.patch.object(driver_factory, 'default_interface', autospec=True)
+    def test_start_registers_driver_names(self, mock_def_iface):
         init_names = ['fake1', 'fake2']
         restart_names = ['fake3', 'fake4']
 
-        df = driver_factory.DriverFactory()
+        mock_def_iface.return_value = 'fake'
+
+        df = driver_factory.HardwareTypesFactory()
         with mock.patch.object(df._extension_manager, 'names') as mock_names:
             # verify driver names are registered
-            self.config(enabled_drivers=init_names)
+            self.config(enabled_hardware_types=init_names)
             mock_names.return_value = init_names
             self._start_service()
             res = objects.Conductor.get_by_hostname(self.context,
@@ -102,20 +103,36 @@ class StartStopTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
             self._stop_service()
 
             # verify that restart registers new driver names
-            self.config(enabled_drivers=restart_names)
+            self.config(enabled_hardware_types=restart_names)
             mock_names.return_value = restart_names
             self._start_service()
             res = objects.Conductor.get_by_hostname(self.context,
                                                     self.hostname)
             self.assertEqual(restart_names, res['drivers'])
-        self.assertEqual(2, net_factory.call_count)
-        self.assertEqual(2, storage_factory.call_count)
 
+    @mock.patch.object(base_manager.BaseConductorManager,
+                       '_register_and_validate_hardware_interfaces',
+                       autospec=True)
     @mock.patch.object(driver_factory, 'all_interfaces', autospec=True)
     @mock.patch.object(driver_factory, 'hardware_types', autospec=True)
-    @mock.patch.object(driver_factory, 'drivers', autospec=True)
-    def test_start_registers_driver_specific_tasks(self, mock_drivers,
-                                                   mock_hw_types, mock_ifaces):
+    def test_start_registers_driver_specific_tasks(self,
+                                                   mock_hw_types, mock_ifaces,
+                                                   mock_reg_hw_ifaces):
+        class TestHwType(generic.GenericHardware):
+            @property
+            def supported_management_interfaces(self):
+                return []
+
+            @property
+            def supported_power_interfaces(self):
+                return []
+
+            # This should not be collected, since we don't collect periodic
+            # tasks from hardware types
+            @periodics.periodic(spacing=100500)
+            def task(self):
+                pass
+
         class TestInterface(object):
             @periodics.periodic(spacing=100500)
             def iface(self):
@@ -126,23 +143,12 @@ class StartStopTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
             def iface(self):
                 pass
 
-        class Driver(object):
-            core_interfaces = []
-            standard_interfaces = ['iface']
-            all_interfaces = core_interfaces + standard_interfaces
-
-            iface = TestInterface()
-
-            @periodics.periodic(spacing=42)
-            def task(self, context):
-                pass
-
-        driver = Driver()
+        hw_type = TestHwType()
         iface1 = TestInterface()
         iface2 = TestInterface2()
         expected = [iface1.iface, iface2.iface]
 
-        mock_drivers.return_value = {'fake1': driver}
+        mock_hw_types.return_value = {'fake1': hw_type}
         mock_ifaces.return_value = {
             'management': {'fake1': iface1},
             'power': {'fake2': iface2}
@@ -155,11 +161,11 @@ class StartStopTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
             self.assertTrue(periodics.is_periodic(item))
             self.assertIn(item, tasks)
 
-        # no periodic tasks from the Driver object
-        self.assertTrue(periodics.is_periodic(driver.task))
-        self.assertNotIn(driver.task, tasks)
+        # no periodic tasks from the hardware type
+        self.assertTrue(periodics.is_periodic(hw_type.task))
+        self.assertNotIn(hw_type.task, tasks)
 
-    @mock.patch.object(driver_factory.DriverFactory, '__init__')
+    @mock.patch.object(driver_factory.HardwareTypesFactory, '__init__')
     def test_start_fails_on_missing_driver(self, mock_df):
         mock_df.side_effect = exception.DriverNotFound('test')
         with mock.patch.object(self.dbapi, 'register_conductor') as mock_reg:
@@ -174,53 +180,16 @@ class StartStopTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
                                'options enabled_boot_interfaces',
                                self.service.init_host)
 
-    def test_starts_without_enabled_hardware_types(self):
-        self.config(enabled_hardware_types=[])
-        self.config(enabled_boot_interfaces=[])
-        self._start_service()
-
     @mock.patch.object(base_manager, 'LOG')
     @mock.patch.object(driver_factory, 'HardwareTypesFactory')
-    @mock.patch.object(driver_factory, 'DriverFactory')
-    def test_start_fails_on_no_driver_or_hw_types(self, df_mock, ht_mock,
-                                                  log_mock):
+    def test_start_fails_on_hw_types(self, ht_mock, log_mock):
         driver_factory_mock = mock.MagicMock(names=[])
-        df_mock.return_value = driver_factory_mock
         ht_mock.return_value = driver_factory_mock
         self.assertRaises(exception.NoDriversLoaded,
                           self.service.init_host)
         self.assertTrue(log_mock.error.called)
-        df_mock.assert_called_once_with()
         ht_mock.assert_called_once_with()
 
-    @mock.patch.object(base_manager, 'LOG')
-    @mock.patch.object(base_manager.BaseConductorManager, 'del_host',
-                       autospec=True)
-    @mock.patch.object(driver_factory, 'DriverFactory')
-    def test_starts_with_only_dynamic_drivers(self, df_mock, del_mock,
-                                              log_mock):
-        # don't load any classic drivers
-        driver_factory_mock = mock.MagicMock(names=[])
-        df_mock.return_value = driver_factory_mock
-        self.service.init_host()
-        self.assertFalse(log_mock.error.called)
-        df_mock.assert_called_with()
-        self.assertFalse(del_mock.called)
-
-    @mock.patch.object(base_manager, 'LOG')
-    @mock.patch.object(base_manager.BaseConductorManager, 'del_host',
-                       autospec=True)
-    @mock.patch.object(driver_factory, 'HardwareTypesFactory')
-    def test_starts_with_only_classic_drivers(self, ht_mock, del_mock,
-                                              log_mock):
-        # don't load any dynamic drivers
-        driver_factory_mock = mock.MagicMock(names=[])
-        ht_mock.return_value = driver_factory_mock
-        self.service.init_host()
-        self.assertFalse(log_mock.error.called)
-        ht_mock.assert_called_with()
-        self.assertFalse(del_mock.called)
-
     @mock.patch.object(base_manager, 'LOG')
     @mock.patch.object(base_manager.BaseConductorManager,
                        '_register_and_validate_hardware_interfaces')
@@ -232,19 +201,6 @@ class StartStopTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
         self.assertTrue(log_mock.error.called)
         del_mock.assert_called_once_with()
 
-    @mock.patch.object(base_manager, 'LOG')
-    @mock.patch.object(driver_factory, 'HardwareTypesFactory')
-    @mock.patch.object(driver_factory, 'DriverFactory')
-    def test_start_fails_on_name_conflict(self, df_mock, ht_mock, log_mock):
-        driver_factory_mock = mock.MagicMock(names=['dupe-driver'])
-        df_mock.return_value = driver_factory_mock
-        ht_mock.return_value = driver_factory_mock
-        self.assertRaises(exception.DriverNameConflict,
-                          self.service.init_host)
-        self.assertTrue(log_mock.error.called)
-        df_mock.assert_called_once_with()
-        ht_mock.assert_called_once_with()
-
     def test_prevent_double_start(self):
         self._start_service()
         self.assertRaisesRegex(RuntimeError, 'already running',
@@ -301,11 +257,6 @@ class CheckInterfacesTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
                                'options enabled_boot_interfaces',
                                base_manager._check_enabled_interfaces)
 
-    def test__check_enabled_interfaces_skip_if_no_hw_types(self):
-        self.config(enabled_hardware_types=[])
-        self.config(enabled_boot_interfaces=[])
-        base_manager._check_enabled_interfaces()
-
 
 class KeepAliveTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
     def test__conductor_service_record_keepalive(self):
@@ -447,76 +398,85 @@ class RegisterInterfacesTestCase(mgr_utils.ServiceSetUpMixin,
         self.assertFalse(reg_mock.called)
 
 
+@mock.patch.object(fake.FakeConsole, 'start_console', autospec=True)
+@mock.patch.object(notification_utils, 'emit_console_notification')
 class StartConsolesTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
-    @mock.patch.object(notification_utils, 'emit_console_notification')
-    def test__start_consoles(self, mock_notify):
+    def test__start_consoles(self, mock_notify, mock_start_console):
         obj_utils.create_test_node(self.context,
-                                   driver='fake',
+                                   driver='fake-hardware',
                                    console_enabled=True)
         obj_utils.create_test_node(
             self.context,
             uuid=uuidutils.generate_uuid(),
-            driver='fake',
+            driver='fake-hardware',
             console_enabled=True
         )
         obj_utils.create_test_node(
             self.context,
             uuid=uuidutils.generate_uuid(),
-            driver='fake'
+            driver='fake-hardware'
         )
         self._start_service()
-        with mock.patch.object(self.driver.console,
-                               'start_console') as mock_start_console:
-            self.service._start_consoles(self.context)
-            self.assertEqual(2, mock_start_console.call_count)
-            mock_notify.assert_has_calls(
-                [mock.call(mock.ANY, 'console_restore',
-                           fields.NotificationStatus.START),
-                 mock.call(mock.ANY, 'console_restore',
-                           fields.NotificationStatus.END)])
+        self.service._start_consoles(self.context)
+        self.assertEqual(2, mock_start_console.call_count)
+        mock_notify.assert_has_calls(
+            [mock.call(mock.ANY, 'console_restore',
+                       fields.NotificationStatus.START),
+             mock.call(mock.ANY, 'console_restore',
+                       fields.NotificationStatus.END)])
 
-    @mock.patch.object(notification_utils, 'emit_console_notification')
-    def test__start_consoles_no_console_enabled(self, mock_notify):
+    def test__start_consoles_no_console_enabled(self, mock_notify,
+                                                mock_start_console):
         obj_utils.create_test_node(self.context,
-                                   driver='fake',
+                                   driver='fake-hardware',
                                    console_enabled=False)
         self._start_service()
-        with mock.patch.object(self.driver.console,
-                               'start_console') as mock_start_console:
-            self.service._start_consoles(self.context)
-            self.assertFalse(mock_start_console.called)
-            self.assertFalse(mock_notify.called)
+        self.service._start_consoles(self.context)
+        self.assertFalse(mock_start_console.called)
+        self.assertFalse(mock_notify.called)
 
-    @mock.patch.object(notification_utils, 'emit_console_notification')
-    def test__start_consoles_failed(self, mock_notify):
+    def test__start_consoles_failed(self, mock_notify, mock_start_console):
         test_node = obj_utils.create_test_node(self.context,
-                                               driver='fake',
+                                               driver='fake-hardware',
                                                console_enabled=True)
         self._start_service()
-        with mock.patch.object(self.driver.console,
-                               'start_console') as mock_start_console:
-            mock_start_console.side_effect = Exception()
-            self.service._start_consoles(self.context)
-            mock_start_console.assert_called_once_with(mock.ANY)
-            test_node.refresh()
-            self.assertFalse(test_node.console_enabled)
-            self.assertIsNotNone(test_node.last_error)
-            mock_notify.assert_has_calls(
-                [mock.call(mock.ANY, 'console_restore',
-                           fields.NotificationStatus.START),
-                 mock.call(mock.ANY, 'console_restore',
-                           fields.NotificationStatus.ERROR)])
+        mock_start_console.side_effect = Exception()
+        self.service._start_consoles(self.context)
+        mock_start_console.assert_called_once_with(mock.ANY, mock.ANY)
+        test_node.refresh()
+        self.assertFalse(test_node.console_enabled)
+        self.assertIsNotNone(test_node.last_error)
+        mock_notify.assert_has_calls(
+            [mock.call(mock.ANY, 'console_restore',
+                       fields.NotificationStatus.START),
+             mock.call(mock.ANY, 'console_restore',
+                       fields.NotificationStatus.ERROR)])
 
-    @mock.patch.object(notification_utils, 'emit_console_notification')
     @mock.patch.object(base_manager, 'LOG')
-    def test__start_consoles_node_locked(self, log_mock, mock_notify):
+    def test__start_consoles_node_locked(self, log_mock, mock_notify,
+                                         mock_start_console):
         test_node = obj_utils.create_test_node(self.context,
-                                               driver='fake',
+                                               driver='fake-hardware',
                                                console_enabled=True,
                                                reservation='fake-host')
         self._start_service()
-        with mock.patch.object(self.driver.console,
-                               'start_console') as mock_start_console:
+        self.service._start_consoles(self.context)
+        self.assertFalse(mock_start_console.called)
+        test_node.refresh()
+        self.assertTrue(test_node.console_enabled)
+        self.assertIsNone(test_node.last_error)
+        self.assertTrue(log_mock.warning.called)
+        self.assertFalse(mock_notify.called)
+
+    @mock.patch.object(base_manager, 'LOG')
+    def test__start_consoles_node_not_found(self, log_mock, mock_notify,
+                                            mock_start_console):
+        test_node = obj_utils.create_test_node(self.context,
+                                               driver='fake-hardware',
+                                               console_enabled=True)
+        self._start_service()
+        with mock.patch.object(task_manager, 'acquire') as mock_acquire:
+            mock_acquire.side_effect = exception.NodeNotFound(node='not found')
             self.service._start_consoles(self.context)
             self.assertFalse(mock_start_console.called)
             test_node.refresh()
@@ -524,22 +484,3 @@ class StartConsolesTestCase(mgr_utils.ServiceSetUpMixin, db_base.DbTestCase):
             self.assertIsNone(test_node.last_error)
             self.assertTrue(log_mock.warning.called)
             self.assertFalse(mock_notify.called)
-
-    @mock.patch.object(notification_utils, 'emit_console_notification')
-    @mock.patch.object(base_manager, 'LOG')
-    def test__start_consoles_node_not_found(self, log_mock, mock_notify):
-        test_node = obj_utils.create_test_node(self.context,
-                                               driver='fake',
-                                               console_enabled=True)
-        self._start_service()
-        with mock.patch.object(task_manager, 'acquire') as mock_acquire:
-            mock_acquire.side_effect = exception.NodeNotFound(node='not found')
-            with mock.patch.object(self.driver.console,
-                                   'start_console') as mock_start_console:
-                self.service._start_consoles(self.context)
-                self.assertFalse(mock_start_console.called)
-                test_node.refresh()
-                self.assertTrue(test_node.console_enabled)
-                self.assertIsNone(test_node.last_error)
-                self.assertTrue(log_mock.warning.called)
-                self.assertFalse(mock_notify.called)
