@@ -193,97 +193,6 @@ class DbConductorTestCase(base.DbTestCase):
         self.assertEqual('power on', node3.target_power_state)
         self.assertIsNone(node3.last_error)
 
-    @mock.patch.object(timeutils, 'utcnow', autospec=True)
-    def test_get_active_driver_dict_one_host_no_driver(self, mock_utcnow):
-        h = 'fake-host'
-        expected = {}
-
-        mock_utcnow.return_value = datetime.datetime.utcnow()
-        self._create_test_cdr(hostname=h, drivers=[])
-        result = self.dbapi.get_active_driver_dict()
-        self.assertEqual(expected, result)
-
-    @mock.patch.object(timeutils, 'utcnow', autospec=True)
-    def test_get_active_driver_dict_one_host_one_driver(self, mock_utcnow):
-        h = 'fake-host'
-        d = 'fake-driver'
-        expected = {d: {h}}
-
-        mock_utcnow.return_value = datetime.datetime.utcnow()
-        self._create_test_cdr(hostname=h, drivers=[d])
-        result = self.dbapi.get_active_driver_dict()
-        self.assertEqual(expected, result)
-
-    @mock.patch.object(timeutils, 'utcnow', autospec=True)
-    def test_get_active_driver_dict_one_host_many_drivers(self, mock_utcnow):
-        h = 'fake-host'
-        d1 = 'driver-one'
-        d2 = 'driver-two'
-        expected = {d1: {h}, d2: {h}}
-
-        mock_utcnow.return_value = datetime.datetime.utcnow()
-        self._create_test_cdr(hostname=h, drivers=[d1, d2])
-        result = self.dbapi.get_active_driver_dict()
-        self.assertEqual(expected, result)
-
-    @mock.patch.object(timeutils, 'utcnow', autospec=True)
-    def test_get_active_driver_dict_many_hosts_one_driver(self, mock_utcnow):
-        h1 = 'host-one'
-        h2 = 'host-two'
-        d = 'fake-driver'
-        expected = {d: {h1, h2}}
-
-        mock_utcnow.return_value = datetime.datetime.utcnow()
-        self._create_test_cdr(id=1, hostname=h1, drivers=[d])
-        self._create_test_cdr(id=2, hostname=h2, drivers=[d])
-        result = self.dbapi.get_active_driver_dict()
-        self.assertEqual(expected, result)
-
-    @mock.patch.object(timeutils, 'utcnow', autospec=True)
-    def test_get_active_driver_dict_many_hosts_and_drivers(self, mock_utcnow):
-        h1 = 'host-one'
-        h2 = 'host-two'
-        h3 = 'host-three'
-        d1 = 'driver-one'
-        d2 = 'driver-two'
-        expected = {d1: {h1, h2}, d2: {h2, h3}}
-
-        mock_utcnow.return_value = datetime.datetime.utcnow()
-        self._create_test_cdr(id=1, hostname=h1, drivers=[d1])
-        self._create_test_cdr(id=2, hostname=h2, drivers=[d1, d2])
-        self._create_test_cdr(id=3, hostname=h3, drivers=[d2])
-        result = self.dbapi.get_active_driver_dict()
-        self.assertEqual(expected, result)
-
-    @mock.patch.object(timeutils, 'utcnow', autospec=True)
-    def test_get_active_driver_dict_with_old_conductor(self, mock_utcnow):
-        past = datetime.datetime(2000, 1, 1, 0, 0)
-        present = past + datetime.timedelta(minutes=2)
-
-        d = 'common-driver'
-
-        h1 = 'old-host'
-        d1 = 'old-driver'
-        mock_utcnow.return_value = past
-        self._create_test_cdr(id=1, hostname=h1, drivers=[d, d1])
-
-        h2 = 'new-host'
-        d2 = 'new-driver'
-        mock_utcnow.return_value = present
-        self._create_test_cdr(id=2, hostname=h2, drivers=[d, d2])
-
-        # verify that old-host does not show up in current list
-        one_minute = 60
-        expected = {d: {h2}, d2: {h2}}
-        result = self.dbapi.get_active_driver_dict(interval=one_minute)
-        self.assertEqual(expected, result)
-
-        # change the interval, and verify that old-host appears
-        two_minute = one_minute * 2
-        expected = {d: {h1, h2}, d1: {h1}, d2: {h2}}
-        result = self.dbapi.get_active_driver_dict(interval=two_minute)
-        self.assertEqual(expected, result)
-
     @mock.patch.object(timeutils, 'utcnow', autospec=True)
     def test_get_active_hardware_type_dict_one_host_no_ht(self, mock_utcnow):
         h = 'fake-host'
