@@ -20,6 +20,7 @@ from oslo_utils import timeutils
 from ironic.common import states
 from ironic.db import api as db_api
 from ironic.drivers import base as drivers_base
+from ironic.objects import bios
 from ironic.objects import chassis
 from ironic.objects import conductor
 from ironic.objects import node
@@ -137,7 +138,14 @@ def get_test_snmp_info(**kw):
     if result["snmp_version"] in ("1", "2c"):
         result["snmp_community"] = kw.get("snmp_community", "public")
     elif result["snmp_version"] == "3":
-        result["snmp_security"] = kw.get("snmp_security", "public")
+        result["snmp_user"] = kw.get(
+            "snmp_user", kw.get("snmp_security", "snmpuser")
+        )
+        for option in ('snmp_auth_protocol', 'snmp_auth_key',
+                       'snmp_priv_protocol', 'snmp_priv_key',
+                       'snmp_context_engine_id', 'snmp_context_name'):
+            if option in kw:
+                result[option] = kw[option]
     return result
 
 
@@ -179,7 +187,7 @@ def get_test_node(**kw):
         'last_error': kw.get('last_error'),
         'instance_uuid': kw.get('instance_uuid'),
         'instance_info': kw.get('instance_info', fake_instance_info),
-        'driver': kw.get('driver', 'fake'),
+        'driver': kw.get('driver', 'fake-hardware'),
         'driver_info': kw.get('driver_info', fake_driver_info),
         'driver_internal_info': kw.get('driver_internal_info',
                                        fake_internal_info),
@@ -188,6 +196,7 @@ def get_test_node(**kw):
         'reservation': kw.get('reservation'),
         'maintenance': kw.get('maintenance', False),
         'maintenance_reason': kw.get('maintenance_reason'),
+        'fault': kw.get('fault'),
         'console_enabled': kw.get('console_enabled', False),
         'extra': kw.get('extra', {}),
         'updated_at': kw.get('updated_at'),
@@ -506,6 +515,10 @@ def get_test_xclarity_properties():
 
 def get_test_xclarity_driver_info():
     return {
+        'xclarity_manager_ip': "1.2.3.4",
+        'xclarity_username': "USERID",
+        'xclarity_password': "fake",
+        'xclarity_port': 443,
         'xclarity_hardware_id': 'fake_sh_id',
     }
 
@@ -544,3 +557,40 @@ def create_test_node_traits(traits, **kw):
     :returns: a list of test NodeTrait DB objects.
     """
     return [create_test_node_trait(trait=trait, **kw) for trait in traits]
+
+
+def create_test_bios_setting(**kw):
+    """Create test bios entry in DB and return BIOSSetting DB object.
+
+    Function to be used to create test BIOSSetting object in the database.
+
+    :param kw: kwargs with overriding values for node bios settings.
+    :returns: Test BIOSSetting DB object.
+
+    """
+    bios_setting = get_test_bios_setting(**kw)
+    dbapi = db_api.get_instance()
+    node_id = bios_setting['node_id']
+    version = bios_setting['version']
+    settings = [{'name': bios_setting['name'],
+                 'value': bios_setting['value']}]
+    return dbapi.create_bios_setting_list(node_id, settings, version)[0]
+
+
+def get_test_bios_setting(**kw):
+    return {
+        'node_id': kw.get('node_id', '123'),
+        'name': kw.get('name', 'virtualization'),
+        'value': kw.get('value', 'on'),
+        'version': kw.get('version', bios.BIOSSetting.VERSION),
+        'created_at': kw.get('created_at'),
+        'updated_at': kw.get('updated_at'),
+    }
+
+
+def get_test_bios_setting_setting_list():
+    return [
+        {'name': 'virtualization', 'value': 'on'},
+        {'name': 'hyperthread', 'value': 'enabled'},
+        {'name': 'numlock', 'value': 'off'}
+    ]
