@@ -24,7 +24,6 @@ from ironic.drivers.modules.ansible import deploy as ansible_deploy
 from ironic.drivers.modules import deploy_utils
 from ironic.drivers.modules import fake
 from ironic.drivers.modules import pxe
-from ironic.tests.unit.conductor import mgr_utils
 from ironic.tests.unit.db import base as db_base
 from ironic.tests.unit.objects import utils as object_utils
 
@@ -56,22 +55,12 @@ class AnsibleDeployTestCaseBase(db_base.DbTestCase):
     def setUp(self):
         super(AnsibleDeployTestCaseBase, self).setUp()
 
-        # NOTE(pas-ha) we load ipmi hw type, and ipmitool interface checks
-        # for validity of the CONF.tempdir when a task object is created.
-        # We need to fake it in some tests and we don't care if and how it
-        # was called, so just mock it out.
-        # TODO(pas-ha) replace impi hw type with more appropriate fake hw type
-        # when such becomes available
-        patcher = mock.patch('ironic.common.utils.check_dir', autospec=True)
-        patcher.start()
-        self.addCleanup(patcher.stop)
-
-        self.config(enabled_deploy_interfaces='direct,iscsi,ansible')
-        mgr_utils.mock_the_extension_manager(driver='ipmi',
-                                             namespace='ironic.hardware.types')
+        self.config(enabled_hardware_types=['manual-management'],
+                    enabled_deploy_interfaces=['ansible'],
+                    enabled_power_interfaces=['fake'],
+                    enabled_management_interfaces=['fake'])
         node = {
-            'driver': 'ipmi',
-            'deploy_interface': 'ansible',
+            'driver': 'manual-management',
             'instance_info': INSTANCE_INFO,
             'driver_info': DRIVER_INFO,
             'driver_internal_info': DRIVER_INTERNAL_INFO,
@@ -103,26 +92,6 @@ class TestAnsibleMethods(AnsibleDeployTestCaseBase):
         self.assertEqual('spam', user)
         self.assertEqual('/ham/eggs', key)
 
-    @mock.patch.object(ansible_deploy.LOG, 'warning', autospec=True)
-    def test__parse_ansible_driver_info_deprecated_opts(self, warn_mock):
-        self.node.driver_info[
-            'ansible_deploy_username'] = self.node.driver_info.pop(
-                'ansible_username')
-        self.node.driver_info[
-            'ansible_deploy_key_file'] = self.node.driver_info.pop(
-                'ansible_key_file')
-        playbook, user, key = ansible_deploy._parse_ansible_driver_info(
-            self.node, 'deploy')
-        self.assertEqual(ansible_deploy.CONF.ansible.default_deploy_playbook,
-                         playbook)
-        self.assertEqual('test', user)
-        self.assertEqual('/path/key', key)
-        self.assertEqual(2, warn_mock.call_count)
-        # check that we remember about warnings havig been displayed
-        playbook, user, key = ansible_deploy._parse_ansible_driver_info(
-            self.node, 'deploy')
-        self.assertEqual(2, warn_mock.call_count)
-
     def test__parse_ansible_driver_info_no_playbook(self):
         self.assertRaises(exception.IronicException,
                           ansible_deploy._parse_ansible_driver_info,
@@ -411,7 +380,9 @@ class TestAnsibleMethods(AnsibleDeployTestCaseBase):
         i_info['configdrive'] = 'fake-content'
         self.node.instance_info = i_info
         self.node.save()
-        self.config(tempdir='/path/to/tmpfiles')
+        configdrive_path = ('%(tempdir)s/%(node)s.cndrive' %
+                            {'tempdir': ansible_deploy.CONF.tempdir,
+                             'node': self.node.uuid})
         expected = {"image": {"url": "http://image",
                               "validate_certs": "yes",
                               "source": "fake-image",
@@ -419,16 +390,14 @@ class TestAnsibleMethods(AnsibleDeployTestCaseBase):
                               "disk_format": "qcow2",
                               "checksum": "md5:checksum"},
                     'configdrive': {'type': 'file',
-                                    'location': '/path/to/tmpfiles/%s.cndrive'
-                                    % self.node.uuid}}
+                                    'location': configdrive_path}}
         with mock.patch.object(ansible_deploy, 'open', mock.mock_open(),
                                create=True) as open_mock:
             with task_manager.acquire(self.context, self.node.uuid) as task:
                 self.assertEqual(expected,
                                  ansible_deploy._prepare_variables(task))
             open_mock.assert_has_calls((
-                mock.call('/path/to/tmpfiles/%s.cndrive' % self.node.uuid,
-                          'w'),
+                mock.call(configdrive_path, 'w'),
                 mock.call().__enter__(),
                 mock.call().write('fake-content'),
                 mock.call().__exit__(None, None, None)))
@@ -513,8 +482,8 @@ class TestAnsibleDeploy(AnsibleDeployTestCaseBase):
 
     def test_get_properties(self):
         self.assertEqual(
-            set(list(ansible_deploy.COMMON_PROPERTIES) +
-                ['deploy_forces_oob_reboot']),
+            set(list(ansible_deploy.COMMON_PROPERTIES)
+                + ['deploy_forces_oob_reboot']),
             set(self.driver.get_properties()))
 
     @mock.patch.object(deploy_utils, 'check_for_missing_params',
@@ -674,10 +643,11 @@ class TestAnsibleDeploy(AnsibleDeployTestCaseBase):
     @mock.patch.object(ansible_deploy, '_parse_ansible_driver_info',
                        return_value=('test_pl', 'test_u', 'test_k'),
                        autospec=True)
+    @mock.patch.object(utils, 'cleaning_error_handler', autospec=True)
     @mock.patch.object(ansible_deploy, '_run_playbook', autospec=True)
     @mock.patch.object(ansible_deploy, 'LOG', autospec=True)
     def test_execute_clean_step_no_success_log(
-            self, log_mock, run_mock, parse_driver_info_mock):
+            self, log_mock, run_mock, utils_mock, parse_driver_info_mock):
 
         run_mock.side_effect = exception.InstanceDeployFailure('Boom')
         step = {'priority': 10, 'interface': 'deploy',
@@ -687,9 +657,11 @@ class TestAnsibleDeploy(AnsibleDeployTestCaseBase):
         self.node.driver_internal_info = di_info
         self.node.save()
         with task_manager.acquire(self.context, self.node.uuid) as task:
-            self.assertRaises(exception.InstanceDeployFailure,
-                              self.driver.execute_clean_step,
-                              task, step)
+            self.driver.execute_clean_step(task, step)
+            log_mock.error.assert_called_once_with(
+                mock.ANY, {'node': task.node['uuid'],
+                           'step': 'erase_devices'})
+            utils_mock.assert_called_once_with(task, 'Boom')
             self.assertFalse(log_mock.info.called)
 
     @mock.patch.object(ansible_deploy, '_run_playbook', autospec=True)
