@@ -24,6 +24,7 @@ from ironic_lib import utils as ironic_utils
 import mock
 from oslo_config import cfg
 from oslo_utils import importutils
+from oslo_utils import uuidutils
 import six
 import six.moves.builtins as __builtin__
 
@@ -35,7 +36,6 @@ from ironic.conductor import task_manager
 from ironic.conductor import utils as manager_utils
 from ironic.drivers.modules import deploy_utils
 from ironic.drivers.modules.ilo import common as ilo_common
-from ironic.tests.unit.conductor import mgr_utils
 from ironic.tests.unit.db import base as db_base
 from ironic.tests.unit.db import utils as db_utils
 from ironic.tests.unit.objects import utils as obj_utils
@@ -53,13 +53,28 @@ if six.PY3:
 CONF = cfg.CONF
 
 
-class IloValidateParametersTestCase(db_base.DbTestCase):
+class BaseIloTest(db_base.DbTestCase):
+
+    boot_interface = None
 
     def setUp(self):
-        super(IloValidateParametersTestCase, self).setUp()
+        super(BaseIloTest, self).setUp()
+        self.config(enabled_hardware_types=['ilo', 'fake-hardware'],
+                    enabled_boot_interfaces=['ilo-pxe', 'ilo-virtual-media',
+                                             'fake'],
+                    enabled_power_interfaces=['ilo', 'fake'],
+                    enabled_management_interfaces=['ilo', 'fake'],
+                    enabled_inspect_interfaces=['ilo', 'fake', 'no-inspect'],
+                    enabled_console_interfaces=['ilo', 'fake', 'no-console'],
+                    enabled_vendor_interfaces=['ilo', 'fake', 'no-vendor'])
+        self.info = INFO_DICT.copy()
         self.node = obj_utils.create_test_node(
-            self.context, driver='fake_ilo',
-            driver_info=INFO_DICT)
+            self.context, uuid=uuidutils.generate_uuid(),
+            driver='ilo', boot_interface=self.boot_interface,
+            driver_info=self.info)
+
+
+class IloValidateParametersTestCase(BaseIloTest):
 
     @mock.patch.object(os.path, 'isfile', return_value=True, autospec=True)
     def _test_parse_driver_info(self, isFile_mock):
@@ -210,14 +225,7 @@ class IloValidateParametersTestCase(db_base.DbTestCase):
         self.assertIn('client_timeout', str(e))
 
 
-class IloCommonMethodsTestCase(db_base.DbTestCase):
-
-    def setUp(self):
-        super(IloCommonMethodsTestCase, self).setUp()
-        mgr_utils.mock_the_extension_manager(driver="fake_ilo")
-        self.info = db_utils.get_test_ilo_info()
-        self.node = obj_utils.create_test_node(
-            self.context, driver='fake_ilo', driver_info=self.info)
+class IloCommonMethodsTestCase(BaseIloTest):
 
     @mock.patch.object(os.path, 'isfile', return_value=True, autospec=True)
     @mock.patch.object(ilo_client, 'IloClient', spec_set=True,
