@@ -18,11 +18,11 @@ Test class for common methods used by iRMC modules.
 
 import mock
 from oslo_config import cfg
+from oslo_utils import uuidutils
 
 from ironic.common import exception
 from ironic.conductor import task_manager
 from ironic.drivers.modules.irmc import common as irmc_common
-from ironic.tests.unit.conductor import mgr_utils
 from ironic.tests.unit.db import base as db_base
 from ironic.tests.unit.db import utils as db_utils
 from ironic.tests.unit.drivers import third_party_driver_mock_specs \
@@ -30,14 +30,27 @@ from ironic.tests.unit.drivers import third_party_driver_mock_specs \
 from ironic.tests.unit.objects import utils as obj_utils
 
 
-class IRMCValidateParametersTestCase(db_base.DbTestCase):
+class BaseIRMCTest(db_base.DbTestCase):
+
+    boot_interface = 'irmc-pxe'
 
     def setUp(self):
-        super(IRMCValidateParametersTestCase, self).setUp()
+        super(BaseIRMCTest, self).setUp()
+        self.config(enabled_hardware_types=['irmc', 'fake-hardware'],
+                    enabled_power_interfaces=['irmc', 'fake'],
+                    enabled_management_interfaces=['irmc', 'fake'],
+                    enabled_boot_interfaces=[self.boot_interface, 'fake'],
+                    enabled_inspect_interfaces=['irmc', 'no-inspect', 'fake'])
+        self.info = db_utils.get_test_irmc_info()
         self.node = obj_utils.create_test_node(
             self.context,
-            driver='fake_irmc',
-            driver_info=db_utils.get_test_irmc_info())
+            driver='irmc',
+            boot_interface=self.boot_interface,
+            driver_info=self.info,
+            uuid=uuidutils.generate_uuid())
+
+
+class IRMCValidateParametersTestCase(BaseIRMCTest):
 
     def test_parse_driver_info(self):
         info = irmc_common.parse_driver_info(self.node)
@@ -139,16 +152,7 @@ class IRMCValidateParametersTestCase(db_base.DbTestCase):
                           irmc_common.parse_driver_info, self.node)
 
 
-class IRMCCommonMethodsTestCase(db_base.DbTestCase):
-
-    def setUp(self):
-        super(IRMCCommonMethodsTestCase, self).setUp()
-        mgr_utils.mock_the_extension_manager(driver="fake_irmc")
-        self.info = db_utils.get_test_irmc_info()
-        self.node = obj_utils.create_test_node(
-            self.context,
-            driver='fake_irmc',
-            driver_info=self.info)
+class IRMCCommonMethodsTestCase(BaseIRMCTest):
 
     @mock.patch.object(irmc_common, 'scci',
                        spec_set=mock_specs.SCCICLIENT_IRMC_SCCI_SPEC)
