@@ -15,7 +15,6 @@
 
 import mock
 
-from ironic.common import driver_factory
 from ironic.common import exception
 from ironic.common import states
 from ironic.conductor import task_manager
@@ -25,23 +24,19 @@ from ironic.drivers.modules import iscsi_deploy
 from ironic.drivers.modules.oneview import common
 from ironic.drivers.modules.oneview import deploy
 from ironic.drivers.modules.oneview import deploy_utils
-from ironic.tests.unit.conductor import mgr_utils
-from ironic.tests.unit.db import base as db_base
-from ironic.tests.unit.db import utils as db_utils
+from ironic.tests.unit.drivers.modules.oneview import test_common
 from ironic.tests.unit.objects import utils as obj_utils
 
 METHODS = ['iter_nodes', 'update_node', 'do_provisioning_action']
-PXE_DRV_INFO_DICT = db_utils.get_test_pxe_driver_info()
-PXE_INST_INFO_DICT = db_utils.get_test_pxe_instance_info()
 
 oneview_error = common.SERVER_HARDWARE_ALLOCATION_ERROR
 maintenance_reason = common.NODE_IN_USE_BY_ONEVIEW
 
 driver_internal_info = {'oneview_error': oneview_error}
-nodes_taken_by_oneview = [(1, 'fake_oneview')]
-nodes_freed_by_oneview = [(1, 'fake_oneview', maintenance_reason)]
-nodes_taken_on_cleanfail = [(1, 'fake_oneview', driver_internal_info)]
-nodes_taken_on_cleanfail_no_info = [(1, 'fake_oneview', {})]
+nodes_taken_by_oneview = [(1, 'oneview')]
+nodes_freed_by_oneview = [(1, 'oneview', maintenance_reason)]
+nodes_taken_on_cleanfail = [(1, 'oneview', driver_internal_info)]
+nodes_taken_on_cleanfail_no_info = [(1, 'oneview', {})]
 
 GET_POWER_STATE_RETRIES = 5
 
@@ -79,12 +74,12 @@ def _setup_node_in_cleanfailed_state_without_oneview_error(node):
 
 
 class OneViewDriverDeploy(deploy.OneViewPeriodicTasks):
-    oneview_driver = 'fake_oneview'
+    oneview_driver = 'oneview'
 
 
 @mock.patch('ironic.objects.Node', spec_set=True, autospec=True)
 @mock.patch.object(deploy_utils, 'is_node_in_use_by_oneview')
-class OneViewPeriodicTasks(db_base.DbTestCase):
+class OneViewPeriodicTasks(test_common.BaseOneViewTest):
 
     def setUp(self):
         super(OneViewPeriodicTasks, self).setUp()
@@ -92,15 +87,8 @@ class OneViewPeriodicTasks(db_base.DbTestCase):
         self.config(username='user', group='oneview')
         self.config(password='password', group='oneview')
 
-        mgr_utils.mock_the_extension_manager(driver='fake_oneview')
-        self.driver = driver_factory.get_driver('fake_oneview')
         self.deploy = OneViewDriverDeploy()
         self.os_primary = mock.MagicMock(spec=METHODS)
-        self.node = obj_utils.create_test_node(
-            self.context, driver='fake_oneview',
-            properties=db_utils.get_test_oneview_properties(),
-            driver_info=db_utils.get_test_oneview_driver_info(),
-        )
 
     def test_node_manageable_maintenance_when_in_use_by_oneview(
         self, mock_is_node_in_use_by_oneview, mock_node_get
@@ -239,7 +227,9 @@ class OneViewPeriodicTasks(db_base.DbTestCase):
         self.assertNotIn('oneview_error', self.node.driver_internal_info)
 
 
-class OneViewIscsiDeployTestCase(db_base.DbTestCase):
+class OneViewIscsiDeployTestCase(test_common.BaseOneViewTest):
+
+    deploy_interface = 'oneview-iscsi'
 
     def setUp(self):
         super(OneViewIscsiDeployTestCase, self).setUp()
@@ -247,24 +237,14 @@ class OneViewIscsiDeployTestCase(db_base.DbTestCase):
         self.config(username='user', group='oneview')
         self.config(password='password', group='oneview')
 
-        mgr_utils.mock_the_extension_manager(driver='iscsi_pxe_oneview')
-        self.driver = driver_factory.get_driver('iscsi_pxe_oneview')
-
-        OV_DRV_INFO_DICT = db_utils.get_test_oneview_driver_info()
-        OV_DRV_INFO_DICT.update(PXE_DRV_INFO_DICT)
-        self.node = obj_utils.create_test_node(
-            self.context, driver='iscsi_pxe_oneview',
-            properties=db_utils.get_test_oneview_properties(),
-            driver_info=OV_DRV_INFO_DICT,
-            instance_info=PXE_INST_INFO_DICT,
-        )
         self.port = obj_utils.create_test_port(self.context,
                                                node_id=self.node.id)
         self.info = common.get_oneview_info(self.node)
 
     def test_get_properties(self):
         expected = common.COMMON_PROPERTIES
-        self.assertEqual(expected, self.driver.deploy.get_properties())
+        self.assertEqual(expected,
+                         deploy.OneViewIscsiDeploy().get_properties())
 
     @mock.patch.object(common, 'validate_oneview_resources_compatibility',
                        spect_set=True, autospec=True)
@@ -367,31 +347,24 @@ class OneViewIscsiDeployTestCase(db_base.DbTestCase):
             self.assertTrue(deallocate_server_hardware_mock.called)
 
 
-class OneViewAgentDeployTestCase(db_base.DbTestCase):
+class OneViewAgentDeployTestCase(test_common.BaseOneViewTest):
+
+    deploy_interface = 'oneview-direct'
+
     def setUp(self):
         super(OneViewAgentDeployTestCase, self).setUp()
         self.config(manager_url='https://1.2.3.4', group='oneview')
         self.config(username='user', group='oneview')
         self.config(password='password', group='oneview')
 
-        mgr_utils.mock_the_extension_manager(driver='agent_pxe_oneview')
-        self.driver = driver_factory.get_driver('agent_pxe_oneview')
-
-        OV_DRV_INFO_DICT = db_utils.get_test_oneview_driver_info()
-        OV_DRV_INFO_DICT.update(PXE_DRV_INFO_DICT)
-        self.node = obj_utils.create_test_node(
-            self.context, driver='agent_pxe_oneview',
-            properties=db_utils.get_test_oneview_properties(),
-            driver_info=OV_DRV_INFO_DICT,
-            instance_info=PXE_INST_INFO_DICT,
-        )
         self.port = obj_utils.create_test_port(self.context,
                                                node_id=self.node.id)
         self.info = common.get_oneview_info(self.node)
 
     def test_get_properties(self):
         expected = common.COMMON_PROPERTIES
-        self.assertEqual(expected, self.driver.deploy.get_properties())
+        self.assertEqual(expected,
+                         deploy.OneViewAgentDeploy().get_properties())
 
     @mock.patch.object(common, 'validate_oneview_resources_compatibility',
                        spect_set=True, autospec=True)
