@@ -29,6 +29,7 @@ from ironic.drivers import base as drivers_base
 from ironic.drivers.modules import agent
 from ironic.drivers.modules import agent_base_vendor
 from ironic.drivers.modules import agent_client
+from ironic.drivers.modules import boot_mode_utils
 from ironic.drivers.modules import deploy_utils
 from ironic.drivers.modules import fake
 from ironic.drivers.modules.network import flat as flat_network
@@ -36,7 +37,6 @@ from ironic.drivers.modules.network import neutron as neutron_network
 from ironic.drivers.modules import pxe
 from ironic.drivers.modules.storage import noop as noop_storage
 from ironic.drivers import utils as driver_utils
-from ironic.tests.unit.conductor import mgr_utils
 from ironic.tests.unit.db import base as db_base
 from ironic.tests.unit.db import utils as db_utils
 from ironic.tests.unit.objects import utils as object_utils
@@ -53,7 +53,8 @@ class TestAgentMethods(db_base.DbTestCase):
     def setUp(self):
         super(TestAgentMethods, self).setUp()
         self.node = object_utils.create_test_node(self.context,
-                                                  driver='fake_agent')
+                                                  boot_interface='pxe',
+                                                  deploy_interface='direct')
         dhcp_factory.DHCPFactory._dhcp_provider = None
 
     @mock.patch.object(images, 'image_show', autospec=True)
@@ -62,7 +63,6 @@ class TestAgentMethods(db_base.DbTestCase):
             'size': 10 * 1024 * 1024,
             'disk_format': 'qcow2',
         }
-        mgr_utils.mock_the_extension_manager(driver='fake_agent')
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=False) as task:
             task.node.properties['memory_mb'] = 10
@@ -71,7 +71,6 @@ class TestAgentMethods(db_base.DbTestCase):
 
     @mock.patch.object(images, 'image_show', autospec=True)
     def test_check_image_size_without_memory_mb(self, show_mock):
-        mgr_utils.mock_the_extension_manager(driver='fake_agent')
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=False) as task:
             task.node.properties.pop('memory_mb', None)
@@ -84,7 +83,6 @@ class TestAgentMethods(db_base.DbTestCase):
             'size': 11 * 1024 * 1024,
             'disk_format': 'qcow2',
         }
-        mgr_utils.mock_the_extension_manager(driver='fake_agent')
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=False) as task:
             task.node.properties['memory_mb'] = 10
@@ -100,7 +98,6 @@ class TestAgentMethods(db_base.DbTestCase):
             'size': 9 * 1024 * 1024,
             'disk_format': 'qcow2',
         }
-        mgr_utils.mock_the_extension_manager(driver='fake_agent')
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=False) as task:
             task.node.properties['memory_mb'] = 10
@@ -118,7 +115,6 @@ class TestAgentMethods(db_base.DbTestCase):
             'size': 15 * 1024 * 1024,
             'disk_format': 'raw',
         }
-        mgr_utils.mock_the_extension_manager(driver='fake_agent')
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=False) as task:
             task.node.properties['memory_mb'] = 10
@@ -132,7 +128,6 @@ class TestAgentMethods(db_base.DbTestCase):
             'size': 15 * 1024 * 1024,
             'disk_format': 'raw',
         }
-        mgr_utils.mock_the_extension_manager(driver='fake_agent')
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=False) as task:
             task.node.properties['memory_mb'] = 10
@@ -147,14 +142,14 @@ class TestAgentMethods(db_base.DbTestCase):
 class TestAgentDeploy(db_base.DbTestCase):
     def setUp(self):
         super(TestAgentDeploy, self).setUp()
-        mgr_utils.mock_the_extension_manager(driver='fake_agent')
         self.driver = agent.AgentDeploy()
         # NOTE(TheJulia): We explicitly set the noop storage interface as the
         # default below for deployment tests in order to raise any change
         # in the default which could be a breaking behavior change
         # as the storage interface is explicitly an "opt-in" interface.
         n = {
-            'driver': 'fake_agent',
+            'boot_interface': 'pxe',
+            'deploy_interface': 'direct',
             'instance_info': INSTANCE_INFO,
             'driver_info': DRIVER_INFO,
             'driver_internal_info': DRIVER_INTERNAL_INFO,
@@ -1040,7 +1035,8 @@ class TestAgentDeploy(db_base.DbTestCase):
             self.assertEqual(states.NOSTATE, task.node.target_provision_state)
 
     @mock.patch.object(agent.LOG, 'warning', spec_set=True, autospec=True)
-    @mock.patch.object(deploy_utils, 'get_boot_mode_for_deploy', autospec=True)
+    @mock.patch.object(boot_mode_utils, 'get_boot_mode_for_deploy',
+                       autospec=True)
     @mock.patch.object(agent.AgentDeployMixin, '_get_uuid_from_result',
                        autospec=True)
     @mock.patch.object(manager_utils, 'node_power_action', autospec=True)
@@ -1125,7 +1121,8 @@ class TestAgentDeploy(db_base.DbTestCase):
             self.assertEqual(states.ACTIVE, task.node.target_provision_state)
 
     @mock.patch.object(agent.LOG, 'warning', spec_set=True, autospec=True)
-    @mock.patch.object(deploy_utils, 'get_boot_mode_for_deploy', autospec=True)
+    @mock.patch.object(boot_mode_utils, 'get_boot_mode_for_deploy',
+                       autospec=True)
     @mock.patch.object(agent.AgentDeployMixin, '_get_uuid_from_result',
                        autospec=True)
     @mock.patch.object(manager_utils, 'node_power_action', autospec=True)
@@ -1257,7 +1254,7 @@ class AgentRAIDTestCase(db_base.DbTestCase):
 
     def setUp(self):
         super(AgentRAIDTestCase, self).setUp()
-        mgr_utils.mock_the_extension_manager(driver="fake_agent")
+        self.config(enabled_raid_interfaces=['fake', 'agent', 'no-raid'])
         self.target_raid_config = {
             "logical_disks": [
                 {'size_gb': 200, 'raid_level': 0, 'is_root_volume': True},
@@ -1266,7 +1263,9 @@ class AgentRAIDTestCase(db_base.DbTestCase):
         self.clean_step = {'step': 'create_configuration',
                            'interface': 'raid'}
         n = {
-            'driver': 'fake_agent',
+            'boot_interface': 'pxe',
+            'deploy_interface': 'direct',
+            'raid_interface': 'agent',
             'instance_info': INSTANCE_INFO,
             'driver_info': DRIVER_INFO,
             'driver_internal_info': DRIVER_INTERNAL_INFO,
@@ -1452,7 +1451,6 @@ class AgentRescueTestCase(db_base.DbTestCase):
         driver_info.update({'rescue_ramdisk': 'my_ramdisk',
                             'rescue_kernel': 'my_kernel'})
         n = {
-            'driver': 'fake-hardware',
             'instance_info': instance_info,
             'driver_info': driver_info,
             'driver_internal_info': DRIVER_INTERNAL_INFO,
@@ -1563,66 +1561,34 @@ class AgentRescueTestCase(db_base.DbTestCase):
     @mock.patch.object(flat_network.FlatNetwork, 'validate_rescue',
                        autospec=True)
     @mock.patch.object(fake.FakeBoot, 'validate', autospec=True)
-    def test_agent_rescue_validate(self, mock_boot_validate,
+    @mock.patch.object(fake.FakeBoot, 'validate_rescue', autospec=True)
+    def test_agent_rescue_validate(self, mock_boot_validate_rescue,
+                                   mock_boot_validate,
                                    mock_validate_network):
         with task_manager.acquire(self.context, self.node.uuid) as task:
             task.driver.rescue.validate(task)
             mock_validate_network.assert_called_once_with(mock.ANY, task)
             mock_boot_validate.assert_called_once_with(mock.ANY, task)
+            mock_boot_validate_rescue.assert_called_once_with(mock.ANY, task)
 
     @mock.patch.object(flat_network.FlatNetwork, 'validate_rescue',
                        autospec=True)
     @mock.patch.object(fake.FakeBoot, 'validate', autospec=True)
-    def test_agent_rescue_validate_no_manage_agent(self, mock_boot_validate,
+    @mock.patch.object(fake.FakeBoot, 'validate_rescue', autospec=True)
+    def test_agent_rescue_validate_no_manage_agent(self,
+                                                   mock_boot_validate_rescue,
+                                                   mock_boot_validate,
                                                    mock_rescuing_net):
-        # If ironic's not managing booting of ramdisks, we don't set up PXE for
-        # the ramdisk/kernel, so validation can pass without this info
         self.config(manage_agent_boot=False, group='agent')
-        driver_info = self.node.driver_info
-        del driver_info['rescue_ramdisk']
-        del driver_info['rescue_kernel']
-        self.node.driver_info = driver_info
-        self.node.save()
         with task_manager.acquire(self.context, self.node.uuid) as task:
             task.driver.rescue.validate(task)
             mock_rescuing_net.assert_called_once_with(mock.ANY, task)
             self.assertFalse(mock_boot_validate.called)
+            self.assertFalse(mock_boot_validate_rescue.called)
 
     @mock.patch.object(flat_network.FlatNetwork, 'validate_rescue',
                        autospec=True)
-    @mock.patch.object(fake.FakeBoot, 'validate', autospec=True)
-    def test_agent_rescue_validate_fails_no_rescue_ramdisk(
-            self, mock_boot_validate, mock_rescuing_net):
-        driver_info = self.node.driver_info
-        del driver_info['rescue_ramdisk']
-        self.node.driver_info = driver_info
-        self.node.save()
-        with task_manager.acquire(self.context, self.node.uuid) as task:
-            self.assertRaisesRegex(exception.MissingParameterValue,
-                                   'Node.*missing.*rescue_ramdisk',
-                                   task.driver.rescue.validate, task)
-            mock_rescuing_net.assert_called_once_with(mock.ANY, task)
-            mock_boot_validate.assert_called_once_with(mock.ANY, task)
-
-    @mock.patch.object(flat_network.FlatNetwork, 'validate_rescue',
-                       autospec=True)
-    @mock.patch.object(fake.FakeBoot, 'validate', autospec=True)
-    def test_agent_rescue_validate_fails_no_rescue_kernel(
-            self, mock_boot_validate, mock_rescuing_net):
-        driver_info = self.node.driver_info
-        del driver_info['rescue_kernel']
-        self.node.driver_info = driver_info
-        self.node.save()
-        with task_manager.acquire(self.context, self.node.uuid) as task:
-            self.assertRaisesRegex(exception.MissingParameterValue,
-                                   'Node.*missing.*rescue_kernel',
-                                   task.driver.rescue.validate, task)
-            mock_rescuing_net.assert_called_once_with(mock.ANY, task)
-            mock_boot_validate.assert_called_once_with(mock.ANY, task)
-
-    @mock.patch.object(flat_network.FlatNetwork, 'validate_rescue',
-                       autospec=True)
-    @mock.patch.object(fake.FakeBoot, 'validate', autospec=True)
+    @mock.patch.object(fake.FakeBoot, 'validate_rescue', autospec=True)
     def test_agent_rescue_validate_fails_no_rescue_password(
             self, mock_boot_validate, mock_rescuing_net):
         instance_info = self.node.instance_info
@@ -1638,7 +1604,7 @@ class AgentRescueTestCase(db_base.DbTestCase):
 
     @mock.patch.object(flat_network.FlatNetwork, 'validate_rescue',
                        autospec=True)
-    @mock.patch.object(fake.FakeBoot, 'validate', autospec=True)
+    @mock.patch.object(fake.FakeBoot, 'validate_rescue', autospec=True)
     def test_agent_rescue_validate_fails_empty_rescue_password(
             self, mock_boot_validate, mock_rescuing_net):
         instance_info = self.node.instance_info
