@@ -68,7 +68,6 @@ class AgentDeployMixinBaseTest(db_base.DbTestCase):
             'instance_info': INSTANCE_INFO,
             'driver_info': DRIVER_INFO,
             'driver_internal_info': DRIVER_INTERNAL_INFO,
-            'network_interface': 'noop'
         }
         self.node = object_utils.create_test_node(self.context, **n)
 
@@ -83,7 +82,7 @@ class HeartbeatMixinTest(AgentDeployMixinBaseTest):
                        autospec=True)
     @mock.patch.object(agent_base_vendor.HeartbeatMixin,
                        'reboot_to_instance', autospec=True)
-    @mock.patch.object(agent_base_vendor, '_notify_conductor_resume_clean',
+    @mock.patch.object(manager_utils, 'notify_conductor_resume_clean',
                        autospec=True)
     def test_heartbeat_in_maintenance(self, ncrc_mock, rti_mock, cd_mock):
         # NOTE(pas-ha) checking only for states that are not noop
@@ -108,11 +107,38 @@ class HeartbeatMixinTest(AgentDeployMixinBaseTest):
             self.assertEqual(0, rti_mock.call_count)
             self.assertEqual(0, cd_mock.call_count)
 
+    @mock.patch('time.sleep', lambda _t: None)
     @mock.patch.object(agent_base_vendor.HeartbeatMixin, 'continue_deploy',
                        autospec=True)
     @mock.patch.object(agent_base_vendor.HeartbeatMixin,
                        'reboot_to_instance', autospec=True)
-    @mock.patch.object(agent_base_vendor, '_notify_conductor_resume_clean',
+    @mock.patch.object(manager_utils, 'notify_conductor_resume_clean',
+                       autospec=True)
+    def test_heartbeat_with_reservation(self, ncrc_mock, rti_mock, cd_mock):
+        # NOTE(pas-ha) checking only for states that are not noop
+        for state in (states.DEPLOYWAIT, states.CLEANWAIT):
+            for m in (ncrc_mock, rti_mock, cd_mock):
+                m.reset_mock()
+            self.node.provision_state = state
+            self.node.reservation = 'localhost'
+            self.node.save()
+            old_drv_info = self.node.driver_internal_info.copy()
+            agent_url = 'url-%s' % state
+            with task_manager.acquire(self.context, self.node.uuid,
+                                      shared=True) as task:
+                self.deploy.heartbeat(task, agent_url, '3.2.0')
+                self.assertTrue(task.shared)
+                self.assertEqual(old_drv_info, task.node.driver_internal_info)
+                self.assertIsNone(task.node.last_error)
+            self.assertEqual(0, ncrc_mock.call_count)
+            self.assertEqual(0, rti_mock.call_count)
+            self.assertEqual(0, cd_mock.call_count)
+
+    @mock.patch.object(agent_base_vendor.HeartbeatMixin, 'continue_deploy',
+                       autospec=True)
+    @mock.patch.object(agent_base_vendor.HeartbeatMixin,
+                       'reboot_to_instance', autospec=True)
+    @mock.patch.object(manager_utils, 'notify_conductor_resume_clean',
                        autospec=True)
     def test_heartbeat_noops_in_wrong_state(self, ncrc_mock, rti_mock,
                                             cd_mock):
@@ -187,7 +213,7 @@ class HeartbeatMixinTest(AgentDeployMixinBaseTest):
     @mock.patch.object(agent_base_vendor.HeartbeatMixin,
                        'refresh_clean_steps', autospec=True)
     @mock.patch.object(manager_utils, 'set_node_cleaning_steps', autospec=True)
-    @mock.patch.object(agent_base_vendor, '_notify_conductor_resume_clean',
+    @mock.patch.object(manager_utils, 'notify_conductor_resume_clean',
                        autospec=True)
     def test_heartbeat_resume_clean(self, mock_notify, mock_set_steps,
                                     mock_refresh, mock_touch):
@@ -208,7 +234,7 @@ class HeartbeatMixinTest(AgentDeployMixinBaseTest):
     @mock.patch.object(agent_base_vendor.HeartbeatMixin,
                        'refresh_clean_steps', autospec=True)
     @mock.patch.object(manager_utils, 'set_node_cleaning_steps', autospec=True)
-    @mock.patch.object(agent_base_vendor, '_notify_conductor_resume_clean',
+    @mock.patch.object(manager_utils, 'notify_conductor_resume_clean',
                        autospec=True)
     def test_heartbeat_resume_clean_fails(self, mock_notify, mock_set_steps,
                                           mock_refresh, mock_touch,
@@ -433,9 +459,9 @@ class AgentDeployMixinTest(AgentDeployMixinBaseTest):
                        spec=types.FunctionType)
     @mock.patch.object(agent_client.AgentClient, 'power_off',
                        spec=types.FunctionType)
-    @mock.patch('ironic.drivers.modules.network.noop.NoopNetwork.'
+    @mock.patch('ironic.drivers.modules.network.flat.FlatNetwork.'
                 'remove_provisioning_network', spec_set=True, autospec=True)
-    @mock.patch('ironic.drivers.modules.network.noop.NoopNetwork.'
+    @mock.patch('ironic.drivers.modules.network.flat.FlatNetwork.'
                 'configure_tenant_networks', spec_set=True, autospec=True)
     def test_reboot_and_finish_deploy_soft_poweroff_doesnt_complete(
             self, configure_tenant_net_mock, remove_provisioning_net_mock,
@@ -464,9 +490,9 @@ class AgentDeployMixinTest(AgentDeployMixinBaseTest):
     @mock.patch.object(manager_utils, 'node_power_action', autospec=True)
     @mock.patch.object(agent_client.AgentClient, 'power_off',
                        spec=types.FunctionType)
-    @mock.patch('ironic.drivers.modules.network.noop.NoopNetwork.'
+    @mock.patch('ironic.drivers.modules.network.flat.FlatNetwork.'
                 'remove_provisioning_network', spec_set=True, autospec=True)
-    @mock.patch('ironic.drivers.modules.network.noop.NoopNetwork.'
+    @mock.patch('ironic.drivers.modules.network.flat.FlatNetwork.'
                 'configure_tenant_networks', spec_set=True, autospec=True)
     def test_reboot_and_finish_deploy_soft_poweroff_fails(
             self, configure_tenant_net_mock, remove_provisioning_net_mock,
@@ -496,9 +522,9 @@ class AgentDeployMixinTest(AgentDeployMixinBaseTest):
                        spec=types.FunctionType)
     @mock.patch.object(agent_client.AgentClient, 'power_off',
                        spec=types.FunctionType)
-    @mock.patch('ironic.drivers.modules.network.noop.NoopNetwork.'
+    @mock.patch('ironic.drivers.modules.network.flat.FlatNetwork.'
                 'remove_provisioning_network', spec_set=True, autospec=True)
-    @mock.patch('ironic.drivers.modules.network.noop.NoopNetwork.'
+    @mock.patch('ironic.drivers.modules.network.flat.FlatNetwork.'
                 'configure_tenant_networks', spec_set=True, autospec=True)
     def test_reboot_and_finish_deploy_get_power_state_fails(
             self, configure_tenant_net_mock, remove_provisioning_net_mock,
@@ -591,9 +617,9 @@ class AgentDeployMixinTest(AgentDeployMixinBaseTest):
                        spec=types.FunctionType)
     @mock.patch.object(agent_client.AgentClient, 'power_off',
                        spec=types.FunctionType)
-    @mock.patch('ironic.drivers.modules.network.noop.NoopNetwork.'
+    @mock.patch('ironic.drivers.modules.network.flat.FlatNetwork.'
                 'remove_provisioning_network', spec_set=True, autospec=True)
-    @mock.patch('ironic.drivers.modules.network.noop.NoopNetwork.'
+    @mock.patch('ironic.drivers.modules.network.flat.FlatNetwork.'
                 'configure_tenant_networks', spec_set=True, autospec=True)
     def test_reboot_and_finish_deploy_power_on_fails(
             self, configure_tenant_net_mock, remove_provisioning_net_mock,
@@ -887,7 +913,7 @@ class AgentDeployMixinTest(AgentDeployMixinBaseTest):
             self.assertFalse(prepare_mock.called)
             self.assertFalse(failed_state_mock.called)
 
-    @mock.patch.object(agent_base_vendor, '_notify_conductor_resume_clean',
+    @mock.patch.object(manager_utils, 'notify_conductor_resume_clean',
                        autospec=True)
     @mock.patch.object(agent_client.AgentClient, 'get_commands_status',
                        autospec=True)
@@ -970,7 +996,7 @@ class AgentDeployMixinTest(AgentDeployMixinBaseTest):
             self.deploy.continue_cleaning(task)
             reboot_mock.assert_called_once_with(task, states.REBOOT)
 
-    @mock.patch.object(agent_base_vendor, '_notify_conductor_resume_clean',
+    @mock.patch.object(manager_utils, 'notify_conductor_resume_clean',
                        autospec=True)
     @mock.patch.object(agent_client.AgentClient, 'get_commands_status',
                        autospec=True)
@@ -997,7 +1023,7 @@ class AgentDeployMixinTest(AgentDeployMixinBaseTest):
 
     @mock.patch.object(agent_base_vendor,
                        '_get_post_clean_step_hook', autospec=True)
-    @mock.patch.object(agent_base_vendor, '_notify_conductor_resume_clean',
+    @mock.patch.object(manager_utils, 'notify_conductor_resume_clean',
                        autospec=True)
     @mock.patch.object(agent_client.AgentClient, 'get_commands_status',
                        autospec=True)
@@ -1024,7 +1050,7 @@ class AgentDeployMixinTest(AgentDeployMixinBaseTest):
             hook_mock.assert_called_once_with(task, command_status)
             notify_mock.assert_called_once_with(task)
 
-    @mock.patch.object(agent_base_vendor, '_notify_conductor_resume_clean',
+    @mock.patch.object(manager_utils, 'notify_conductor_resume_clean',
                        autospec=True)
     @mock.patch.object(agent_base_vendor,
                        '_get_post_clean_step_hook', autospec=True)
@@ -1057,7 +1083,7 @@ class AgentDeployMixinTest(AgentDeployMixinBaseTest):
             error_handler_mock.assert_called_once_with(task, mock.ANY)
             self.assertFalse(notify_mock.called)
 
-    @mock.patch.object(agent_base_vendor, '_notify_conductor_resume_clean',
+    @mock.patch.object(manager_utils, 'notify_conductor_resume_clean',
                        autospec=True)
     @mock.patch.object(agent_client.AgentClient, 'get_commands_status',
                        autospec=True)
@@ -1086,7 +1112,7 @@ class AgentDeployMixinTest(AgentDeployMixinBaseTest):
             self.deploy.continue_cleaning(task)
             self.assertFalse(notify_mock.called)
 
-    @mock.patch.object(agent_base_vendor, '_notify_conductor_resume_clean',
+    @mock.patch.object(manager_utils, 'notify_conductor_resume_clean',
                        autospec=True)
     @mock.patch.object(agent_client.AgentClient, 'get_commands_status',
                        autospec=True)
@@ -1118,7 +1144,7 @@ class AgentDeployMixinTest(AgentDeployMixinBaseTest):
             error_mock.assert_called_once_with(task, mock.ANY)
 
     @mock.patch.object(manager_utils, 'set_node_cleaning_steps', autospec=True)
-    @mock.patch.object(agent_base_vendor, '_notify_conductor_resume_clean',
+    @mock.patch.object(manager_utils, 'notify_conductor_resume_clean',
                        autospec=True)
     @mock.patch.object(agent_base_vendor.AgentDeployMixin,
                        'refresh_clean_steps', autospec=True)
@@ -1157,7 +1183,7 @@ class AgentDeployMixinTest(AgentDeployMixinBaseTest):
 
     @mock.patch.object(manager_utils, 'cleaning_error_handler', autospec=True)
     @mock.patch.object(manager_utils, 'set_node_cleaning_steps', autospec=True)
-    @mock.patch.object(agent_base_vendor, '_notify_conductor_resume_clean',
+    @mock.patch.object(manager_utils, 'notify_conductor_resume_clean',
                        autospec=True)
     @mock.patch.object(agent_base_vendor.AgentDeployMixin,
                        'refresh_clean_steps', autospec=True)
