@@ -26,6 +26,7 @@ from oslo_utils import fileutils
 from oslo_utils import uuidutils
 
 from ironic.common import boot_devices
+from ironic.common import boot_modes
 from ironic.common import dhcp_factory
 from ironic.common import exception
 from ironic.common.glance_service import base_image_service
@@ -34,11 +35,11 @@ from ironic.common import states
 from ironic.common import utils as common_utils
 from ironic.conductor import task_manager
 from ironic.conductor import utils as manager_utils
+from ironic.drivers import base as drivers_base
 from ironic.drivers.modules import agent_base_vendor
 from ironic.drivers.modules import deploy_utils
 from ironic.drivers.modules import pxe
 from ironic.drivers.modules.storage import noop as noop_storage
-from ironic.tests.unit.conductor import mgr_utils
 from ironic.tests.unit.db import base as db_base
 from ironic.tests.unit.db import utils as db_utils
 from ironic.tests.unit.objects import utils as obj_utils
@@ -50,17 +51,19 @@ DRV_INFO_DICT = db_utils.get_test_pxe_driver_info()
 DRV_INTERNAL_INFO_DICT = db_utils.get_test_pxe_driver_internal_info()
 
 
+@mock.patch.object(pxe.PXEBoot, '__init__', lambda self: None)
 class PXEPrivateMethodsTestCase(db_base.DbTestCase):
 
     def setUp(self):
         super(PXEPrivateMethodsTestCase, self).setUp()
         n = {
-            'driver': 'fake_pxe',
+            'driver': 'fake-hardware',
+            'boot_interface': 'pxe',
             'instance_info': INST_INFO_DICT,
             'driver_info': DRV_INFO_DICT,
             'driver_internal_info': DRV_INTERNAL_INFO_DICT,
         }
-        mgr_utils.mock_the_extension_manager(driver="fake_pxe")
+        self.config_temp_dir('http_root', group='deploy')
         self.node = obj_utils.create_test_node(self.context, **n)
 
     def _test__parse_driver_info_missing_kernel(self, mode='deploy'):
@@ -223,8 +226,8 @@ class PXEPrivateMethodsTestCase(db_base.DbTestCase):
                                          ramdisk_label))
         }
 
-        if (whle_dsk_img or
-            deploy_utils.get_boot_option(self.node) == 'local'):
+        if (whle_dsk_img
+            or deploy_utils.get_boot_option(self.node) == 'local'):
                 ramdisk = 'no_ramdisk'
                 kernel = 'no_kernel'
         else:
@@ -384,8 +387,8 @@ class PXEPrivateMethodsTestCase(db_base.DbTestCase):
 
         kernel = os.path.join(http_url, self.node.uuid, 'kernel')
         ramdisk = os.path.join(http_url, self.node.uuid, 'ramdisk')
-        if (whle_dsk_img or
-            deploy_utils.get_boot_option(self.node) == 'local'):
+        if (whle_dsk_img
+            or deploy_utils.get_boot_option(self.node) == 'local'):
                 ramdisk = 'no_ramdisk'
                 kernel = 'no_kernel'
         else:
@@ -426,9 +429,13 @@ class PXEPrivateMethodsTestCase(db_base.DbTestCase):
                 'boot_from_volume': True,
                 'iscsi_boot_url': 'iscsi:fake_host::3260:0:fake_iqn',
                 'iscsi_initiator_iqn': 'fake_iqn_initiator',
-                'iscsi_volumes': ['iscsi:fake_host::3260:1:fake_iqn'],
+                'iscsi_volumes': [{'url': 'iscsi:fake_host::3260:1:fake_iqn',
+                                   'username': 'fake_username_1',
+                                   'password': 'fake_password_1'
+                                   }],
                 'username': 'fake_username',
-                'password': 'fake_password'})
+                'password': 'fake_password'
+            })
             expected_options.pop('deployment_aki_path')
             expected_options.pop('deployment_ari_path')
             expected_options.pop('initrd_filename')
@@ -486,7 +493,9 @@ class PXEPrivateMethodsTestCase(db_base.DbTestCase):
             boot_index=1, volume_id='1235', uuid=vol_id2,
             properties={'target_lun': 1,
                         'target_portal': 'fake_host:3260',
-                        'target_iqn': 'fake_iqn'})
+                        'target_iqn': 'fake_iqn',
+                        'auth_username': 'fake_username_1',
+                        'auth_password': 'fake_password_1'})
         self.node.driver_internal_info.update({'boot_from_volume': vol_id})
         self._test_build_pxe_config_options_ipxe(boot_from_volume=True)
 
@@ -513,7 +522,9 @@ class PXEPrivateMethodsTestCase(db_base.DbTestCase):
             boot_index=1, volume_id='1235', uuid=vol_id2,
             properties={'target_lun': [1, 3],
                         'target_portal': ['fake_host:3260', 'faker_host:3261'],
-                        'target_iqn': ['fake_iqn', 'faker_iqn']})
+                        'target_iqn': ['fake_iqn', 'faker_iqn'],
+                        'auth_username': 'fake_username_1',
+                        'auth_password': 'fake_password_1'})
         self.node.driver_internal_info.update({'boot_from_volume': vol_id})
         self._test_build_pxe_config_options_ipxe(boot_from_volume=True)
 
@@ -539,7 +550,9 @@ class PXEPrivateMethodsTestCase(db_base.DbTestCase):
             boot_index=1, volume_id='1235', uuid=vol_id2,
             properties={'target_lun': 1,
                         'target_portal': 'fake_host:3260',
-                        'target_iqn': 'fake_iqn'})
+                        'target_iqn': 'fake_iqn',
+                        'auth_username': 'fake_username_1',
+                        'auth_password': 'fake_password_1'})
         self.node.driver_internal_info.update({'boot_from_volume': vol_id})
         driver_internal_info = self.node.driver_internal_info
         driver_internal_info['boot_from_volume'] = vol_id
@@ -550,7 +563,12 @@ class PXEPrivateMethodsTestCase(db_base.DbTestCase):
                     'username': 'fake_username', 'password': 'fake_password',
                     'iscsi_boot_url': 'iscsi:fake_host::3260:0:fake_iqn',
                     'iscsi_initiator_iqn': 'fake_iqn_initiator',
-                    'iscsi_volumes': ['iscsi:fake_host::3260:1:fake_iqn']}
+                    'iscsi_volumes': [{
+                        'url': 'iscsi:fake_host::3260:1:fake_iqn',
+                        'username': 'fake_username_1',
+                        'password': 'fake_password_1'
+                    }]
+                    }
         with task_manager.acquire(self.context, self.node.uuid,
                                   shared=True) as task:
             options = pxe._get_volume_pxe_options(task)
@@ -710,11 +728,10 @@ class PXEPrivateMethodsTestCase(db_base.DbTestCase):
 class CleanUpPxeEnvTestCase(db_base.DbTestCase):
     def setUp(self):
         super(CleanUpPxeEnvTestCase, self).setUp()
-        mgr_utils.mock_the_extension_manager(driver="fake_pxe")
         instance_info = INST_INFO_DICT
         instance_info['deploy_key'] = 'fake-56789'
         self.node = obj_utils.create_test_node(
-            self.context, driver='fake_pxe',
+            self.context, boot_interface='pxe',
             instance_info=instance_info,
             driver_info=DRV_INFO_DICT,
             driver_internal_info=DRV_INTERNAL_INFO_DICT,
@@ -731,26 +748,33 @@ class CleanUpPxeEnvTestCase(db_base.DbTestCase):
         mock_cache.return_value.clean_up.assert_called_once_with()
 
 
+@mock.patch.object(pxe.PXEBoot, '__init__', lambda self: None)
 class PXEBootTestCase(db_base.DbTestCase):
 
-    driver = 'fake_pxe'
+    driver = 'fake-hardware'
+    boot_interface = 'pxe'
+    driver_info = DRV_INFO_DICT
+    driver_internal_info = DRV_INTERNAL_INFO_DICT
 
     def setUp(self):
         super(PXEBootTestCase, self).setUp()
         self.context.auth_token = 'fake'
-        self.temp_dir = tempfile.mkdtemp()
-        self.config(tftp_root=self.temp_dir, group='pxe')
-        self.temp_dir = tempfile.mkdtemp()
-        self.config(images_path=self.temp_dir, group='pxe')
-        mgr_utils.mock_the_extension_manager(driver=self.driver)
+        self.config_temp_dir('tftp_root', group='pxe')
+        self.config_temp_dir('images_path', group='pxe')
+        self.config_temp_dir('http_root', group='deploy')
         instance_info = INST_INFO_DICT
         instance_info['deploy_key'] = 'fake-56789'
+
+        self.config(enabled_boot_interfaces=[self.boot_interface, 'fake'])
         self.node = obj_utils.create_test_node(
             self.context,
             driver=self.driver,
+            boot_interface=self.boot_interface,
+            # Avoid fake properties in get_properties() output
+            vendor_interface='no-vendor',
             instance_info=instance_info,
-            driver_info=DRV_INFO_DICT,
-            driver_internal_info=DRV_INTERNAL_INFO_DICT)
+            driver_info=self.driver_info,
+            driver_internal_info=self.driver_internal_info)
         self.port = obj_utils.create_test_port(self.context,
                                                node_id=self.node.id)
         self.config(group='conductor', api_url='http://127.0.0.1:1234/')
@@ -819,8 +843,8 @@ class PXEBootTestCase(db_base.DbTestCase):
         new_node = obj_utils.create_test_node(
             self.context,
             uuid='aaaaaaaa-bbbb-cccc-dddd-eeeeeeeeeeee',
-            driver=self.driver, instance_info=INST_INFO_DICT,
-            driver_info=DRV_INFO_DICT)
+            driver=self.driver, boot_interface=self.boot_interface,
+            instance_info=INST_INFO_DICT, driver_info=DRV_INFO_DICT)
         with task_manager.acquire(self.context, new_node.uuid,
                                   shared=True) as task:
             self.assertRaises(exception.MissingParameterValue,
@@ -881,6 +905,7 @@ class PXEBootTestCase(db_base.DbTestCase):
                 self.assertRaises(exception.InvalidParameterValue,
                                   task.driver.boot.validate, task)
 
+    @mock.patch.object(manager_utils, 'node_get_boot_mode', autospec=True)
     @mock.patch.object(manager_utils, 'node_set_boot_device', autospec=True)
     @mock.patch.object(dhcp_factory, 'DHCPFactory')
     @mock.patch.object(pxe, '_get_instance_image_info', autospec=True)
@@ -894,11 +919,13 @@ class PXEBootTestCase(db_base.DbTestCase):
                               mock_instance_img_info,
                               dhcp_factory_mock,
                               set_boot_device_mock,
+                              get_boot_mode_mock,
                               uefi=False,
                               cleaning=False,
                               ipxe_use_swift=False,
                               whole_disk_image=False,
-                              mode='deploy'):
+                              mode='deploy',
+                              node_boot_mode=None):
         mock_build_pxe.return_value = {}
         kernel_label = '%s_kernel' % mode
         ramdisk_label = '%s_ramdisk' % mode
@@ -912,6 +939,7 @@ class PXEBootTestCase(db_base.DbTestCase):
         mock_cache_r_k.return_value = None
         provider_mock = mock.MagicMock()
         dhcp_factory_mock.return_value = provider_mock
+        get_boot_mode_mock.return_value = node_boot_mode
         driver_internal_info = self.node.driver_internal_info
         driver_internal_info['is_whole_disk_image'] = whole_disk_image
         self.node.driver_internal_info = driver_internal_info
@@ -925,6 +953,8 @@ class PXEBootTestCase(db_base.DbTestCase):
             task.driver.boot.prepare_ramdisk(task, {'foo': 'bar'})
             mock_deploy_img_info.assert_called_once_with(task.node, mode=mode)
             provider_mock.update_dhcp.assert_called_once_with(task, dhcp_opts)
+            if self.node.provision_state == states.DEPLOYING:
+                get_boot_mode_mock.assert_called_once_with(task)
             set_boot_device_mock.assert_called_once_with(task,
                                                          boot_devices.PXE,
                                                          persistent=False)
@@ -1070,6 +1100,104 @@ class PXEBootTestCase(db_base.DbTestCase):
         self.node.save()
         self._test_prepare_ramdisk(cleaning=True)
 
+    @mock.patch.object(manager_utils, 'node_set_boot_mode', autospec=True)
+    def test_prepare_ramdisk_set_boot_mode_on_bm(
+            self, set_boot_mode_mock):
+        self.node.provision_state = states.DEPLOYING
+        properties = self.node.properties
+        properties['capabilities'] = 'boot_mode:uefi'
+        self.node.properties = properties
+        self.node.save()
+        self._test_prepare_ramdisk(uefi=True)
+        set_boot_mode_mock.assert_called_once_with(mock.ANY, boot_modes.UEFI)
+
+    @mock.patch.object(manager_utils, 'node_set_boot_mode', autospec=True)
+    def test_prepare_ramdisk_set_boot_mode_on_ironic(
+            self, set_boot_mode_mock):
+        self.node.provision_state = states.DEPLOYING
+        self.node.save()
+        self._test_prepare_ramdisk(node_boot_mode=boot_modes.LEGACY_BIOS)
+
+        with task_manager.acquire(self.context, self.node.uuid) as task:
+            driver_internal_info = task.node.driver_internal_info
+            self.assertIn('deploy_boot_mode', driver_internal_info)
+            self.assertEqual(boot_modes.LEGACY_BIOS,
+                             driver_internal_info['deploy_boot_mode'])
+            self.assertEqual(set_boot_mode_mock.call_count, 0)
+
+    @mock.patch.object(manager_utils, 'node_set_boot_mode', autospec=True)
+    def test_prepare_ramdisk_set_default_boot_mode_on_ironic_bios(
+            self, set_boot_mode_mock):
+        self.node.provision_state = states.DEPLOYING
+        self.node.save()
+
+        self.config(default_boot_mode=boot_modes.LEGACY_BIOS, group='deploy')
+
+        self._test_prepare_ramdisk()
+
+        with task_manager.acquire(self.context, self.node.uuid) as task:
+            driver_internal_info = task.node.driver_internal_info
+            self.assertIn('deploy_boot_mode', driver_internal_info)
+            self.assertEqual(boot_modes.LEGACY_BIOS,
+                             driver_internal_info['deploy_boot_mode'])
+            self.assertEqual(set_boot_mode_mock.call_count, 1)
+
+    @mock.patch.object(manager_utils, 'node_set_boot_mode', autospec=True)
+    def test_prepare_ramdisk_set_default_boot_mode_on_ironic_uefi(
+            self, set_boot_mode_mock):
+        self.node.provision_state = states.DEPLOYING
+        self.node.save()
+
+        self.config(default_boot_mode=boot_modes.UEFI, group='deploy')
+
+        self._test_prepare_ramdisk(uefi=True)
+
+        with task_manager.acquire(self.context, self.node.uuid) as task:
+            driver_internal_info = task.node.driver_internal_info
+            self.assertIn('deploy_boot_mode', driver_internal_info)
+            self.assertEqual(boot_modes.UEFI,
+                             driver_internal_info['deploy_boot_mode'])
+            self.assertEqual(set_boot_mode_mock.call_count, 1)
+
+    @mock.patch.object(manager_utils, 'node_set_boot_mode', autospec=True)
+    def test_prepare_ramdisk_conflicting_boot_modes(
+            self, set_boot_mode_mock):
+        self.node.provision_state = states.DEPLOYING
+        properties = self.node.properties
+        properties['capabilities'] = 'boot_mode:uefi'
+        self.node.properties = properties
+        self.node.save()
+        self._test_prepare_ramdisk(uefi=True,
+                                   node_boot_mode=boot_modes.LEGACY_BIOS)
+        set_boot_mode_mock.assert_called_once_with(mock.ANY, boot_modes.UEFI)
+
+    @mock.patch.object(manager_utils, 'node_set_boot_mode', autospec=True)
+    def test_prepare_ramdisk_conflicting_boot_modes_set_unsupported(
+            self, set_boot_mode_mock):
+        self.node.provision_state = states.DEPLOYING
+        properties = self.node.properties
+        properties['capabilities'] = 'boot_mode:uefi'
+        self.node.properties = properties
+        self.node.save()
+        set_boot_mode_mock.side_effect = exception.UnsupportedDriverExtension(
+            extension='management', driver='test-driver'
+        )
+        self.assertRaises(exception.UnsupportedDriverExtension,
+                          self._test_prepare_ramdisk,
+                          uefi=True, node_boot_mode=boot_modes.LEGACY_BIOS)
+
+    @mock.patch.object(manager_utils, 'node_set_boot_mode', autospec=True)
+    def test_prepare_ramdisk_set_boot_mode_not_called(
+            self, set_boot_mode_mock):
+        self.node.provision_state = states.DEPLOYING
+        self.node.save()
+        properties = self.node.properties
+        properties['capabilities'] = 'boot_mode:uefi'
+        self.node.properties = properties
+        self.node.save()
+        self._test_prepare_ramdisk(uefi=True, node_boot_mode=boot_modes.UEFI)
+        self.assertEqual(set_boot_mode_mock.call_count, 0)
+
     @mock.patch.object(pxe, '_clean_up_pxe_env', autospec=True)
     @mock.patch.object(pxe, '_get_image_info', autospec=True)
     def _test_clean_up_ramdisk(self, get_image_info_mock,
@@ -1263,6 +1391,7 @@ class PXEBootTestCase(db_base.DbTestCase):
             dhcp_opts = pxe_utils.dhcp_options_for_instance(task)
             pxe_config_path = pxe_utils.get_pxe_config_file_path(
                 task.node.uuid)
+            task.node.properties['capabilities'] = 'boot_mode:bios'
             task.driver.boot.prepare_instance(task)
             self.assertFalse(get_image_info_mock.called)
             self.assertFalse(cache_mock.called)
@@ -1270,7 +1399,8 @@ class PXEBootTestCase(db_base.DbTestCase):
             create_pxe_config_mock.assert_called_once_with(
                 task, mock.ANY, CONF.pxe.pxe_config_template)
             switch_pxe_config_mock.assert_called_once_with(
-                pxe_config_path, None, None, False, iscsi_boot=True)
+                pxe_config_path, None, boot_modes.LEGACY_BIOS, False,
+                iscsi_boot=True)
             set_boot_device_mock.assert_called_once_with(task,
                                                          boot_devices.PXE,
                                                          persistent=True)
@@ -1280,7 +1410,10 @@ class PXEBootTestCase(db_base.DbTestCase):
     def test_prepare_instance_localboot(self, clean_up_pxe_config_mock,
                                         set_boot_device_mock):
         with task_manager.acquire(self.context, self.node.uuid) as task:
-            task.node.instance_info['capabilities'] = {'boot_option': 'local'}
+            instance_info = task.node.instance_info
+            instance_info['capabilities'] = {'boot_option': 'local'}
+            task.node.instance_info = instance_info
+            task.node.save()
             task.driver.boot.prepare_instance(task)
             clean_up_pxe_config_mock.assert_called_once_with(task)
             set_boot_device_mock.assert_called_once_with(task,
@@ -1292,7 +1425,10 @@ class PXEBootTestCase(db_base.DbTestCase):
     def test_is_force_persistent_boot_device_enabled(
             self, clean_up_pxe_config_mock, set_boot_device_mock):
         with task_manager.acquire(self.context, self.node.uuid) as task:
-            task.node.instance_info['capabilities'] = {'boot_option': 'local'}
+            instance_info = task.node.instance_info
+            instance_info['capabilities'] = {'boot_option': 'local'}
+            task.node.instance_info = instance_info
+            task.node.save()
             task.driver.boot.prepare_instance(task)
             clean_up_pxe_config_mock.assert_called_once_with(task)
             driver_info = task.node.driver_info
@@ -1309,7 +1445,10 @@ class PXEBootTestCase(db_base.DbTestCase):
         self.node.provision_state = states.ACTIVE
         self.node.save()
         with task_manager.acquire(self.context, self.node.uuid) as task:
-            task.node.instance_info['capabilities'] = {'boot_option': 'local'}
+            instance_info = task.node.instance_info
+            instance_info['capabilities'] = {'boot_option': 'local'}
+            task.node.instance_info = instance_info
+            task.node.save()
             task.driver.boot.prepare_instance(task)
             clean_up_pxe_config_mock.assert_called_once_with(task)
             self.assertFalse(set_boot_device_mock.called)
@@ -1326,3 +1465,57 @@ class PXEBootTestCase(db_base.DbTestCase):
             clean_up_pxe_env_mock.assert_called_once_with(task, image_info)
             get_image_info_mock.assert_called_once_with(
                 task.node, task.context)
+
+
+class PXEValidateRescueTestCase(db_base.DbTestCase):
+
+    def setUp(self):
+        super(PXEValidateRescueTestCase, self).setUp()
+        for iface in drivers_base.ALL_INTERFACES:
+            impl = 'fake'
+            if iface == 'network':
+                impl = 'flat'
+            if iface == 'rescue':
+                impl = 'agent'
+            if iface == 'boot':
+                impl = 'pxe'
+            config_kwarg = {'enabled_%s_interfaces' % iface: [impl],
+                            'default_%s_interface' % iface: impl}
+            self.config(**config_kwarg)
+        self.config(enabled_hardware_types=['fake-hardware'])
+        driver_info = DRV_INFO_DICT
+        driver_info.update({'rescue_ramdisk': 'my_ramdisk',
+                            'rescue_kernel': 'my_kernel'})
+        instance_info = INST_INFO_DICT
+        instance_info.update({'rescue_password': 'password'})
+        n = {
+            'driver': 'fake-hardware',
+            'instance_info': instance_info,
+            'driver_info': driver_info,
+            'driver_internal_info': DRV_INTERNAL_INFO_DICT,
+        }
+        self.node = obj_utils.create_test_node(self.context, **n)
+
+    def test_validate_rescue(self):
+        with task_manager.acquire(self.context, self.node.uuid) as task:
+            task.driver.boot.validate_rescue(task)
+
+    def test_validate_rescue_no_rescue_ramdisk(self):
+        driver_info = self.node.driver_info
+        del driver_info['rescue_ramdisk']
+        self.node.driver_info = driver_info
+        self.node.save()
+        with task_manager.acquire(self.context, self.node.uuid) as task:
+            self.assertRaisesRegex(exception.MissingParameterValue,
+                                   'Missing.*rescue_ramdisk',
+                                   task.driver.boot.validate_rescue, task)
+
+    def test_validate_rescue_fails_no_rescue_kernel(self):
+        driver_info = self.node.driver_info
+        del driver_info['rescue_kernel']
+        self.node.driver_info = driver_info
+        self.node.save()
+        with task_manager.acquire(self.context, self.node.uuid) as task:
+            self.assertRaisesRegex(exception.MissingParameterValue,
+                                   'Missing.*rescue_kernel',
+                                   task.driver.boot.validate_rescue, task)
