@@ -28,7 +28,6 @@ from ironic.common import states
 from ironic.conductor import task_manager
 from ironic.drivers.modules import snmp
 from ironic.tests import base
-from ironic.tests.unit.conductor import mgr_utils
 from ironic.tests.unit.db import base as db_base
 from ironic.tests.unit.db import utils as db_utils
 from ironic.tests.unit.objects import utils as obj_utils
@@ -53,7 +52,7 @@ class SNMPClientTestCase(base.TestCase):
         self.assertEqual(self.port, client.port)
         self.assertEqual(snmp.SNMP_V1, client.version)
         self.assertIsNone(client.community)
-        self.assertNotIn('security', client.__dict__)
+        self.assertNotIn('user', client.__dict__)
         self.assertEqual(mock_cmdgen.return_value, client.cmd_gen)
 
     @mock.patch.object(cmdgen, 'CommunityData', autospec=True)
@@ -68,7 +67,15 @@ class SNMPClientTestCase(base.TestCase):
         client = snmp.SNMPClient(self.address, self.port, snmp.SNMP_V3)
         client._get_auth()
         mock_cmdgen.assert_called_once_with()
-        mock_user.assert_called_once_with(client.security)
+        mock_user.assert_called_once_with(
+            client.user,
+            authKey=client.auth_key,
+            authProtocol=client.auth_proto,
+            privKey=client.priv_key,
+            privProtocol=client.priv_proto,
+            contextEngineId=client.context_engine_id,
+            contextName=client.context_name
+        )
 
     @mock.patch.object(cmdgen, 'UdpTransportTarget', autospec=True)
     def test__get_transport(self, mock_transport, mock_cmdgen):
@@ -238,7 +245,7 @@ class SNMPValidateParametersTestCase(db_base.DbTestCase):
         self.assertEqual(INFO_DICT['snmp_outlet'], str(info['outlet']))
         self.assertEqual(INFO_DICT['snmp_version'], info['version'])
         self.assertEqual(INFO_DICT['snmp_community'], info['community'])
-        self.assertNotIn('security', info)
+        self.assertNotIn('user', info)
 
     def test__parse_driver_info_apc(self):
         # Make sure the APC driver type is parsed.
@@ -315,13 +322,156 @@ class SNMPValidateParametersTestCase(db_base.DbTestCase):
         self.assertEqual('private', info['community'])
 
     def test__parse_driver_info_snmp_v3(self):
+        # Make sure SNMPv3 is parsed with user string.
+        info = db_utils.get_test_snmp_info(snmp_version='3',
+                                           snmp_user='pass')
+        node = self._get_test_node(info)
+        info = snmp._parse_driver_info(node)
+        self.assertEqual('3', info['version'])
+        self.assertEqual('pass', info['user'])
+
+    def test__parse_driver_info_snmp_v3_auth_default_proto(self):
+        info = db_utils.get_test_snmp_info(snmp_version='3',
+                                           snmp_user='pass',
+                                           snmp_auth_key='12345678')
+        node = self._get_test_node(info)
+        info = snmp._parse_driver_info(node)
+        self.assertEqual('12345678', info['auth_key'])
+        self.assertEqual(snmp.snmp_auth_protocols['md5'],
+                         info['auth_protocol'])
+
+    def test__parse_driver_info_snmp_v3_auth_key_proto(self):
+        info = db_utils.get_test_snmp_info(snmp_version='3',
+                                           snmp_user='pass',
+                                           snmp_auth_key='12345678',
+                                           snmp_auth_protocol='sha')
+        node = self._get_test_node(info)
+        info = snmp._parse_driver_info(node)
+        self.assertEqual('12345678', info['auth_key'])
+        self.assertEqual(snmp.snmp_auth_protocols['sha'],
+                         info['auth_protocol'])
+
+    def test__parse_driver_info_snmp_v3_auth_nokey(self):
+        info = db_utils.get_test_snmp_info(snmp_version='3',
+                                           snmp_user='pass',
+                                           snmp_auth_protocol='sha')
+        node = self._get_test_node(info)
+        self.assertRaisesRegex(
+            exception.InvalidParameterValue,
+            'missing.*authentication key',
+            snmp._parse_driver_info,
+            node
+        )
+
+    def test__parse_driver_info_snmp_v3_auth_badproto(self):
+        info = db_utils.get_test_snmp_info(snmp_version='3',
+                                           snmp_user='pass',
+                                           snmp_auth_key='12345678',
+                                           snmp_auth_protocol='whatever')
+        node = self._get_test_node(info)
+        self.assertRaisesRegex(
+            exception.InvalidParameterValue,
+            '.*?unknown SNMPv3 authentication protocol.*',
+            snmp._parse_driver_info,
+            node
+        )
+
+    def test__parse_driver_info_snmp_v3_auth_short_key(self):
+        info = db_utils.get_test_snmp_info(snmp_version='3',
+                                           snmp_user='pass',
+                                           snmp_auth_key='1234567')
+        node = self._get_test_node(info)
+        self.assertRaisesRegex(
+            exception.InvalidParameterValue,
+            '.*?short SNMPv3 authentication key.*',
+            snmp._parse_driver_info,
+            node
+        )
+
+    def test__parse_driver_info_snmp_v3_priv_default_proto(self):
+        info = db_utils.get_test_snmp_info(snmp_version='3',
+                                           snmp_user='pass',
+                                           snmp_auth_key='12345678',
+                                           snmp_priv_key='87654321')
+        node = self._get_test_node(info)
+        info = snmp._parse_driver_info(node)
+        self.assertEqual('87654321', info['priv_key'])
+        self.assertEqual(snmp.snmp_priv_protocols['des'],
+                         info['priv_protocol'])
+
+    def test__parse_driver_info_snmp_v3_priv_key_proto(self):
+        info = db_utils.get_test_snmp_info(snmp_version='3',
+                                           snmp_user='pass',
+                                           snmp_auth_key='12345678',
+                                           snmp_priv_protocol='3des',
+                                           snmp_priv_key='87654321')
+        node = self._get_test_node(info)
+        info = snmp._parse_driver_info(node)
+        self.assertEqual('87654321', info['priv_key'])
+        self.assertEqual(snmp.snmp_priv_protocols['3des'],
+                         info['priv_protocol'])
+
+    def test__parse_driver_info_snmp_v3_priv_nokey(self):
+        info = db_utils.get_test_snmp_info(snmp_version='3',
+                                           snmp_user='pass',
+                                           snmp_priv_protocol='3des')
+        node = self._get_test_node(info)
+        self.assertRaisesRegex(
+            exception.InvalidParameterValue,
+            '.*?SNMPv3 privacy requires authentication.*',
+            snmp._parse_driver_info,
+            node
+        )
+
+    def test__parse_driver_info_snmp_v3_priv_badproto(self):
+        info = db_utils.get_test_snmp_info(snmp_version='3',
+                                           snmp_user='pass',
+                                           snmp_priv_key='12345678',
+                                           snmp_priv_protocol='whatever')
+        node = self._get_test_node(info)
+        self.assertRaisesRegex(
+            exception.InvalidParameterValue,
+            '.*?unknown SNMPv3 privacy protocol.*',
+            snmp._parse_driver_info,
+            node
+        )
+
+    def test__parse_driver_info_snmp_v3_priv_short_key(self):
+        info = db_utils.get_test_snmp_info(snmp_version='3',
+                                           snmp_user='pass',
+                                           snmp_priv_key='1234567')
+        node = self._get_test_node(info)
+        self.assertRaisesRegex(
+            exception.InvalidParameterValue,
+            '.*?short SNMPv3 privacy key.*',
+            snmp._parse_driver_info,
+            node
+        )
+
+    def test__parse_driver_info_snmp_v3_compat(self):
         # Make sure SNMPv3 is parsed with a security string.
         info = db_utils.get_test_snmp_info(snmp_version='3',
                                            snmp_security='pass')
         node = self._get_test_node(info)
         info = snmp._parse_driver_info(node)
         self.assertEqual('3', info['version'])
-        self.assertEqual('pass', info['security'])
+        self.assertEqual('pass', info['user'])
+
+    def test__parse_driver_info_snmp_v3_context_engine_id(self):
+        info = db_utils.get_test_snmp_info(snmp_version='3',
+                                           snmp_user='pass',
+                                           snmp_context_engine_id='whatever')
+        node = self._get_test_node(info)
+        info = snmp._parse_driver_info(node)
+        self.assertEqual('whatever', info['context_engine_id'])
+
+    def test__parse_driver_info_snmp_v3_context_name(self):
+        info = db_utils.get_test_snmp_info(snmp_version='3',
+                                           snmp_user='pass',
+                                           snmp_context_name='whatever')
+        node = self._get_test_node(info)
+        info = snmp._parse_driver_info(node)
+        self.assertEqual('whatever', info['context_name'])
 
     def test__parse_driver_info_snmp_port_default(self):
         # Make sure default SNMP UDP port numbers are correct
@@ -395,7 +545,7 @@ class SNMPValidateParametersTestCase(db_base.DbTestCase):
         # Make sure exception is raised when version is invalid.
         info = db_utils.get_test_snmp_info(snmp_version='42',
                                            snmp_community='public',
-                                           snmp_security='pass')
+                                           snmp_user='pass')
         node = self._get_test_node(info)
         self.assertRaises(exception.InvalidParameterValue,
                           snmp._parse_driver_info,
@@ -429,10 +579,10 @@ class SNMPValidateParametersTestCase(db_base.DbTestCase):
                           snmp._parse_driver_info,
                           node)
 
-    def test__parse_driver_info_missing_security(self):
-        # Make sure exception is raised when security is missing with SNMPv3.
+    def test__parse_driver_info_missing_user(self):
+        # Make sure exception is raised when user is missing with SNMPv3.
         info = db_utils.get_test_snmp_info(snmp_version='3')
-        del info['snmp_security']
+        del info['snmp_user']
         node = self._get_test_node(info)
         self.assertRaises(exception.MissingParameterValue,
                           snmp._parse_driver_info,
@@ -445,12 +595,26 @@ class SNMPDeviceDriverTestCase(db_base.DbTestCase):
 
     The SNMP client object is mocked to allow various error cases to be tested.
     """
+    pdus = {
+        (1, 3, 6, 1, 4, 1, 318, 1, 1, 4): 'apc_masterswitch',
+        # also try longer sysObjectID
+        (1, 3, 6, 1, 4, 1, 318, 1, 1, 4, 1, 2, 3, 4): 'apc_masterswitch',
+        (1, 3, 6, 1, 4, 1, 318, 1, 1, 6): 'apc_masterswitchplus',
+        (1, 3, 6, 1, 4, 1, 318, 1, 1, 12): 'apc_rackpdu',
+        (1, 3, 6, 1, 4, 1, 21317): 'aten',
+        (1, 3, 6, 1, 4, 1, 3808): 'cyberpower',
+        (1, 3, 6, 1, 4, 1, 23620): 'teltronix',
+        # TODO(etingof): SNMPDriverEatonPower misses the `.oid` attribute
+        # and therefore fails tests
+        # (1, 3, 6, 1, 4, 1, 534): 'eatonpower',
+    }
 
     def setUp(self):
         super(SNMPDeviceDriverTestCase, self).setUp()
+        self.config(enabled_power_interfaces=['fake', 'snmp'])
         self.node = obj_utils.get_test_node(
             self.context,
-            driver='fake_snmp',
+            power_interface='snmp',
             driver_info=INFO_DICT)
 
     def _update_driver_info(self, **kwargs):
@@ -741,8 +905,8 @@ class SNMPDeviceDriverTestCase(db_base.DbTestCase):
         mock_client = mock_get_client.return_value
         driver = snmp._get_driver(self.node)
         attempts = CONF.snmp.power_timeout // driver.retry_interval
-        mock_client.get.side_effect = ([driver.value_power_off] +
-                                       [42] * attempts)
+        mock_client.get.side_effect = ([driver.value_power_off]
+                                       + [42] * attempts)
         pstate = driver.power_reset()
         calls = [mock.call(driver._snmp_oid(), driver.value_power_off),
                  mock.call(driver._snmp_oid(), driver.value_power_on)]
@@ -773,8 +937,8 @@ class SNMPDeviceDriverTestCase(db_base.DbTestCase):
         mock_client = mock_get_client.return_value
         driver = snmp._get_driver(self.node)
         attempts = CONF.snmp.power_timeout // driver.retry_interval
-        mock_client.get.side_effect = ([driver.value_power_off] *
-                                       (1 + attempts))
+        mock_client.get.side_effect = ([driver.value_power_off]
+                                       * (1 + attempts))
         pstate = driver.power_reset()
         calls = [mock.call(driver._snmp_oid(), driver.value_power_off),
                  mock.call(driver._snmp_oid(), driver.value_power_on)]
@@ -1105,6 +1269,130 @@ class SNMPDeviceDriverTestCase(db_base.DbTestCase):
     def test_teltronix_power_reset(self, mock_get_client):
         self._test_simple_device_power_reset('teltronix', mock_get_client)
 
+    def test_auto_power_state_unknown_pdu(self, mock_get_client):
+        mock_client = mock_get_client.return_value
+        mock_client.get.return_value = 'unknown'
+        self._update_driver_info(snmp_driver="auto")
+        self.assertRaises(exception.InvalidParameterValue,
+                          snmp._get_driver,
+                          self.node)
+
+    def test_auto_power_state_pdu_discovery_failure(self, mock_get_client):
+        mock_client = mock_get_client.return_value
+        mock_client.get.side_effect = exception.SNMPFailure(operation='get',
+                                                            error='')
+        self._update_driver_info(snmp_driver="auto")
+        self.assertRaises(exception.SNMPFailure, snmp._get_driver, self.node)
+
+    def test_auto_power_state_on(self, mock_get_client):
+        for sys_obj_oid, expected_snmp_driver in self.pdus.items():
+            mock_client = mock_get_client.return_value
+            mock_client.reset_mock()
+            mock_client.get.return_value = sys_obj_oid
+            self._update_driver_info(snmp_driver="auto")
+            driver = snmp._get_driver(self.node)
+
+            second_node = obj_utils.get_test_node(
+                self.context,
+                driver='fake_snmp',
+                driver_info=INFO_DICT)
+            second_node["driver_info"].update(snmp_driver=expected_snmp_driver)
+            second_node_driver = snmp._get_driver(second_node)
+            mock_client.get.return_value = second_node_driver.value_power_on
+
+            pstate = driver.power_state()
+            mock_client.get.assert_called_with(second_node_driver.oid)
+            self.assertEqual(states.POWER_ON, pstate)
+
+    def test_auto_power_state_off(self, mock_get_client):
+        for sys_obj_oid, expected_snmp_driver in self.pdus.items():
+            mock_client = mock_get_client.return_value
+            mock_client.reset_mock()
+            mock_client.get.return_value = sys_obj_oid
+            self._update_driver_info(snmp_driver="auto")
+            driver = snmp._get_driver(self.node)
+
+            second_node = obj_utils.get_test_node(
+                self.context,
+                driver='fake_snmp',
+                driver_info=INFO_DICT)
+            second_node["driver_info"].update(snmp_driver=expected_snmp_driver)
+            second_node_driver = snmp._get_driver(second_node)
+            mock_client.get.return_value = second_node_driver.value_power_off
+
+            pstate = driver.power_state()
+            mock_client.get.assert_called_with(second_node_driver.oid)
+            self.assertEqual(states.POWER_OFF, pstate)
+
+    def test_auto_power_on(self, mock_get_client):
+        for sys_obj_oid, expected_snmp_driver in self.pdus.items():
+            mock_client = mock_get_client.return_value
+            mock_client.reset_mock()
+            mock_client.get.return_value = sys_obj_oid
+            self._update_driver_info(snmp_driver="auto")
+            driver = snmp._get_driver(self.node)
+
+            second_node = obj_utils.get_test_node(
+                self.context,
+                driver='fake_snmp',
+                driver_info=INFO_DICT)
+            second_node["driver_info"].update(snmp_driver=expected_snmp_driver)
+            second_node_driver = snmp._get_driver(second_node)
+            mock_client.get.return_value = second_node_driver.value_power_on
+
+            pstate = driver.power_on()
+            mock_client.set.assert_called_once_with(
+                second_node_driver.oid,
+                second_node_driver.value_power_on)
+            self.assertEqual(states.POWER_ON, pstate)
+
+    def test_auto_power_off(self, mock_get_client):
+        for sys_obj_oid, expected_snmp_driver in self.pdus.items():
+            mock_client = mock_get_client.return_value
+            mock_client.reset_mock()
+            mock_client.get.return_value = sys_obj_oid
+            self._update_driver_info(snmp_driver="auto")
+            driver = snmp._get_driver(self.node)
+
+            second_node = obj_utils.get_test_node(
+                self.context,
+                driver='fake_snmp',
+                driver_info=INFO_DICT)
+            second_node["driver_info"].update(snmp_driver=expected_snmp_driver)
+            second_node_driver = snmp._get_driver(second_node)
+            mock_client.get.return_value = second_node_driver.value_power_off
+
+            pstate = driver.power_off()
+            mock_client.set.assert_called_once_with(
+                second_node_driver.oid,
+                second_node_driver.value_power_off)
+            self.assertEqual(states.POWER_OFF, pstate)
+
+    def test_auto_power_reset(self, mock_get_client):
+        for sys_obj_oid, expected_snmp_driver in self.pdus.items():
+            mock_client = mock_get_client.return_value
+            mock_client.reset_mock()
+            mock_client.get.side_effect = [sys_obj_oid, sys_obj_oid]
+            self._update_driver_info(snmp_driver="auto")
+            driver = snmp._get_driver(self.node)
+
+            second_node = obj_utils.get_test_node(
+                self.context,
+                driver='fake_snmp',
+                driver_info=INFO_DICT)
+            second_node["driver_info"].update(snmp_driver=expected_snmp_driver)
+            second_node_driver = snmp._get_driver(second_node)
+            mock_client.get.side_effect = [second_node_driver.value_power_off,
+                                           second_node_driver.value_power_on]
+
+            pstate = driver.power_reset()
+            calls = [mock.call(second_node_driver.oid,
+                               second_node_driver.value_power_off),
+                     mock.call(second_node_driver.oid,
+                               second_node_driver.value_power_on)]
+            mock_client.set.assert_has_calls(calls)
+            self.assertEqual(states.POWER_ON, pstate)
+
     def test_eaton_power_snmp_objects(self, mock_get_client):
         # Ensure the correct SNMP object OIDs and values are used by the Eaton
         # Power driver
@@ -1208,6 +1496,34 @@ class SNMPDeviceDriverTestCase(db_base.DbTestCase):
         mock_client.get.assert_has_calls(calls)
         self.assertEqual(states.POWER_ON, pstate)
 
+    def test_baytech_mrp27_power_snmp_objects(self, mock_get_client):
+        # Ensure the correct SNMP object OIDs and values are used by the
+        # Baytech MRP 27 Power driver
+        self._update_driver_info(snmp_driver="baytech_mrp27", snmp_outlet="3")
+        driver = snmp._get_driver(self.node)
+
+        oid = (1, 3, 6, 1, 4, 1, 4779, 1, 3, 5, 3, 1, 3, 1, 3)
+        self.assertEqual(oid, driver._snmp_oid())
+        self.assertEqual(1, driver.value_power_on)
+        self.assertEqual(0, driver.value_power_off)
+
+    def test_baytech_mrp27_power_state_on(self, mock_get_client):
+        self._test_simple_device_power_state_on('baytech_mrp27',
+                                                mock_get_client)
+
+    def test_baytech_mrp27_power_state_off(self, mock_get_client):
+        self._test_simple_device_power_state_off('baytech_mrp27',
+                                                 mock_get_client)
+
+    def test_baytech_mrp27_power_on(self, mock_get_client):
+        self._test_simple_device_power_on('baytech_mrp27', mock_get_client)
+
+    def test_baytech_mrp27_power_off(self, mock_get_client):
+        self._test_simple_device_power_off('baytech_mrp27', mock_get_client)
+
+    def test_baytech_mrp27_power_reset(self, mock_get_client):
+        self._test_simple_device_power_reset('baytech_mrp27', mock_get_client)
+
 
 @mock.patch.object(snmp, '_get_driver', autospec=True)
 class SNMPDriverTestCase(db_base.DbTestCase):
@@ -1220,10 +1536,11 @@ class SNMPDriverTestCase(db_base.DbTestCase):
 
     def setUp(self):
         super(SNMPDriverTestCase, self).setUp()
-        mgr_utils.mock_the_extension_manager(driver='fake_snmp')
+        self.config(enabled_power_interfaces=['fake', 'snmp'])
 
         self.node = obj_utils.create_test_node(self.context,
-                                               driver='fake_snmp',
+                                               power_interface='snmp',
+                                               vendor_interface='no-vendor',
                                                driver_info=INFO_DICT)
 
     def _get_snmp_failure(self):
