@@ -16,9 +16,7 @@
 import json
 
 import mock
-import stevedore
 
-from ironic.common import driver_factory
 from ironic.common import exception
 from ironic.common import raid
 from ironic.drivers import base as driver_base
@@ -102,6 +100,16 @@ class PassthruDecoratorTestCase(base.TestCase):
         self.assertNotEqual(inst1.driver_routes['driver_noexception']['func'],
                             inst2.driver_routes['driver_noexception']['func'])
 
+    @mock.patch.object(driver_base.LOG, 'warning')
+    def test_old_async_warning(self, mock_log_warning):
+        driver_base.passthru(['POST'], async=True)
+        mock_log_warning.assert_called_once()
+
+    @mock.patch.object(driver_base.LOG, 'warning')
+    def test_new_async_call_without_warning(self, mock_log_warning):
+        driver_base.passthru(['POST'], async_call=True)
+        mock_log_warning.assert_not_called()
+
 
 class CleanStepDecoratorTestCase(base.TestCase):
 
@@ -412,6 +420,53 @@ class TestDeployInterface(base.TestCase):
         self.assertTrue(mock_log.called)
 
 
+class MyBIOSInterface(driver_base.BIOSInterface):
+
+    def get_properties(self):
+        pass
+
+    def validate(self, task):
+        pass
+
+    def apply_configuration(self, task, settings):
+        pass
+
+    def factory_reset(self, task):
+        pass
+
+    def cache_bios_settings(self, task):
+        pass
+
+
+class TestBIOSInterface(base.TestCase):
+
+    @mock.patch.object(MyBIOSInterface, 'cache_bios_settings', autospec=True)
+    def test_apply_configuration_wrapper(self, cache_bios_settings_mock):
+        bios = MyBIOSInterface()
+        task_mock = mock.MagicMock()
+
+        bios.apply_configuration(task_mock, "")
+        cache_bios_settings_mock.assert_called_once_with(bios, task_mock)
+
+    @mock.patch.object(MyBIOSInterface, 'cache_bios_settings', autospec=True)
+    def test_factory_reset_wrapper(self, cache_bios_settings_mock):
+        bios = MyBIOSInterface()
+        task_mock = mock.MagicMock()
+
+        bios.factory_reset(task_mock)
+        cache_bios_settings_mock.assert_called_once_with(bios, task_mock)
+
+
+class TestBootInterface(base.TestCase):
+
+    def test_validate_rescue_default_impl(self):
+        boot = fake.FakeBoot()
+        task_mock = mock.MagicMock(spec_set=['node'])
+
+        self.assertRaises(exception.UnsupportedDriverExtension,
+                          boot.validate_rescue, task_mock)
+
+
 class TestManagementInterface(base.TestCase):
 
     def test_inject_nmi_default_impl(self):
@@ -421,153 +476,34 @@ class TestManagementInterface(base.TestCase):
         self.assertRaises(exception.UnsupportedDriverExtension,
                           management.inject_nmi, task_mock)
 
+    def test_get_supported_boot_modes_default_impl(self):
+        management = fake.FakeManagement()
+        task_mock = mock.MagicMock(spec_set=['node'])
 
-class TestBaseDriver(base.TestCase):
+        self.assertRaises(exception.UnsupportedDriverExtension,
+                          management.get_supported_boot_modes, task_mock)
 
-    def test_class_variables_immutable(self):
-        # Test to make sure that our *_interfaces variables in the class don't
-        # get modified by a child class
-        self.assertEqual(('deploy', 'power'),
-                         driver_base.BaseDriver.core_interfaces)
-        self.assertEqual(('boot', 'console', 'inspect', 'management', 'raid'),
-                         driver_base.BaseDriver.standard_interfaces)
-        # Ensure that instantiating an instance of a derived class does not
-        # change our variables.
-        driver_base.BareDriver()
+    def test_set_boot_mode_default_impl(self):
+        management = fake.FakeManagement()
+        task_mock = mock.MagicMock(spec_set=['node'])
 
-        self.assertEqual(('deploy', 'power'),
-                         driver_base.BaseDriver.core_interfaces)
-        self.assertEqual(('boot', 'console', 'inspect', 'management', 'raid'),
-                         driver_base.BaseDriver.standard_interfaces)
+        self.assertRaises(exception.UnsupportedDriverExtension,
+                          management.set_boot_mode, task_mock, 'whatever')
+
+    def test_get_boot_mode_default_impl(self):
+        management = fake.FakeManagement()
+        task_mock = mock.MagicMock(spec_set=['node'])
+
+        self.assertRaises(exception.UnsupportedDriverExtension,
+                          management.get_boot_mode, task_mock)
 
 
 class TestBareDriver(base.TestCase):
 
-    def test_class_variables_immutable(self):
-        # Test to make sure that our *_interfaces variables in the class don't
-        # get modified by a child class
-        self.assertEqual(('deploy', 'power', 'network'),
-                         driver_base.BareDriver.core_interfaces)
+    def test_class_variables(self):
+        self.assertEqual(['boot', 'deploy', 'management', 'network', 'power'],
+                         driver_base.BareDriver().core_interfaces)
         self.assertEqual(
-            ('boot', 'console', 'inspect', 'management', 'raid',
-             'rescue', 'storage'),
-            driver_base.BareDriver.standard_interfaces
+            ['bios', 'console', 'inspect', 'raid', 'rescue', 'storage'],
+            driver_base.BareDriver().optional_interfaces
         )
-
-
-class TestToHardwareType(base.TestCase):
-    def setUp(self):
-        super(TestToHardwareType, self).setUp()
-        self.driver_classes = list(
-            driver_factory.classic_drivers_to_migrate().values())
-        self.existing_ifaces = {}
-        for iface in driver_base.ALL_INTERFACES:
-            self.existing_ifaces[iface] = stevedore.ExtensionManager(
-                'ironic.hardware.interfaces.%s' % iface,
-                invoke_on_load=False).names()
-        self.hardware_types = stevedore.ExtensionManager(
-            'ironic.hardware.types', invoke_on_load=False).names()
-        # These are the interfaces that don't have a no-op version
-        self.mandatory_interfaces = ['boot', 'deploy', 'management', 'power']
-
-    def test_to_hardware_type_returns_hardware_type(self):
-        for driver in self.driver_classes:
-            try:
-                hw_type = driver.to_hardware_type()[0]
-            except NotImplementedError:
-                continue
-            except KeyError:
-                self.fail('%s does not return a tuple' % driver)
-
-            self.assertIn(hw_type, self.hardware_types,
-                          '%s returns unknown hardware type %s' %
-                          (driver, hw_type))
-
-    def test_to_hardware_type_returns_existing_interfaces(self):
-        self.config(enabled=False, group='inspector')
-        # Check that all defined implementations of to_hardware_type
-        # contain only existing interface types
-        for driver in self.driver_classes:
-            try:
-                delta = driver.to_hardware_type()[1]
-            except Exception:
-                continue  # covered by other tests
-            for iface, value in delta.items():
-                self.assertIn(iface, self.existing_ifaces,
-                              '%s returns unknown interface %s' %
-                              (driver, iface))
-                self.assertIn(value, self.existing_ifaces[iface],
-                              '%s returns unknown %s interface %s' %
-                              (driver, iface, value))
-
-    def test_to_hardware_type_returns_existing_interfaces_inspector(self):
-        self.config(enabled=True, group='inspector')
-        # Check that all defined implementations of to_hardware_type
-        # contain only existing interface types
-        for driver in self.driver_classes:
-            try:
-                delta = driver.to_hardware_type()[1]
-            except Exception:
-                continue  # covered by other tests
-            for iface, value in delta.items():
-                self.assertIn(iface, self.existing_ifaces,
-                              '%s returns unknown interface %s' %
-                              (driver, iface))
-                self.assertIn(value, self.existing_ifaces[iface],
-                              '%s returns unknown %s interface %s' %
-                              (driver, iface, value))
-
-    def test_to_hardware_type_mandatory_interfaces(self):
-        for driver in self.driver_classes:
-            try:
-                delta = driver.to_hardware_type()[1]
-            except Exception:
-                continue  # covered by other tests
-            for iface in self.mandatory_interfaces:
-                self.assertIn(iface, delta,
-                              '%s does not return mandatory interface %s' %
-                              (driver, iface))
-
-    def test_to_hardware_type_for_all_in_tree_drivers(self):
-        missing = set()
-        for driver in self.driver_classes:
-            # We don't want to test out-of-tree drivers installed locally
-            if not driver.__module__.startswith('ironic.'):
-                continue
-            try:
-                driver.to_hardware_type()
-            except NotImplementedError:
-                missing.add(driver.__name__)
-
-        if missing:
-            self.fail('to_hardware_type not implemented for %s' %
-                      ', '.join(missing))
-
-    def test_to_hardware_type_boot_deploy(self):
-        for driver in self.driver_classes:
-            # We don't want to test out-of-tree drivers installed locally
-            if not driver.__module__.startswith('ironic.'):
-                continue
-
-            try:
-                delta = driver.to_hardware_type()[1]
-                boot = delta['boot']
-                deploy = delta['deploy']
-            except NotImplementedError:
-                continue  # covered by other tests
-
-            name = driver.__name__.lower()
-            # Try to guess the correct values for boot and deploy based on our
-            # naming schema
-            if 'pxe' in name:
-                self.assertIn('pxe', boot,
-                              'boot interface should be based on pxe for %s' %
-                              driver)
-            if 'agent' in name:
-                self.assertIn('direct', deploy,
-                              'deploy interface should be direct for %s' %
-                              driver)
-            elif 'iscsi' in name or 'pxe' in name:
-                self.assertIn('iscsi', deploy,
-                              'deploy interface should be iscsi for %s' %
-                              driver)
