@@ -16,19 +16,9 @@
 Test class for iLO Drivers
 """
 
-import mock
-import testtools
-
-from ironic.common import exception
 from ironic.conductor import task_manager
 from ironic.drivers import ilo
 from ironic.drivers.modules import agent
-from ironic.drivers.modules.ilo import boot
-from ironic.drivers.modules.ilo import console
-from ironic.drivers.modules.ilo import inspect
-from ironic.drivers.modules.ilo import management
-from ironic.drivers.modules.ilo import power
-from ironic.drivers.modules.ilo import vendor
 from ironic.drivers.modules import inspector
 from ironic.drivers.modules import iscsi_deploy
 from ironic.drivers.modules import noop
@@ -48,6 +38,7 @@ class IloHardwareTestCase(db_base.DbTestCase):
                     enabled_management_interfaces=['ilo'],
                     enabled_power_interfaces=['ilo'],
                     enabled_raid_interfaces=['no-raid', 'agent'],
+                    enabled_rescue_interfaces=['no-rescue', 'agent'],
                     enabled_vendor_interfaces=['ilo', 'no-vendor'])
 
     def test_default_interfaces(self):
@@ -70,6 +61,8 @@ class IloHardwareTestCase(db_base.DbTestCase):
                                   noop.NoRAID)
             self.assertIsInstance(task.driver.vendor,
                                   ilo.vendor.VendorPassthru)
+            self.assertIsInstance(task.driver.rescue,
+                                  noop.NoRescue)
 
     def test_override_with_inspector(self):
         self.config(enabled_inspect_interfaces=['inspector', 'ilo'])
@@ -94,6 +87,8 @@ class IloHardwareTestCase(db_base.DbTestCase):
                                   ilo.power.IloPower)
             self.assertIsInstance(task.driver.raid,
                                   agent.AgentRAID)
+            self.assertIsInstance(task.driver.rescue,
+                                  noop.NoRescue)
             self.assertIsInstance(task.driver.vendor,
                                   noop.NoVendor)
 
@@ -117,54 +112,34 @@ class IloHardwareTestCase(db_base.DbTestCase):
                                   ilo.power.IloPower)
             self.assertIsInstance(task.driver.raid,
                                   agent.AgentRAID)
+            self.assertIsInstance(task.driver.rescue,
+                                  noop.NoRescue)
             self.assertIsInstance(task.driver.vendor,
                                   ilo.vendor.VendorPassthru)
 
-
-@mock.patch.object(ilo.importutils, 'try_import', spec_set=True,
-                   autospec=True)
-class IloVirtualMediaIscsiDriversTestCase(testtools.TestCase):
-
-    def test_ilo_iscsi_driver(self, mock_try_import):
-        mock_try_import.return_value = True
-
-        driver = ilo.IloVirtualMediaIscsiDriver()
-
-        self.assertIsInstance(driver.power, power.IloPower)
-        self.assertIsInstance(driver.boot, boot.IloVirtualMediaBoot)
-        self.assertIsInstance(driver.deploy, iscsi_deploy.ISCSIDeploy)
-        self.assertIsInstance(driver.console, console.IloConsoleInterface)
-        self.assertIsInstance(driver.management, management.IloManagement)
-        self.assertIsInstance(driver.vendor, vendor.VendorPassthru)
-        self.assertIsInstance(driver.inspect, inspect.IloInspect)
-        self.assertIsInstance(driver.raid, agent.AgentRAID)
-
-    def test_ilo_iscsi_driver_exc(self, mock_try_import):
-        mock_try_import.return_value = None
-
-        self.assertRaises(exception.DriverLoadError,
-                          ilo.IloVirtualMediaIscsiDriver)
-
-
-@mock.patch.object(ilo.importutils, 'try_import', spec_set=True,
-                   autospec=True)
-class IloVirtualMediaAgentDriversTestCase(testtools.TestCase):
-
-    def test_ilo_agent_driver(self, mock_try_import):
-        mock_try_import.return_value = True
-
-        driver = ilo.IloVirtualMediaAgentDriver()
-
-        self.assertIsInstance(driver.power, power.IloPower)
-        self.assertIsInstance(driver.boot, boot.IloVirtualMediaBoot)
-        self.assertIsInstance(driver.deploy, agent.AgentDeploy)
-        self.assertIsInstance(driver.console, console.IloConsoleInterface)
-        self.assertIsInstance(driver.management, management.IloManagement)
-        self.assertIsInstance(driver.inspect, inspect.IloInspect)
-        self.assertIsInstance(driver.raid, agent.AgentRAID)
-
-    def test_ilo_iscsi_driver_exc(self, mock_try_import):
-        mock_try_import.return_value = None
-
-        self.assertRaises(exception.DriverLoadError,
-                          ilo.IloVirtualMediaAgentDriver)
+    def test_override_with_agent_rescue(self):
+        self.config(enabled_inspect_interfaces=['inspector', 'ilo'])
+        node = obj_utils.create_test_node(
+            self.context, driver='ilo',
+            deploy_interface='direct',
+            rescue_interface='agent',
+            raid_interface='agent')
+        with task_manager.acquire(self.context, node.id) as task:
+            self.assertIsInstance(task.driver.boot,
+                                  ilo.boot.IloVirtualMediaBoot)
+            self.assertIsInstance(task.driver.console,
+                                  ilo.console.IloConsoleInterface)
+            self.assertIsInstance(task.driver.deploy,
+                                  agent.AgentDeploy)
+            self.assertIsInstance(task.driver.inspect,
+                                  ilo.inspect.IloInspect)
+            self.assertIsInstance(task.driver.management,
+                                  ilo.management.IloManagement)
+            self.assertIsInstance(task.driver.power,
+                                  ilo.power.IloPower)
+            self.assertIsInstance(task.driver.raid,
+                                  agent.AgentRAID)
+            self.assertIsInstance(task.driver.rescue,
+                                  agent.AgentRescue)
+            self.assertIsInstance(task.driver.vendor,
+                                  ilo.vendor.VendorPassthru)
