@@ -163,6 +163,8 @@ SUSHY_CONSTANTS_SPEC = (
     'RESET_NMI',
     'BOOT_SOURCE_ENABLED_CONTINUOUS',
     'BOOT_SOURCE_ENABLED_ONCE',
+    'BOOT_SOURCE_MODE_BIOS',
+    'BOOT_SOURCE_MODE_UEFI',
 )
 
 XCLARITY_SPEC = (
