@@ -483,8 +483,7 @@ class TestConvertToVersion(db_base.DbTestCase):
         node._convert_to_version("1.22", False)
 
         self.assertIsNone(node.traits)
-        self.assertEqual({'traits': None},
-                         node.obj_get_changes())
+        self.assertEqual({'traits': None}, node.obj_get_changes())
 
     def test_trait_unsupported_set_no_remove_default(self):
         # traits set, no change required.
@@ -497,6 +496,121 @@ class TestConvertToVersion(db_base.DbTestCase):
         self.assertIsNone(node.traits)
         self.assertEqual({}, node.obj_get_changes())
 
+    def test_bios_supported_missing(self):
+        # bios_interface not set, should be set to default.
+        node = obj_utils.get_test_node(self.ctxt, **self.fake_node)
+        delattr(node, 'bios_interface')
+        node.obj_reset_changes()
+
+        node._convert_to_version("1.24")
+
+        self.assertIsNone(node.bios_interface)
+        self.assertEqual({'bios_interface': None},
+                         node.obj_get_changes())
+
+    def test_bios_supported_set(self):
+        # bios_interface set, no change required.
+        node = obj_utils.get_test_node(self.ctxt, **self.fake_node)
+
+        node.bios_interface = 'fake'
+        node.obj_reset_changes()
+        node._convert_to_version("1.24")
+        self.assertEqual('fake', node.bios_interface)
+        self.assertEqual({}, node.obj_get_changes())
+
+    def test_bios_unsupported_missing(self):
+        # bios_interface not set, no change required.
+        node = obj_utils.get_test_node(self.ctxt, **self.fake_node)
+
+        delattr(node, 'bios_interface')
+        node.obj_reset_changes()
+        node._convert_to_version("1.23")
+        self.assertNotIn('bios_interface', node)
+        self.assertEqual({}, node.obj_get_changes())
+
+    def test_bios_unsupported_set_remove(self):
+        # bios_interface set, should be removed.
+        node = obj_utils.get_test_node(self.ctxt, **self.fake_node)
+
+        node.bios_interface = 'fake'
+        node.obj_reset_changes()
+        node._convert_to_version("1.23")
+        self.assertNotIn('bios_interface', node)
+        self.assertEqual({}, node.obj_get_changes())
+
+    def test_bios_unsupported_set_no_remove_non_default(self):
+        # bios_interface set, should be set to default.
+        node = obj_utils.get_test_node(self.ctxt, **self.fake_node)
+
+        node.bios_interface = 'fake'
+        node.obj_reset_changes()
+        node._convert_to_version("1.23", False)
+        self.assertIsNone(node.bios_interface)
+        self.assertEqual({'bios_interface': None}, node.obj_get_changes())
+
+    def test_bios_unsupported_set_no_remove_default(self):
+        # bios_interface set, no change required.
+        node = obj_utils.get_test_node(self.ctxt, **self.fake_node)
+
+        node.bios_interface = None
+        node.obj_reset_changes()
+        node._convert_to_version("1.23", False)
+        self.assertIsNone(node.bios_interface)
+        self.assertEqual({}, node.obj_get_changes())
+
+    def test_fault_supported_missing(self):
+        node = obj_utils.get_test_node(self.ctxt, **self.fake_node)
+        delattr(node, 'fault')
+        node.obj_reset_changes()
+
+        node._convert_to_version("1.25")
+
+        self.assertIsNone(node.fault)
+        self.assertEqual({'fault': None}, node.obj_get_changes())
+
+    def test_fault_supported_untouched(self):
+        node = obj_utils.get_test_node(self.ctxt, **self.fake_node)
+        node.maintenance = True
+        node.fault = 'a fake fault'
+        node.obj_reset_changes()
+
+        node._convert_to_version("1.25")
+
+        self.assertEqual('a fake fault', node.fault)
+        self.assertEqual({}, node.obj_get_changes())
+
+    def test_fault_unsupported_missing(self):
+        node = obj_utils.get_test_node(self.ctxt, **self.fake_node)
+        delattr(node, 'fault')
+        node.obj_reset_changes()
+
+        node._convert_to_version("1.24")
+
+        self.assertNotIn('fault', node)
+        self.assertEqual({}, node.obj_get_changes())
+
+    def test_fault_unsupported_set_remove(self):
+        node = obj_utils.get_test_node(self.ctxt, **self.fake_node)
+        node.maintenance = True
+        node.fault = 'some fake fault'
+        node.obj_reset_changes()
+
+        node._convert_to_version("1.24")
+
+        self.assertNotIn('fault', node)
+        self.assertEqual({}, node.obj_get_changes())
+
+    def test_fault_unsupported_set_remove_in_maintenance(self):
+        node = obj_utils.get_test_node(self.ctxt, **self.fake_node)
+        node.maintenance = True
+        node.fault = 'some fake type'
+        node.obj_reset_changes()
+
+        node._convert_to_version("1.24", False)
+
+        self.assertIsNone(node.fault)
+        self.assertEqual({'fault': None}, node.obj_get_changes())
+
 
 class TestNodePayloads(db_base.DbTestCase):
 
@@ -522,6 +636,8 @@ class TestNodePayloads(db_base.DbTestCase):
         self.assertEqual(self.node.maintenance, payload.maintenance)
         self.assertEqual(self.node.maintenance_reason,
                          payload.maintenance_reason)
+        self.assertEqual(self.node.fault, payload.fault)
+        self.assertEqual(self.node.bios_interface, payload.bios_interface)
         self.assertEqual(self.node.boot_interface, payload.boot_interface)
         self.assertEqual(self.node.console_interface,
                          payload.console_interface)
