@@ -69,7 +69,7 @@ class MyObj(base.IronicObject, object_base.VersionedObjectDictCompat):
 
     @object_base.remotable
     def update_test(self, context=None):
-        if context and context.tenant == 'alternate':
+        if context and context.project_id == 'alternate':
             self.bar = 'alternate-context'
         else:
             self.bar = 'updated'
@@ -232,8 +232,9 @@ class _TestObject(object):
                           base.IronicObject.obj_class_from_name, 'foo', '1.0')
 
     def test_with_alternate_context(self):
-        ctxt1 = context.RequestContext(auth_token='foo', tenant='foo')
-        ctxt2 = context.RequestContext(auth_token='bar', tenant='alternate')
+        ctxt1 = context.RequestContext(auth_token='foo', project_id='foo')
+        ctxt2 = context.RequestContext(auth_token='bar',
+                                       project_id='alternate')
         obj = MyObj.query(ctxt1)
         obj.update_test(ctxt2)
         self.assertEqual('alternate-context', obj.bar)
@@ -349,11 +350,11 @@ class _TestObject(object):
         myobj3_fields = ['new_field']
         self.assertTrue(issubclass(TestSubclassedObject, MyObj))
         self.assertEqual(len(myobj_fields), len(MyObj.fields))
-        self.assertEqual(set(myobj_fields), set(MyObj.fields.keys()))
+        self.assertEqual(set(myobj_fields), set(MyObj.fields))
         self.assertEqual(len(myobj_fields) + len(myobj3_fields),
                          len(TestSubclassedObject.fields))
         self.assertEqual(set(myobj_fields) | set(myobj3_fields),
-                         set(TestSubclassedObject.fields.keys()))
+                         set(TestSubclassedObject.fields))
 
     def _test_get_changes(self, target_version='1.5'):
         obj = MyObj(self.context)
@@ -506,66 +507,11 @@ class _TestObject(object):
     def test__from_db_object_old(self):
         self._test__from_db_object('1.4')
 
-    @mock.patch('ironic.common.release_mappings.RELEASE_MAPPING',
-                autospec=True)
-    def test__from_db_object_no_version(self, mock_release_mapping):
-        # DB doesn't have version; get it from mapping
-        mock_release_mapping.__getitem__.return_value = {
-            'objects': {
-                'MyObj': ['1.4'],
-            }
-        }
+    def test__from_db_object_map_version_bad(self):
         obj = MyObj(self.context)
         dbobj = {'created_at': timeutils.utcnow(),
                  'updated_at': timeutils.utcnow(),
-                 'version': None,
-                 'foo': 123, 'bar': 'test', 'missing': ''}
-        MyObj._from_db_object(self.context, obj, dbobj)
-        self.assertEqual(obj.__class__.VERSION, obj.VERSION)
-        self.assertEqual(123, obj.foo)
-        self.assertEqual('test', obj.bar)
-        self.assertEqual('foo', obj.missing)
-
-    @mock.patch('ironic.common.release_mappings.RELEASE_MAPPING',
-                autospec=True)
-    def test__from_db_object_no_version_subset_of_fields(self,
-                                                         mock_release_mapping):
-        # DB doesn't have version; get it from mapping
-        mock_release_mapping.__getitem__.return_value = {
-            'objects': {
-                'MyObj': ['1.5'],
-            }
-        }
-        obj = MyObj(self.context)
-        dbobj = {'created_at': timeutils.utcnow(),
-                 'updated_at': timeutils.utcnow(),
-                 'version': None,
-                 'foo': 123, 'bar': 'test', 'missing': '',
-                 'nested_object': 'test'}
-        # Mock obj_load_attr as this is what is called if an attribute that we
-        # try to access is not yet set. For all ironic objects it's a noop,
-        # we've implemented it in MyObj purely for testing
-        with mock.patch.object(obj, 'obj_load_attr', autospec=True):
-            MyObj._from_db_object(self.context, obj, dbobj,
-                                  fields=['foo', 'bar'])
-            self.assertEqual(obj.__class__.VERSION, obj.VERSION)
-            self.assertEqual(123, obj.foo)
-            self.assertEqual('test', obj.bar)
-            self.assertRaises(AttributeError, getattr, obj, 'missing')
-            self.assertEqual('testtest', obj.nested_object)
-
-    @mock.patch('ironic.common.release_mappings.RELEASE_MAPPING',
-                autospec=True)
-    def test__from_db_object_map_version_bad(self, mock_release_mapping):
-        mock_release_mapping.__getitem__.return_value = {
-            'objects': {
-                'MyObj': ['1.6'],
-            }
-        }
-        obj = MyObj(self.context)
-        dbobj = {'created_at': timeutils.utcnow(),
-                 'updated_at': timeutils.utcnow(),
-                 'version': None,
+                 'version': '1.99',
                  'foo': 123, 'bar': 'test', 'missing': ''}
         self.assertRaises(object_exception.IncompatibleObjectVersion,
                           MyObj._from_db_object, self.context, obj, dbobj)
@@ -718,29 +664,29 @@ class TestObject(_LocalTest, _TestObject):
 # version bump. It is an MD5 hash of the object fields and remotable methods.
 # The fingerprint values should only be changed if there is a version bump.
 expected_object_fingerprints = {
-    'Node': '1.23-6bebf8dbcd2ce15407c946bd091f80b4',
+    'Node': '1.25-3a468b3e88d0a8fe7709f822fc654e4b',
     'MyObj': '1.5-9459d30d6954bffc7a9afd347a807ca6',
     'Chassis': '1.3-d656e039fd8ae9f34efc232ab3980905',
-    'Port': '1.7-898a47921f4a1f53fcdddd4eeb179e0b',
-    'Portgroup': '1.3-71923a81a86743b313b190f5c675e258',
+    'Port': '1.8-898a47921f4a1f53fcdddd4eeb179e0b',
+    'Portgroup': '1.4-71923a81a86743b313b190f5c675e258',
     'Conductor': '1.2-5091f249719d4a465062a1b3dc7f860d',
     'EventType': '1.1-aa2ba1afd38553e3880c267404e8d370',
     'NotificationPublisher': '1.0-51a09397d6c0687771fb5be9a999605d',
-    'NodePayload': '1.6-33b165d11b05f8e1a91da5b1bd344dda',
+    'NodePayload': '1.8-2c327dfc77ce8c7a28b02eafaed509f0',
     'NodeSetPowerStateNotification': '1.0-59acc533c11d306f149846f922739c15',
-    'NodeSetPowerStatePayload': '1.6-b32af2903cba9a61be83f5b119188e4d',
+    'NodeSetPowerStatePayload': '1.8-916c39939609254de1f8050a1c8da7d1',
     'NodeCorrectedPowerStateNotification':
         '1.0-59acc533c11d306f149846f922739c15',
-    'NodeCorrectedPowerStatePayload': '1.6-e44d9d913afba61281190082605dd1fb',
+    'NodeCorrectedPowerStatePayload': '1.8-1afef2b5c0538941defa00232302cd89',
     'NodeSetProvisionStateNotification':
         '1.0-59acc533c11d306f149846f922739c15',
-    'NodeSetProvisionStatePayload': '1.6-5e608497664b122ae53240f35072d731',
+    'NodeSetProvisionStatePayload': '1.8-d4aa7eb5a449e9f8271b3127a2d2b6ea',
     'VolumeConnector': '1.0-3e0252c0ab6e6b9d158d09238a577d97',
     'VolumeTarget': '1.0-0b10d663d8dae675900b2c7548f76f5e',
     'ChassisCRUDNotification': '1.0-59acc533c11d306f149846f922739c15',
     'ChassisCRUDPayload': '1.0-dce63895d8186279a7dd577cffccb202',
     'NodeCRUDNotification': '1.0-59acc533c11d306f149846f922739c15',
-    'NodeCRUDPayload': '1.4-e55b77489d3ff71d561d0ae03668e298',
+    'NodeCRUDPayload': '1.6-3ab7db0fc4574999fca91a1f04b4998f',
     'PortCRUDNotification': '1.0-59acc533c11d306f149846f922739c15',
     'PortCRUDPayload': '1.2-233d259df442eb15cc584fae1fe81504',
     'NodeMaintenanceNotification': '1.0-59acc533c11d306f149846f922739c15',
@@ -753,6 +699,8 @@ expected_object_fingerprints = {
     'VolumeTargetCRUDPayload': '1.0-30dcc4735512c104a3a36a2ae1e2aeb2',
     'Trait': '1.0-3f26cb70c8a10a3807d64c219453e347',
     'TraitList': '1.0-33a2e1bb91ad4082f9f63429b77c1244',
+    'BIOSSetting': '1.0-fd4a791dc2139a7cc21cefbbaedfd9e7',
+    'BIOSSettingList': '1.0-33a2e1bb91ad4082f9f63429b77c1244',
 }
 
 
